//----- SparkDetectorAppl.c ----- 12/14/09 -----(05/21/08)----

// Sections that typically require application-specific code are marked with
// the comment:  //APPLCODE

//-----------------------------------------------------------------------------
//  Application specific code for the main application task that is run at
//  a fixed interval and application specific commands run on demand.
//
//APPLCODE
//  Application code to monitor and count spark events
//-----------------------------------------------------------------------------
//
//  2009-12-14 - Mark Davis
//               - Changed order of param files to deal with problems caused
//                 by the order in which they were restored at startup.
//
//  2009-03-23 - Mark Davis -
//                - Added digInDescr[] and digOutDescr[] to support the newer
//                  generic methof for logging changes of interest in the state
//                  of the digital I/O signals.
//
//  2008-10-21 - Mark Davis -
//               - added logging of state changes and external commands
//               - added spark detected state for statusBitsAppls
//
//  2008-07-01 - Mark Davis -
//               - resumed work on code
//
//  2008-05-21 - Mark Davis -
//               - first work on SparkDetector code
//
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

#include <common.h>

#include <unistd.h>
#include <sys/types.h>


//APPLCODE - change to match the name of this file
#ifndef  ApplicSource
  #define  ApplicSource  "SparkDetectorAppl.c"
#endif

#ifndef DidRegDefs
  #define  DidRegDefs
  #include <modbusRegs.c>
#endif
#ifndef RegDefs


//APPLCODE - change to the application name
const char far applName[] = { "SparkDetector" };
const char ApplAbbrev[] = { "Spark" };


//APPLCODE - add ID to unistd.h and place it here
#define DeviceType  DT_SparkDetector

//APPLCODE - set the # of input and outputs used
#define  NUM_APPL_AIN      1   // analog inputs
#define  NUM_APPL_AOUT     1   // analog outputs
#define  NUM_APPL_DIN      8   // digital inputs
#define  NUM_APPL_DOUT     8   // digital outputs

#define  ADC_MASK_APPL  0x00000000L
#define  DAC_MASK_APPL  0x00000000L

//APPLCODE - set to state we want outputs in during initialization
#define InitialDigOutState  ( 0x0000000F )


#define SharedINC
#include <sharedAppl.c>



//=============================================================================
//  definitions related to use of the Digital and Analog I/O signals
//=============================================================================

//APPLCODE - define macros for each Digital bit and each Analog index

//=============== digitalIn bit usage ===============
#define  SPARK_CH1       0x00000001  // Spark Detected on CH1
#define  SPARK_CH2       0x00000002  // Spark Detected on CH2
#define  SPARK_CH3       0x00000004  // Spark Detected on CH3
#define  SPARK_CH4       0x00000008  // Spark Detected on CH4

#define  CH1_NO_CON      0x00000010  // No connection on CH1
#define  CH2_NO_CON      0x00000020  // No connection on CH2
#define  CH3_NO_CON      0x00000040  // No connection on CH3
#define  CH4_NO_CON      0x00000080  // No connection on CH4

const uchar  LogicOrderDIn[NUM_APPL_DIN] =  {
   0, 1, 2, 3, 4, 5, 6, 7 };

/*EXAMPLE:
#define  ClosedLoop(din)        ( (bool)((din) & CLOSED_LOOP) )
#define  FinalBtnPushed(din)    ( (bool)((din) & FINAL_BTN) )
#define  InitialBtnPushed(din)  ( (bool)((din) & INITIAL_BTN) )
#define  RFisOff(din)           ( (bool)((din) & RF_IS_OFF) )
#define  LocalMode(din)         ( (bool)((din) & LOC_REM_SW) )
*/

//--- msgs to log automatically when a digital input changes state ---
//--- use NULL for bits that we don't want to log changes for ---
const char far *digInDescr[NUM_APPL_DIN*2] = {
//--0 state descr--    --1 state descr--
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL
};


//=============== digitalOut bit usage ===============
#define  ENABLE_CH1      0x00000001   // Enable CH1
#define  ENABLE_CH2      0x00000002   // Enable CH2
#define  ENABLE_CH3      0x00000004   // Enable CH3
#define  ENABLE_CH4      0x00000008   // Enable CH4

#define  RESET_FAULT     0x00000010   // Reset Fault
#define  ActivityLED     0x00000020

const uchar  LogicOrderDOut[NUM_APPL_DOUT] =  {
   0, 1, 2, 3, 4, 5, 6, 7 };

#define  TurnOnActivityLED()    ClearBits(Next.digitalOut, ActivityLED)
#define  TurnOffActivityLED()   SetBits(Next.digitalOut, ActivityLED)

//--- msgs to log automatically when a digital output changes state ---
//--- use NULL for bits that we don't want to log changes for ---
const char far *digOutDescr[NUM_APPL_DOUT*2] = {
//--0 state descr--    --1 state descr--
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL,
  NULL,                NULL
};


//=============== Analog inputs ===============
//--- indexes for anaIn[] and readVal[] ---
/*EXAMPLE:
#define  RF_CAV_VOLT      0  // RF cavity voltage
#define  LIMITER_VOLT     1  // Setpoint Limiter voltage
#define  HDW_CFG          2  // Fixed volt indicating HD1 component set
*/

const uchar  LogicOrderAIn[NUM_APPL_AIN] = { 0 };


//=============== Anlog outputs ===============
//--- indexes for anaOut[] and setVal[] ---
/*EXAMPLE:
#define  SETPOINT         0  // Setpoint for Cav Voltage Regulator circuit
#define  FP_METER         1  // FrontPanel Meter if INITIAL or SET pressed
*/

const uchar  LogicOrderAOut[NUM_APPL_AOUT] = { 0 };


//=============================================================================
//  List of daemon processes.  The inetd process listens for connection
//  requests on each port in the list and creates a new instance of the
//  specified process to service each new connection.
//=============================================================================
#include <services.h>

//APPLCODE - add tasks that service network clients

//--- service funcs declared the same as standard command funcs:
//     e.g.:   void _task fastdatad(int argc, char **argv);

void _task tsh(int argc, char **argv);


ServiceInfo inetdConf[] = {
//--- add any application-specific inetd-type network services here ---
//--- (add port # names to services.h) ---
//  NAME          PORT           PROTOCOL  FUNC       PRI  -1  Config Params
//  ------------  -------------  --------  ---------- ---  --  ----------------

//--- standard ones defined in sharedAppl.c ---
//{ "console",    TTY0,          ASYNC,    tsh,        10, -1, &consCfg },
//{ "fastdatad",  FASTDATA_PORT, ProtoTCP, fastdatad,   5, -1, NULL },
//{ "modbusTCPd", MODBUS_PORT,   ProtoTCP, modbusTCPd,  6, -1, NULL },
//{ "tsh",        TELNET_PORT,   ProtoTCP, tsh,        10, -1, NULL }

  #define SharedServices
  #include <sharedAppl.c>
};

const int  NumServices = sizeof(inetdConf) / sizeof(ServiceInfo);



//=============================================================================
//  Application-specific data structures and definitions
//=============================================================================

// values that can be different between prev/cur or cur/next during processing


// ***** WARNING ***** WARNING ***** WARNING ***** WARNING *****
//  Changing the applState structure means that a soft reset will fail!  A
//  change to the size will cause a soft restart to abort and do a CPU
//  restart instead.  And while a change that does NOT result in a new
//  size will not cause the soft restart to abort, it does mean that any
//  shift in the position of values can cause all kinds of mayhem.
// ***** WARNING ***** WARNING ***** WARNING ***** WARNING *****


typedef struct {
//--- values updated in "cur" instance at start of each pass ---
//  ulong  processTime;     // time (in ms) when processing was started
  ulong  digitalIn;       // state of digital inputs

  bool   enabled1;  // state of enable/disable commands for inputs
  bool   enabled2;
  bool   enabled3;
  bool   enabled4;

  bool   con1;     // connected/disconnected state of inputs
  bool   con2;
  bool   con3;
  bool   con4;

  bool   spark1;   // digital inputs indicate a spark
  bool   spark2;
  bool   spark3;
  bool   spark4;

  bool   spark1Use;  // sparks on enabled & connected inputs
  bool   spark2Use;
  bool   spark3Use;
  bool   spark4Use;

  bool   fault;

  bool   resetFault;

//--- values updated in "next" instance at start of each pass ---
  ulong  digitalOut;      // state for digital outputs
} ApplState;


ApplState  prevState, curState, nextState;

#define  Prev  prevState
#define  Cur   curState
#define  Next  nextState

const uint ApplStateDataSize =  ( sizeof(ApplState)*3 + sizeof(setVal) );



//=============================================================================
//  Defintions for variables that are accessable as ModBus registers
//=============================================================================
#else  // end of before-RegDefs portion

/*
  The purpose of this section is to define a set of C variables that are
  accessable as ModBus registers.  This approach avoids the need to use
  special functions or macros throughout the code when accessing register
  values, which keeps the application code fast and simple.

  The model used provides support for multiple logical devices.  The 2nd
  parameter in each register definition macro indicates if one instanace
  applies to all logical devices (specified by an "x" for "don't care"), or
  if each logical device requires its own instance (specified by an under-
  score "_" to signify "fill in the blank").  In the 1st case, the variable
  is defined as a scalar.  In the 2nd case, it is defined as an array.

  The definition macros support data types (floats and longs in particular)
  that require more than one register to store them.  To avoid the possibility
  of corrupted values, all reads of and writes to such values must be done
  using the multi-register commands and including all the registers that
  make up the value in the command.  Differences in register order between
  different client types is handled by a table that automatically swaps
  register values as needed based on the client type.

  The definition for each writable register also includes the function to
  call when the value is set via the ModBus interface.  Many registers can
  use the default setXXX() functions, but you can specify your own to provide
  any special logic needed (including ignoring the new setting if invalid).

  For registers, the format for the function definitions is:

    bool mySetFunc(SetRegFuncParams);

       where SetRegFuncParams is currently defined as:
              void *info, uchar chanNum, void *newVal, fd_t eventsfd

  For coils, the format for the function definitions is:

     bool (*SetCoilFunc)(SetCoilFuncParams);

         where SetCoilFuncParams is currently define as:
               void *info, uchar chanNum, uchar newVal, fd_t eventsfd

  The parameters to the call are as follows:
    *info:     A pointer to the Coil's/Register's CoilInfo/RegInfo structure
    chanNum:   The logical device # (0 is the 1st)
    *newVal:   A pointer to the new value requested in the Modbus packet
               (changed to native-byte order where needed)
    eventsfd:  The fd for the FIFO used to send msgs to the main application
               process

  The function returns true if the new value was accepted, false if not.

  The register numbers for multiple "channels" or logical devices are
  handled automatically based on the xxChanMult values.  These values specify
  the offset between a register value for one logical device and the "same"
  register for the Next.

  Example:

    Assume the following definitions:
      #define  NumLogicalDevices  2
      #define  R4ChanMult  100
      REG4( motorPos,       _, 40050, F32, RW, setMotorPos )

    Then we would get:
      global variable definition:  float motorPos[2];
      register pair:  40050,40051  refers to:  motorPos[0]
      register pair:  40150.40151  refers to:  motorPos[1]

    If instead we had specified:
      REG4( motorPos,       x, 40050, F32, RW, setMotorPos )

    Then we would get:
      global variable definition:  float motorPos;
      register pairs:  40050,40051 & 40150,40151  BOTH refer to:  motorPos


  To keep the definitions as simple as possible, and avoid requiring multiple
  macros for each register definition, each macro is used more than once,
  with a different definition (#define) each time (a practice generally
  referred to as X-Macros).  While trying to follow the order of inclusion
  of files (including a recursive reference back to this file) can be rather
  confusing, the X-Macro method keeps the source code simple to read and
  maintain by doing everything with a single macro for each register.

  ==========================================================================
                             !!! WARNING !!!
  ==========================================================================
    Each sequence of register definitions MUST be contiguous!  There must
    NOT be any holes (i.e. no unused register #'s between used ones) and no
    overlap!
-----------------------------------------------------------------------------*/


//APPLCODE - set # of devices for which we want duplicate register sets
#ifndef NumLogicalDevices
  #define  NumLogicalDevices  1
#endif

//--------------------------------------------------------------
//APPLCODE - set max # of registers per set of coils
#ifndef NumC0Chans
  #define NumC0Chans_  NumLogicalDevices
  #define C0ChanMult_  100
#endif

#ifdef COIL0

COIL0( DoReboot,       x, 1, setCoil )
//COIL0( Coil00002,      _, 2, setCoil )
//COIL0( Coil00003,      _, 3, setCoil )

#undef COIL0
#endif


//--------------------------------------------------------------
//APPLCODE - set max # of registers per set of read-only registers
#ifndef NumR3Chans
  #define NumR3Chans_  NumLogicalDevices
  #define R3ChanMult_  100
#endif

#ifdef REG3

REG3( dlmVersTime,    x, 30001, U32 )
REG3( rtosVersTime,   x, 30003, U32 )
REG3( applVersTime,   x, 30005, U32 )
REG3( devType_,       x, 30007, U16 )
REG3( deviceID_,      x, 30008, U32 )
REG3( upSecs,         x, 30010, U32 )
REG3( percentIdle_,   x, 30012, U16 )

REG3( numChans,       x, 30013, U16 )
REG3( coilChanMult,   x, 30014, U16 )
REG3( reg3ChanMult,   x, 30015, U16 )
REG3( reg4ChanMult,   x, 30016, U16 )

REG3( digInBits,      x, 30017, U32 )

REG3( statusBitsStdA, x, 30019, U16 )
REG3( statusBitsStdB, x, 30020, U16 )
REG3( statusBitsAppl, x, 30021, U32 )

//APPLCODE - add registers for externally available read-only values
REG3( latency,        x, 30023, U16 )  // actual vs scheduled appl start in ms
REG3( tcpTickTime,    x, 30024, U16 )  // # ms inside tcp_tick() call
REG3( applLoopTime,   x, 30025, U16 )  // # ms used by applic task loop
REG3( kernelLoopTime, x, 30026, U16 )  // # ms used by last kernel pass


#undef REG3
#endif


#ifdef RegMacros
//===== application-specific defintions related to register values =====

  #define  READING(idx)  REG_ARRAY(float, rdg_1, idx)

#endif

//----- values for statusBitsAppl -----
#define  SBA_ENABLED_CH1     0x00000001
#define  SBA_ENABLED_CH2     0x00000002
#define  SBA_ENABLED_CH3     0x00000004
#define  SBA_ENABLED_CH4     0x00000008
#define  SBA_ENABLED_BITS    0x0000000F

#define  SBA_CH1_CON         0x00000010  // No connection on CH1
#define  SBA_CH2_CON         0x00000020  // No connection on CH2
#define  SBA_CH3_CON         0x00000040  // No connection on CH3
#define  SBA_CH4_CON         0x00000080  // No connection on CH4
#define  SBA_CON_BITS        0x000000F0  // connection bits for all 4 inputs

#define  SBA_FAULT           0x00000100

#define  SBA_CH1_SPARK       0x00001000  // CH1 sparking
#define  SBA_CH2_SPARK       0x00002000  // CH2 sparking
#define  SBA_CH3_SPARK       0x00004000  // CH3 sparking
#define  SBA_CH4_SPARK       0x00008000  // CH4 sparking


//--------------------------------------------------------------
//APPLCODE - set max # of registers per set of read/write registers
#ifndef NumR4Chans
  #define NumR4Chans_  NumLogicalDevices
  #define R4ChanMult_  100
#endif

/*
  The "standard" functions for processing writes to Modbus register values:

    setS16, setU16, setS32, setU32, setF32

  These functions simply log a message for the change and update the
  value in a critical block (so the change cannot be interrupted).

  In many cases, just allowing any setting to be written to a register (or
  multiple registers comprising a larger value) is not acceptable.


  If any of the following apply, then a custom func must be used instead:

    - The order of changes is important.  For example, if you get ON,
      OFF, and RESET commands in rapid succession, you have to be sure
      they are processed in the order they were recieved.

    - Not all possible settings are acceptable.  The acceptable settings
      for a value are often a subset of the possible ones, and frequently
      are dependent upon the current state of the controller.


  IMPORTANT NOTES:

    The "applic" task periodically executes the code that makes up the
    bulk of the application logic.  Because it runs at the highest priority
    there is no chance that a lower priority task (like the daemon handling
    Modbus register writes) can change a value in the middle of this
    processing.

    However, the applic task CAN (and often does) preempt other processes,
    so any process that does read or make changes to the register values
    must lock the register values before doing so.  This is currently done
    by disabling the scheduler using the KLOCK() and KUNLOCK() macros.

    Use of such a lock insures that changes occur only between passes
    through the application logic, but there are still several issues
    that must be addressed when writing functions to handle writes to
    register values:

      - Priority inversion and latency:  Allowing a lower priority task
        to lock out higher ones (or, in this case, ALL tasks) allows
        the lower priority task to run when a higher-priority one should
        be running.  Care must be taken to keep such periods as short
        as possible to avoid disrupting the timing of the control logic.

      - Loss of encapsulation:  The last thing we want is to break the
        application logic in to small pieces spread through a bunch of
        functions executed by different tasks at different times.

        While the logic for processing writes to register values is often
        dependent upon the state of the control logic, functions outside
        the applic task must NOT directly alter the state of the application
        logic.  To achieve this, such functions must limit changes to
        values that act as input to the applic task.

        To this end, you need to treat the register values that act as
        inputs just like they were knobs, switches, analog inputs, or
        latches.  Like their hardware equivalent, the applic task logic
        reads their state at the beginning of each pass and uses this
        information when determining what the next state of the control
        logic should be.  And just like a hardware latch, values can be
        reset after reading them so the next write to the value can be
        treated as another instance of a request.

    The trickest circumstances to handle are the ones the order of separate
    writes is important.  The most common example are commands for ON, OFF,
    and RESET.  If, for example, you had 3 separate flags, and simply set
    each once corresponding to which of the 3 commands you received, you
    end up with multiple flags set when the applic logic is processed, and
    no way to determine what the end result should be.  In these cases,
    the state of each of the flags is dependent upon the occurance of
    multiple commands.

    Ideally, the logic for setting each variable would be triggered each
    time an event occurs that could affect it:  A publish-subscribe type
    of mechanism.  One possibility to do this efficiently?  Use a bit-mask
    "array" to determine which functions to call for each register that is
    changed.  That we it doesn't take many bytes to determine which functions
    to call, and each function can update its corresponding register value
    based on its current state and the event that just occured.

    Or should I simply combine such values in to one (a struct, if necessary),
    and have a single function that is called for all the related values
    and updates the state of the single logical entity based on its
    current state and what event called it?

*/

#ifdef REG4

REG4( timeZone_,      x, 40001, S16, RW, MB_timeZone )
REG4( curTime,        x, 40002, U32, RW, setU32 )

REG4( ipAddr,         x, 40004, U32, RW, setU32 )
REG4( netmask,        x, 40006, U32, RW, setU32 )
REG4( gateway,        x, 40008, U32, RW, setU32 )

REG4( setACoil,       _, 40010, U16, RW, setU16 )
REG4( resetACoil,     _, 40011, U16, RW, setU16 )

REG4( digOutBits,     x, 40012, U32, RW, setU32 )

REG4( cmdBitsStd,     x, 40014, U32, RW, MB_cmdBitsStd )
REG4( cmdBitsAppl,    x, 40016, U32, RW, MB_cmdBitsAppl )

//APPLCODE - add registers for externally available read/write values
REG4( count_ch1,      x, 40018, U32, RW, setU32 )
REG4( count_ch2,      x, 40020, U32, RW, setU32 )
REG4( count_ch3,      x, 40022, U32, RW, setU32 )
REG4( count_ch4,      x, 40024, U32, RW, setU32 )

#undef REG4
#endif


#ifdef RegMacros
//===== application-specific defintions related to register values =====

//  #define  ISET(idx)   REG_ARRAY(float, iSet_1, idx)

#endif

//--------------------------------------------------------------
#undef RegMacros

#endif
#ifndef RegDefs // start of after-RegDefs portion


//----- values for cmdBitsAppl -----
#define  CBA_ENABLE_CH1     0x00000001
#define  CBA_ENABLE_CH2     0x00000002
#define  CBA_ENABLE_CH3     0x00000004
#define  CBA_ENABLE_CH4     0x00000008

#define  CBA_DISABLE_CH1    0x00000010
#define  CBA_DISABLE_CH2    0x00000020
#define  CBA_DISABLE_CH3    0x00000040
#define  CBA_DISABLE_CH4    0x00000080

#define  CBA_CLEAR_CH1      0x00000100
#define  CBA_CLEAR_CH2      0x00000200
#define  CBA_CLEAR_CH3      0x00000400
#define  CBA_CLEAR_CH4      0x00000800

#define  CBA_RESET_FAULT    0x00001000


const ulong cmdEnableBits[4] =  { CBA_ENABLE_CH1,  CBA_ENABLE_CH2,
                                   CBA_ENABLE_CH3,  CBA_ENABLE_CH4  };
const ulong cmdDisableBits[4] = { CBA_DISABLE_CH1, CBA_DISABLE_CH2,
                                   CBA_DISABLE_CH3, CBA_DISABLE_CH4 };
const ulong cmdClearBits[4] =   { CBA_CLEAR_CH1, CBA_CLEAR_CH2,
                                   CBA_CLEAR_CH3, CBA_CLEAR_CH4 };
const ulong cmdEnableAllBits =
  CBA_ENABLE_CH1 | CBA_ENABLE_CH2 | CBA_ENABLE_CH3 | CBA_ENABLE_CH4;

const ulong cmdDisableAllBits =
  CBA_DISABLE_CH1 | CBA_DISABLE_CH2 | CBA_DISABLE_CH3 | CBA_DISABLE_CH4;

const ulong cmdClearAllBits =
  CBA_CLEAR_CH1 | CBA_CLEAR_CH2 | CBA_CLEAR_CH3 | CBA_CLEAR_CH4;


//=============================================================================
//  defintions for "parameterized" data values - lists of values that we want
//  to be "persistent" (i.e. saved to a file and restored on startup)
//=============================================================================
//APPLCODE - settings/values we want saved to/restored from files


//--- parameter ID's for data in Device Data/calibration file ---
//--------------------------------------------------------------------------
// WARNING:  ID's can be added, but existing ID's (and the format of the
//           data they represent) should NEVER change!  NOTE:  Existing ones
//           can be "eliminated" by changing their name and no longer
//           using them, but never REUSE an old one for something new!!!
//--------------------------------------------------------------------------
//  NOTE:  ID's >= 1000 are rsvd for values shared by most applications
//--------------------------------------------------------------------------

const ParamData far  devDataParams[] = {
//APPLDATA
//                ID   ---value---  --RESTORE_FUNC--
//PERSISTENT_VAR(  1,  myVarName,     NULL ),
//                ID   ---value---  granularity  --RESTORE_FUNC--
//PERSISTENT_TBL(  2   myArrayName, elemSize,     NULL ),

  #define SharedDevParams
  #include <sharedAppl.c>

  END_PARAM_DATA
};

//----------------------------------------------------
const ParamData far  settingsParams[] = {
//APPLDATA
//                ID   ---value---  --RESTORE_FUNC--
//  PERSISTENT_VAR(  1,  rfOffSetpoint,   NULL ),
//  PERSISTENT_VAR(  2,  rfOnSetpoint,    NULL ),

  END_PARAM_DATA
};


//-----------------------------------------------------------------
//  Application-specific values needed to restore the state of the
//  controller following a "soft" reset (i.e. a restart without a
//  CPU reset)
//-----------------------------------------------------------------
const ParamData far  ctlrStateParams[] = {
// sample values from sharedAppl.c, common to all applications
//                   --ID--  ---value,granularity---  --RESTORE_FUNC--
//    PERSISTENT_VARx(  1000,  prevState,               NULL ),
//    PERSISTENT_VARx(  1001,  curState,                NULL ),
//    PERSISTENT_VARx(  1002,  nextState,               NULL ),
//    PERSISTENT_TBL (  1003,  setVal,4,                NULL ),
// // PERSISTENT_TBL (  1004,  shadowRegValues,4,       NULL ),

//APPLDATA
//    PERSISTENT_VAR(  1,  myApplStateVar,        NULL ),


  #define SharedCtlrStateParams
  #include <sharedAppl.c>

  END_PARAM_DATA
};


//-----------------------------------------------------------------------------
//  A list of parameterized "persistent" data.  Backup copies of such values
//  are stored in files and restored on startup.
//-----------------------------------------------------------------------------

//    Base File Name    Category  ParamData
//    ----------------  --------  ---------
//--- add any application-specific parameterized file defs here ---
//ParamFile  myAppCfgFile =
//  { "myFileName",     PDC_CTLR, myCfgParams,   0 };

ParamFile  settingsFile =
    { "settings",       PDC_SET,  settingsParams  };


#define SharedParams
#include <sharedAppl.c>


ParamFile *paramFiles[] = {
  #define SharedParamsList
  #include <sharedAppl.c>
//--- add any application-specific ParamData references here ---
//  &myAppCfgFile,
  &settingsFile
};

const int NumParamFiles = sizeof(paramFiles) / sizeof(paramFiles[0]);


//=============================================================================
//  Definitions for event msgs.  Event msgs are the means that other processes
//  use to inform the application process of an event that it needs to handle.
//
//  In general, allowing multiple processes to concurrently modify the state
//  of the controller is a sure recipie for chaos.  By restricting all such
//  changes to the context of a single process, and sending messages to that
//  process to request changes, we eliminate all such concurrency issues (not
//  the least of which is keeping track of the order of events).
//
//  The most common event is a request for change of state that corresponds to
//  a message received from an external client via a protocol like ModBus.
//=============================================================================

//EVENT_DEF( myEventX )
//EVENT_DEF( myEventY )



//=============================================================================
//  This is where misc global variables should be defined
//=============================================================================
#define SharedGlobals
#include <sharedAppl.c>

//APPLCODE
//--- add definitions for application-specific global variables here ---



//=============================================================================
//  Functions referenced in the Modbus register definitions to handle writes to
//  specific global values defined as ModBus registers
//=============================================================================

//APPLCODE - add functions to process writes to ModBus register values

//-----------------------------------------------------------------------------
//  External ModBus client sent a new value for "cmdBitsAppl"
//
//  Check the bits that where set and trigger the corresponding events as
//  appropriate (but do NOT save the new value - just use it to trigger
//  events)
//-----------------------------------------------------------------------------
bool _nearcall MB_cmdBitsAppl(SetRegFuncParams)
{
  cmdBitsAppl = *((ulong *)newVal);
/*
  ulong  cmdBits;
  EventData  eventMsg;  // used by EVENT macro


  cmdBitsAppl = *((ulong *)newVal);

  if (cmdBits & x)  EVENT(MyEventX, chan, newVal, sizeof(int)); // w/ data
  if (cmdBits & y)  EVENT(MyEVentY, chan, NULL, 0);             // w/o data
*/

  return true;
}



//=============================================================================
//  functions to handle restoration of saved "persistent" values
//=============================================================================

/*
//-----------------------------------------------------------------------------
//  one of the setpoint values needs restoring from the settings file
//-----------------------------------------------------------------------------
RESTORE_FUNC(myVal)
{
  uint  u;
  float  fval;
  char  temp[32];

  for (u=0; u<NumPS; ++u)  if (curVal == (void far *) &ISET(u))  {
    fval = *(float far *)newVal;
    printf("restoring iSet_%u to: %s\r\n", u, fmtFloat(fval, 8,2, temp));
    newSetpoint(u, fval);
  }

}

*/

//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
#define SharedFuncs
#include <sharedAppl.c>


//=============================================================================
//  Misc Application-specific functions
//=============================================================================
//APPLCODE



//=============================================================================
//  Task functions to handle application-specific commands
//=============================================================================
//APPLCODE

//-----------------------------------------------------------------------------
//  example func to process a command entered via a user shell
//-----------------------------------------------------------------------------
/*
void _task miscTest(int argc, char **argv)
{
  int  idx;
  bool  validCmd;

//used by EVENT macro
  fd_t  eventsfd = -1;
  EventMsg  eventMsg;


  validCmd = false;

  for (;;)  {
    if (argc < 3)  break;
    if (! isdigit(argv[1][0]))  break;

    idx = atoi(argv[1]);
    if ((idx < 1) OR (idx > NumChan))  {
      printf("Invalid chan #\n");  Exit(-2); }

    --idx;  validCmd = true;

    if (StriEq(argv[2], "x"))     { EVENT(MyEventX, idx, NULL, 0);  break; }
    if (StriEq(argv[2], "y"))     { EVENT(MyEventY, idx, NULL, 0);  break; }

    validCmd = false;  break;
  }

  if (! validCmd)  { printf("Usage:  misc # x|y\r\n");  Exit(-1); }

  Exit(0);
}
*/

//=============================================================================
//  Generic structure to handle application-specific presentation of the
//  digital and analog input/output values.  Code to handle additional
//  formatting that cannot be handled in this generic manner should be
//  placed in the showState() task routine.
//=============================================================================
//APPLCODE - text and formatting info for application state summary

/*EXAMPLE:
//-----------------------------------------------------------------------------
_nearcall char *fmtMode(DispElem far *elem, char *buf)
{
  return (char *) ModeDesc[ELEM_VAL(U16)];
}

//-----------------------------------------------------------------------------
_nearcall char *fmtHdwCfg(DispElem far *elem, char *buf)
{
  return (char *) HdwCfgDesc[ELEM_VAL(U16)];
}
*/

//-----------------------------------------------------------------------------
_nearcall char *fmtStrR(DispElem far *elem, char *buf)
{
  sprintf(buf, "%*.*ls", elem->width, elem->prec, elem->val);  return buf;
}


//-----------------------------------------------------------------------------
  const char far DevStateStr[] = {
"               Counts    Connected   Enabled  Sparking\r\n"
"              ---------  ---------   -------  --------\r\n"
"         CH1: #########     ###        ###      ##### \r\n"
"         CH2: #########     ###        ###      ##### \r\n"
"         CH3: #########     ###        ###      ##### \r\n"
"         CH4: #########     ###        ###      ##### \r\n"

/*EXAMPLE:
"Cavity Voltage:  ###.## kV    Open/Closed: ######    HdwConfig: ##### ",
"Active setpoint: ###.## kV    RF On/Off:   ### ",
"Error:          -###.## kV    Loc/Rem:     ###### ",
" ",
"Limiter Voltage: -##.### V    Limited:     ### ",
"                              Regulation:  ##### ",
"RF OFF setpoint: ###.## kV ",
"RF ON setpoint:  ###.## kV    Ramp Mode:   #### ",
" ",
*/
""
};

const DispElem  far dispElems[] = {
//  { 0, 5, VFP(dinLabel[0]),    DARK_BLUE,        fmtStrR,    12,12 },
//  { 0, 6, VFP(dinLabel[1]),    DARK_BLUE,        fmtStrR,    12,12 },
//  { 0, 7, VFP(dinLabel[2]),    DARK_BLUE,        fmtStrR,    12,12 },
//  { 0, 8, VFP(dinLabel[3]),    DARK_BLUE,        fmtStrR,    12,12 },

  { 14, 5, VFPV(count_ch1),     DARK_BLUE, fmtU32,    9,0 },
  { 14, 6, VFPV(count_ch2),     DARK_BLUE, fmtU32,    9,0 },
  { 14, 7, VFPV(count_ch3),     DARK_BLUE, fmtU32,    9,0 },
  { 14, 8, VFPV(count_ch4),     DARK_BLUE, fmtU32,    9,0 },

/*EXAMPLE:
  { 17, 3, VFPV(cavityVolts),     DARK_BLUE, fmtF32,    6,2 },
  { 17, 4, VFPV(activeSetpoint),  DARK_BLUE, fmtF32,    6,2 },
  { 16, 5, VFPV(regulationError), DARK_BLUE, fmtF32,    7,2 },

  { 17, 7, VFPV(limiterVolts),    DARK_BLUE, fmtF32,    7,3 },

  { 17, 9, VFPV(rfOffSetpoint),   DARK_BLUE, fmtF32,    6,2 },
  { 17,10, VFPV(rfOnSetpoint),    DARK_BLUE, fmtF32,    6,2 },

  { 43,10, VFPV(rampMode),        DARK_BLUE, fmtMode,   4,4 },

  { 64, 3, VFPV(hdwCfg),          DARK_BLUE, fmtHdwCfg, 5,5 },
*/
  { 0,0, NULL, NULL, NULL, 0,0 }
};

//--- ATTR_NAME macros currently defined in fcntl.h ---
const DigDispElem far digInElems[] = {
  { 28, 5,  CH1_NO_CON,   "YES"  ,DARK_BLUE,  "NO "  ,DARK_RED },
  { 28, 6,  CH2_NO_CON,   "YES"  ,DARK_BLUE,  "NO "  ,DARK_RED },
  { 28, 7,  CH3_NO_CON,   "YES"  ,DARK_BLUE,  "NO "  ,DARK_RED },
  { 28, 8,  CH4_NO_CON,   "YES"  ,DARK_BLUE,  "NO "  ,DARK_RED },

  { 48, 5,  SPARK_CH1,  "---- "  ,DARK_BLUE, "SPARK"  ,DARK_RED },
  { 48, 6,  SPARK_CH2,  "---- "  ,DARK_BLUE, "SPARK"  ,DARK_RED },
  { 48, 7,  SPARK_CH3,  "---- "  ,DARK_BLUE, "SPARK"  ,DARK_RED },
  { 48, 8,  SPARK_CH4,  "---- "  ,DARK_BLUE, "SPARK"  ,DARK_RED },

/*EXAMPLE:
//{ xx,yy,  BIT_MASK,     "0STR"  ,ATTR_NAME,  "1STR"  ,ATTR_NAME },
  { 43, 3,  CLOSED_LOOP,  "OPEN  ",DARK_RED,   "CLOSED",DARK_BLUE },
  { 43, 4,  RF_IS_OFF,    "ON "   ,DARK_BLUE,  "OFF"   ,DARK_RED },
  { 43, 5,  LOC_REM_SW,   "REMOTE",DARK_BLUE,  "LOCAL ",DARK_BLUE },
*/
  { 0,0, 0, NULL,NULL, NULL,NULL }
};

const DigDispElem far digOutElems[] = {
  { 39, 5,  ENABLE_CH1,   "NO "  ,DARK_RED,  "YES "  ,DARK_BLUE },
  { 39, 6,  ENABLE_CH2,   "NO "  ,DARK_RED,  "YES "  ,DARK_BLUE },
  { 39, 7,  ENABLE_CH3,   "NO "  ,DARK_RED,  "YES "  ,DARK_BLUE },
  { 39, 8,  ENABLE_CH4,   "NO "  ,DARK_RED,  "YES "  ,DARK_BLUE },

/*EXAMPLE:
//{ xx,yy,  BIT_MASK,     "0STR"  ,ATTR_NAME,  "1STR" ,ATTR_NAME },
  { 43, 7,  LIMITED_LED,   "NO "  ,DARK_BLUE,  "YES"  ,DARK_RED },
  { 43, 8,  ERROR_LED,     "OK   ",DARK_BLUE,  "ERROR",DARK_RED },
*/
  { 0,0,  0,  NULL,NULL,  NULL,NULL }
};

//---------------------------------------------------------------
//---- older style - do not use for newer applications -----
const DigFmtInfo digInFmt[] = {
//{ xx,yy, "0-str"  ,1,  "1-str" ,0 },  //  descr
  { -1,-1, "",0, "",0 }
};

const DigFmtInfo digOutFmt[] = {
//{ xx,yy, "0-str"  ,1,  "1-str" ,0 },  //  descr
  { -1,-1, "",0, "",0 }
};


const AnalogFmtInfo  adcFmt[] = {
  { -1,-1, 0,0 }
};

const AnalogFmtInfo  dacFmt[] = {
  { -1,-1, 0,0 }
};


//-----------------------------------------------------------------------------
//  Application-specific commands
//-----------------------------------------------------------------------------
void _task cmd(int argc, char **argv)
{
  bool  validCmd, allChans, validChan;
  int  chan;
  char  *cmdStr;


  validCmd = allChans = validChan = false;

#pragma warn -775

  do  {
    if (argc < 2)  break;

    cmdStr = argv[1];

    if (argc > 2)  {
      if (StriEq(argv[2], "all"))
        allChans = validChan = true;
      else  {
        if (isdigit(*argv[2]))  chan = atoi(argv[2]) - 1;
        if ((chan >= 0) AND (chan < 4))  validChan = true;
      }
    }

    if (StriEq(cmdStr, "enable"))  {
      if (!validChan)  { printf("Invalid channel\r\n");  break; }
      KLOCK();
      if (allChans)
        cmdBitsAppl |= cmdEnableAllBits;
      else
        cmdBitsAppl |= cmdEnableBits[chan];
      KUNLOCK();  validCmd = true;
    }

    if (StriEq(cmdStr, "disable"))  {
      if (!validChan)  { printf("Invalid channel\r\n");  break; }
      KLOCK();
      if (allChans)
        cmdBitsAppl |= cmdDisableAllBits;
      else
        cmdBitsAppl |= cmdDisableBits[chan];
      KUNLOCK();  validCmd = true;
    }

    if (StriEq(cmdStr, "clear"))  {
      if (!validChan)  { printf("Invalid channel\r\n");  break; }
      KLOCK();
      if (allChans)
        cmdBitsAppl |= cmdClearAllBits;
      else
        cmdBitsAppl |= cmdClearBits[chan];
      KUNLOCK();  validCmd = true;
    }

    if (StriEq(cmdStr, "reset"))  {
      KLOCK();  cmdBitsAppl |= CBA_RESET_FAULT;  KUNLOCK();  validCmd = true; }

  }  while(0);

#pragma warn 775

  if (! validCmd)  {
    printf("\r\n"
           "Usage:  cmd enable #|all   (enable one or all channels)\r\n"
           "            disable #|all  (disable one or all channels)\r\n"
           "            clear #|all    (clear count for one or all channels)\r\n"
           "            reset          (reset current spark fault)\r\n" );
    Exit(-1);
  }

  Exit(0);
}


//-----------------------------------------------------------------------------
//  "state":  Show continuously updated state of the system
//-----------------------------------------------------------------------------
void _task showState(int argc, char **argv)
{
  bool  blinkOff;
  char  temp[128];


  printf(clsCmd);  blinkOff = false;

  CursorTo(0, 3);  printf(DevStateStr);

  for (;;)  {
    CursorTo(0, 0);  blinkOff = !blinkOff;

    stdHeader();  displayElems((DispElem far *)dispElems, blinkOff, temp);

    displayDigElems(digitalIn, (DigDispElem far *)digInElems, blinkOff, temp);
    displayDigElems(digitalOut, (DigDispElem far *)digOutElems, blinkOff, temp);

    CursorTo(0,22);  fflush(stdout);  CheckETX();  sleepMS(500);
  }

}


//=============================================================================
//  Event-Handler functions called by the appl task to process event messages
//=============================================================================
//APPLCODE

//-----------------------------------------------------------------------------
//  Event-Handler for a request to ...
//-----------------------------------------------------------------------------
/*
void EVENT_MyEvent(EventMsg *event)
{
  uint  idx;
  float  setpoint;
  char  temp[16];


  idx = event->chanNum;

  if (error)  {
    fprintf(DiagLog, "error msg, chan:\n", idx+1);
    return;
  }

  setpoint = *((float *)event->data);

  fprintf(DiagLog, "New SetPoint for PS%d: %s\n",
     idx+1, fmtFloat(setpoint, 8,3, temp) );

  *((float *)((&iSet_1_RegInfo + idx)->regVal)) = setpoint;

  anaOut[idx] = ISETtoVolts(idx, setpoint);
}
*/


//=============================================================================
//  Copy the "cur" state from last time to "prev", and update "cur"
//=============================================================================
//  Just before this function is called, readings were taken of the current
//  state of the hardware, and any changes to the output states specified by
//  the last pass were applied (just after taking the readings).
//
//  This func first copies "cur" to "prev" and then "next" to "cur" which does
//  2 things for us:
//
//    - We have a copy of the controller's logical state from the last pass
//      (so we can determine what has changed)
//
//    - We have a copy of values set by other tasks indicating events/requests
//      that happend/were made since the beginning of the last pass
//
//  We then reset the values (as needed) that are set by other tasks to
//  indicate events and requests, and update the current state based on the
//  readings just taken and the requests and events communicated by other
//  tasks.
//
//  All of this is in preperation for the updateNextState() function, which
//  implements the main "decision making" portion of the application logic,
//  determining what the next state should be based on the latest conditions
//  (as determined by the updateCurState() function).
//
//=============================================================================
void updateCurState(void)
{

  if (current != applTask)  return;  // must only be called by "applic" task

//-----------------------------------------------------------------
// The following conditions exist at this point:
// - The inputs have just been read
// - The outputs were just set to values computed during the last pass
// - "cur" contains the state the inputs were in at the START of
//   of the previous pass, and the state the outputs were in DURING
//   the previous pass
// - "next" contains the values that the outputs were just set
//   to and any "input" values that were changed by other tasks
//------------------------------------------------------------------

// Now we update the values in "Next" that are based solely on the
// state of the hardware inputs we just read and values maintained by
// by other tasks (basically, we deal with READINGS here: Values that
// we need but are not written to "Next" by another task)

//  Next.processTime = GetMS();  curTime = time(NULL);

//  msSincePrev = msSince(Cur.processTime);

  Next.digitalIn = digitalIn;


// Note that a copy is done (rather than just switching pointer values) to
// insure all values are set in both the source and destination.  If we
// simply kept pointers and swapped the pointer values, then EVERY value
// would have to be updated independently by the code, which is a pain to
// program, adds overhead and code, and is a VERY easy way to introduce odd
// bugs.  Overall, copying everything here is actually faster overall,
// simpler, and much safer.

  Prev = Cur;  Cur = Next;

//--- At this point the Cur copy contains what the inputs were at the start ---
//--- of this pass, and the Prev copy contains the output values from the ---
//--- the previous pass (i.e. the output values that resulted in the input ---
//--- values in Cur) ---


//--------------------------------------------------------------------
// Next we update the rest of the "cur" values needed by the control
// logic (the ones that do NOT come from "next" values set by other
// tasks)
//
// NOTES:
//  - This portion of the logic has order-of-execution dependencies,
//    so be sure that all values referenced in an expression are ones
//    set before they are used.
//
//  - References to "prev" values are always safe since they are just
//    a copy of values from the last pass
//--------------------------------------------------------------------

//--- log msgs for bit chgs only if not starving other processes for cycles ---
  if (applLoopTime < 8)  logDigIOChgs(false);


//APPLCODE
  Cur.enabled1 = BOOL(Cur.digitalOut & ENABLE_CH1);
  Cur.enabled2 = BOOL(Cur.digitalOut & ENABLE_CH2);
  Cur.enabled3 = BOOL(Cur.digitalOut & ENABLE_CH3);
  Cur.enabled4 = BOOL(Cur.digitalOut & ENABLE_CH4);

  Cur.con1 = (!(Cur.digitalIn & CH1_NO_CON));
  Cur.con2 = (!(Cur.digitalIn & CH2_NO_CON));
  Cur.con3 = (!(Cur.digitalIn & CH3_NO_CON));
  Cur.con4 = (!(Cur.digitalIn & CH4_NO_CON));

  Cur.spark1 = ((Cur.digitalIn & SPARK_CH1) AND Cur.con1);
  Cur.spark2 = ((Cur.digitalIn & SPARK_CH2) AND Cur.con2);
  Cur.spark3 = ((Cur.digitalIn & SPARK_CH3) AND Cur.con3);
  Cur.spark4 = ((Cur.digitalIn & SPARK_CH4) AND Cur.con4);

  Cur.spark1Use = (Cur.spark1 AND Cur.enabled1);
  Cur.spark2Use = (Cur.spark2 AND Cur.enabled2);
  Cur.spark3Use = (Cur.spark3 AND Cur.enabled3);
  Cur.spark4Use = (Cur.spark4 AND Cur.enabled4);

  if (Cur.spark1Use AND !Prev.spark1)  ++count_ch1;
  if (Cur.spark2Use AND !Prev.spark2)  ++count_ch2;
  if (Cur.spark3Use AND !Prev.spark3)  ++count_ch3;
  if (Cur.spark4Use AND !Prev.spark4)  ++count_ch4;

  Cur.fault = Cur.spark1Use OR Cur.spark2Use OR Cur.spark3Use OR Cur.spark4Use;

  Cur.resetFault = BOOL(Cur.digitalOut & RESET_FAULT);


//--- log msgs about changes in the state ---
  if (Cur.enabled1 != Prev.enabled1)
    printf("Channel 1 %s\r\n",  Cur.enabled1 ? "ENABLED" : "DISABLED");
  if (Cur.enabled2 != Prev.enabled2)
    printf("Channel 2 %s\r\n",  Cur.enabled2 ? "ENABLED" : "DISABLED");
  if (Cur.enabled3 != Prev.enabled3)
    printf("Channel 3 %s\r\n",  Cur.enabled3 ? "ENABLED" : "DISABLED");
  if (Cur.enabled4 != Prev.enabled4)
    printf("Channel 4 %s\r\n",  Cur.enabled4 ? "ENABLED" : "DISABLED");

  if (Cur.con1 != Prev.con1)
    printf("Channel 1 %s\r\n",  Cur.con1 ? "CONNECTED" : "DISCONNECTED");
  if (Cur.con2 != Prev.con2)
    printf("Channel 2 %s\r\n",  Cur.con2 ? "CONNECTED" : "DISCONNECTED");
  if (Cur.con3 != Prev.con3)
    printf("Channel 3 %s\r\n",  Cur.con3 ? "CONNECTED" : "DISCONNECTED");
  if (Cur.con4 != Prev.con4)
    printf("Channel 4 %s\r\n",  Cur.con4 ? "CONNECTED" : "DISCONNECTED");

  if (Cur.spark1 AND !Prev.spark1)  printf("Channel 1 SPARKED!\r\n");
  if (Cur.spark2 AND !Prev.spark2)  printf("Channel 2 SPARKED!\r\n");
  if (Cur.spark3 AND !Prev.spark3)  printf("Channel 3 SPARKED!\r\n");
  if (Cur.spark4 AND !Prev.spark4)  printf("Channel 4 SPARKED!\r\n");

  if (Cur.resetFault != Prev.resetFault)
    printf("Reset Fault %s\r\n",  Cur.resetFault ? "ASSERTED" : "CLEARED");
}


//=============================================================================
//  Now we update the values in "next" based on the "cur" and "prev"
//  values.
//=============================================================================
//  All values changed by updateNextState() are "outputs" from this func.
//  All other values referenced in this function are "inputs" to this func.
//
//  All the values in "next" provide information about the next logical state
//  for the controller:  Values to be applied during the next pass of the
//  control logic.  All "next" values fall in to one of 2 categories:
//
//    - They are set/updated ONLY by this function (which is executed only
//      by the "applic" task)
//
//    - They are set ONLY by other tasks (within a lock), and possibly
//      reset/cleared by the "applic" task in updateCurState() (after copying
//      "next" to "cur")
//
//
//  All values that act as input to this func must fall in to one of these
//  categories:
//
//    - "prev" values
//
//    - "cur" values updated directly by statements in updateCurState()
//
//    - "cur" values updated indirectly by updateCurState() due to the copy
//      of "next" to "cur", which includes the values in "next" that are
//      set by other tasks
//
//    - a shared/global value that is the equivalent of a "cur" value but
//      for which we don't need to track changes
//
//
//  All "outputs" from this function must be altered ONLY within this function.
//
//  All "inputs" to this function must NEVER be altered within this function.
//
//  Each output value (or VERY small "set" of ouput values) must be updated
//  in a single, self-contained block of code.  NO output may be referenced
//  OR altered in ANOTHER block of code:  This is a declarative style of
//  programming and has multiple advantages:
//
//     - There are no order-of-execution dependencies across nodes/blocks
//       within a pass.  They can be executed in any order and the result
//       will be the same (get used to it - this method will be the norm as
//       computer languages and hardware add support for atomic transactions
//       and computers evolve in to neural-net-like processor arrays - FPGAs
//       are just the beginning of this trend).
//
//     - Encapsulating all logic for an output within a single node/block
//       leaves just one place to start when tracking a problem backward from
//       the result (far simpler than guessing at the possible causes and
//       trying to work forward, looking for paths to the observed result).
//
//     - Drastically reduces the complexity of the code by greatly reducing
//       the number of possible paths through the code.  This simplifies
//       testing and verification processes.
//
//  Each node/block-of-code should be prefaced with the following information:
//   - The names of the values it modifies
//   - A descr of the meaning of the values and the logic used to set them
//   - A list of the values that serve as "inputs" to the node
//
//=============================================================================
void updateNextState(void)
{

  if (current != applTask)  return;  // must only be called by "applic" task

//EXAMPLE:
//--------------------------------------------------------------------
// Next.state
// Next.onRequest, Next.offRequest, Next.resetRequest (resets)
//    Description:
//       Determines the next state of for the application logic
//       (STATE_ON, STATE_OFF, STATE_INTLK)
//    Inputs:
//       Cur.onRequest
//       Cur.offRequest
//       Cur.resetRequest
//       Cur.externIntlk
//       Cur.bulkSup[]
//       holecOK
//       psReady
//----------------------------------------------------------------------
/*
//--- chg state from OFF to ON? ---
  if (Cur.onRequest)  do  {
    Next.onRequest = false;
    if (!supplyType)  {
      printf("Must set supplyType 1st!\n");  break;  }
    if ((Cur.state == STATE_INTLK) OR Cur.externIntlk)  {
      printf("Must clear Interlock first\n\n");  break; }
    if (Cur.state == STATE_ON)  { printf("Already ON\n\n");  break; }
    Next.state = STATE_ON;  printf("Turning ON\n\n");
  } while(0);

//--- chg state from ON to OFF? ---
  if (Cur.offRequest)  do {
    Next.offRequest = false;
    if ((Cur.state == STATE_INTLK) OR Cur.externIntlk)  {
      printf("Must clear Interlock first\n\n");  break; }
    if (Cur.state == STATE_OFF)  { printf("Already OFF\n\n");  break; }
    Next.state = STATE_OFF;  printf("Turning OFF\n\n");
  } while(0);

//--- chg state from INTLK to OFF? ---
  if (Cur.resetRequest)  do {
    Next.resetRequest = false;
    if (Cur.state != STATE_INTLK)  {
      printf("No Interlock to clear\n\n");  break;  }
    if (Cur.externIntlk)  {
      printf("External Interlock still exists\n\n");  break; }
    Next.state = STATE_OFF;  printf("Interlock cleared\n\n");
  } while(0);

//--- chg state to INTLK due to external interlock? ---
  if (Cur.externIntlk OR !holecOK)  do {
    if (Cur.state == STATE_INTLK)  break;
    Next.state = STATE_INTLK;
  } while(0);
//--- change state to INTLK due to a bulk supply error? ---
  for (ui=0; ui<NumPS; ++ui)  if (Cur.bs[ui].error)  Next.state = STATE_INTLK;

*/


}



//=============================================================================
//  Next we update output signal values to be written to the outputs at
//  the beginning of the next pass (just after the inputs are read).
//=============================================================================
void updateOutputState(void)
{
  if (current != applTask)  return;  // must only be called by "applic" task

//--- update output values that are based on current conditions (values ---
//--- in "cur" or any values used in a conditional in the applic logic) ---

/*EXAMPLE:
  // (insured earlier that only ONE of these can be true when setting them)
  if (initialBtnPressed)  setVal[FP_METER] = rfOffSetpoint;  else
  if (finalBtnPressed)    setVal[FP_METER] = rfOnSetpoint;  else
                          setVal[FP_METER] = cavityVolts;


  SetOffApplLEDs(Next.digitalOut);

  if (limited)  SetOnLimitLED(Next.digitalOut);
  if (regErr)   SetOnErrorLED(Next.digitalOut);
*/

//--- update output values based on values that were set by the application ---
//--- logic (e.g. values in "next" or variables that are the target of an ---
//--- assignment statement in the application logic) ---

//  setVal[CTL_VOLTS] = Next.ctlVolts;  // ctl voltage to DC Servo amplifier

//  setVal[EXTERN_METER] = Next.meterValue;


  Next.digitalOut = Cur.digitalOut;

//APPLCODE
  if (cmdBitsAppl & CBA_ENABLE_CH1)  SetBits(Next.digitalOut, ENABLE_CH1);
  if (cmdBitsAppl & CBA_ENABLE_CH2)  SetBits(Next.digitalOut, ENABLE_CH2);
  if (cmdBitsAppl & CBA_ENABLE_CH3)  SetBits(Next.digitalOut, ENABLE_CH3);
  if (cmdBitsAppl & CBA_ENABLE_CH4)  SetBits(Next.digitalOut, ENABLE_CH4);

  if (cmdBitsAppl & CBA_DISABLE_CH1)  ClearBits(Next.digitalOut, ENABLE_CH1);
  if (cmdBitsAppl & CBA_DISABLE_CH2)  ClearBits(Next.digitalOut, ENABLE_CH2);
  if (cmdBitsAppl & CBA_DISABLE_CH3)  ClearBits(Next.digitalOut, ENABLE_CH3);
  if (cmdBitsAppl & CBA_DISABLE_CH4)  ClearBits(Next.digitalOut, ENABLE_CH4);

  if (cmdBitsAppl & CBA_RESET_FAULT)
    SetBits(Next.digitalOut, RESET_FAULT);
  else
    ClearBits(Next.digitalOut, RESET_FAULT);

}




//=============================================================================
// The last thing we do is update values read by other tasks
//=============================================================================
// IMPORTANT NOTES and WARNINGS:
//
//   To avoid concurrency issues, other tasks need to use locks when reading
//   values set here.  But be careful to keep such locks short to avoid
//   delaying the timing of the main application task.  If necessary, split
//   such code in to sections, getting and releasing the lock(s) in each
//   section to avoid increasing latency for the main application task.
//
//   Values in the "next" instance of the application state fall in to 2
//   categories:  Inputs and Outputs.
//
//   The "next" Outputs are those that may be changed in updateOutputState().
//   The reset are Inputs, and may be changed by other tasks within a lock.
//
//   Since each pass through the application logic starts by setting
//   cur = next, any "next" values set by other tasks become part of the
//   "cur" instance used by the rest of the applic logic.  Because the
//   "applic" task at a higher priority than any other real-time task, the
//   changes to "next" values by other tasks can ONLY happen between passes
//   through the applic logic loop.
//
//   NOTES:
//     - Some of these "next" input values values must be initialized in
//       updateCurState() in order to tell when a value is set by another
//       task.  For example, if we have a boolean flag that is set when we
//       get an ON command, then (after its "next" value is copied to "cur"),
//       we have to clear the flag in order to tell if another ON command is
//       received between passes.
//
//     - As with reads to these values, other tasks must use locks when
//       writing to the "next" input values to insure they are not preempted
//       by the "applic" task in the middle of a write.
//
//=============================================================================
void updateExternals(void)
{
  if (current != applTask)  return;  // must only be called by "applic" task

  digInBits = Cur.digitalIn;
  digOutBits = Cur.digitalOut;

//APPLCODE

  if (cmdBitsAppl & CBA_CLEAR_CH1)  {
    count_ch1 = 0;  printf("CH1 count CLEARED\r\n");  }
  if (cmdBitsAppl & CBA_CLEAR_CH2)  {
    count_ch2 = 0;  printf("CH2 count CLEARED\r\n");  }
  if (cmdBitsAppl & CBA_CLEAR_CH3)  {
    count_ch3 = 0;  printf("CH3 count CLEARED\r\n");  }
  if (cmdBitsAppl & CBA_CLEAR_CH4)  {
    count_ch4 = 0;  printf("CH4 count CLEARED\r\n");  }


  cmdBitsAppl = 0;

  statusBitsAppl = 0;

  if (digitalOut & ENABLE_CH1)  SetBits(statusBitsAppl, SBA_ENABLED_CH1);
  if (digitalOut & ENABLE_CH2)  SetBits(statusBitsAppl, SBA_ENABLED_CH2);
  if (digitalOut & ENABLE_CH3)  SetBits(statusBitsAppl, SBA_ENABLED_CH3);
  if (digitalOut & ENABLE_CH4)  SetBits(statusBitsAppl, SBA_ENABLED_CH4);

  if (Cur.con1)  SetBits(statusBitsAppl, SBA_CH1_CON);
  if (Cur.con2)  SetBits(statusBitsAppl, SBA_CH2_CON);
  if (Cur.con3)  SetBits(statusBitsAppl, SBA_CH3_CON);
  if (Cur.con4)  SetBits(statusBitsAppl, SBA_CH4_CON);

  if (Cur.fault)  SetBits(statusBitsAppl, SBA_FAULT);

  if (Cur.spark1) SetBits(statusBitsAppl, SBA_CH1_SPARK);
  if (Cur.spark2) SetBits(statusBitsAppl, SBA_CH2_SPARK);
  if (Cur.spark3) SetBits(statusBitsAppl, SBA_CH3_SPARK);
  if (Cur.spark4) SetBits(statusBitsAppl, SBA_CH4_SPARK);
}



//=============================================================================
_nearcall float readGetRaw(uint ui)  {
  return (ui < NumHdwADC ? anaIn[ui] : 0.0);  }

_nearcall float readMinRaw(uint ui)  {
  return (ui < NumHdwADC) ? (adcMinRaw(ui) - adcOffset[ui]) * adcGain[ui] : 0.0; }

_nearcall float readMaxRaw(uint ui)  {
  return (ui < NumHdwADC) ? (adcMaxRaw(ui) - adcOffset[ui]) * adcGain[ui] : 9.8; }

_nearcall void readDefaults(uint ui, float *gain, float *offset)  {
  *gain = 1.0;  *offset = 0.0; }

_nearcall void readCalibPoints(uint ui, float *low, float *high)  {
  *low = -9.5;  *high = 9.5; }

_nearcall char *readUnits(uint ui, char *buf)  { return "V"; }


//=============================================================================
_nearcall float setGetRaw(uint ui)  {
  return (ui < NumHdwADC ? anaOut[ui] : 0.0);  }

_nearcall float setMinRaw(uint ui)  {
  return (ui < NumHdwDAC) ? (dacMinRaw(ui) - dacOffset[ui]) * dacGain[ui] : 0.0; }

_nearcall float setMaxRaw(uint ui)  {
  return (ui < NumHdwDAC) ? (dacMaxRaw(ui) - dacOffset[ui]) * dacGain[ui] : 9.8; }

_nearcall void setDefaults(uint ui, float *gain, float *offset)  {
  *gain = 1.0;  *offset = 0.0;  }

_nearcall void setCalibPoints(uint ui, float *low, float *high)  {
  *low = -9.5;  *high = 9.5;  }

_nearcall char *setUnits(uint ui, char *buf)  { return "V"; }


//=============================================================================
//  Set default labels for all the I/O signals
//=============================================================================
void setDefaultLabels(void)
{
  printf("Setting default labels for I/O signals\r\n");

//--- set application-appropriate default labels ---
//APPLCODE

//--- set default labels for the I/O signals ---
  fstrcpy(ainLabel[0], "N/A");

  fstrcpy(aoutLabel[0], "N/A");

  fstrcpy(dinLabel[0], "SPARK_CH1");
  fstrcpy(dinLabel[1], "SPARK_CH2");
  fstrcpy(dinLabel[2], "SPARK_CH3");
  fstrcpy(dinLabel[3], "SPARK_CH4");

  fstrcpy(dinLabel[4], "CH1_NO_CON");
  fstrcpy(dinLabel[5], "CH2_NO_CON");
  fstrcpy(dinLabel[6], "CH3_NO_CON");
  fstrcpy(dinLabel[7], "CH4_NO_CON");

  fstrcpy(doutLabel[0], "ENABLE_CH1");
  fstrcpy(doutLabel[1], "ENABLE_CH2");
  fstrcpy(doutLabel[2], "ENABLE_CH3");
  fstrcpy(doutLabel[3], "ENABLE_CH4");

  fstrcpy(doutLabel[4], "FAULT_RESET");
  fstrcpy(doutLabel[5], "ACTIVITY_LED");
  fstrcpy(doutLabel[6], "not-used");
  fstrcpy(doutLabel[7], "not-used");
}


//=============================================================================
//  This function is called to force software configurable values to default
//  values.
//=============================================================================
void setDefaultConfig(void)
{
  printf("Setting in-RAM device config to default values\r\n");

//===== set default values for persistent data values =====
//APPLCODE

}


//=============================================================================
//  This function is called by the application loop just before a shutdown.
//  The appliation must do whatever is necessary for a clean shutdown and
//  then exit
//=============================================================================
void applicShutdown(void)
{
  printf("Application prepping for shutdown\r\n");  fflush(stdout);


  Exit(0);
}


//=============================================================================
//  This is the highest priority task, usually resumed at a regular interval
//  so that it can do what it needs and sleep again until the start of the
//  next interval.
//=============================================================================
void _task applic(int argc, char **argv)
{
  #define SharedApplLocals
  #include <sharedAppl.c>

//--- add local variable definitions here ---
//APPLCODE


  #define SharedApplLocals2
  #include <sharedAppl.c>



  #define SharedApplInit
  #include <sharedAppl.c>

//--- add application initialization code here ---
//APPLCODE


  #define SharedApplInit2
  #include <sharedAppl.c>


//--- The following loop is run at a periodic interval (normally 100 times ---
//--- per second, or every 10ms).  The application process normally ---
//--- consists of reading the current state of the digital and analog ---
//--- inputs, changing the state of the analog and digital outputs to the ---
//--- values computed during the previous pass, then computing new values ---
//--- for the outputs and updating values that outside clients can view ---
//--- (currently accessable as a set of ModBus registers) ---

  for (;;)  {
    #define SharedApplLoopStart
    #include <sharedAppl.c>


  //--- read the current state of the analog and digital Input signals, and ---
  //--- write the values computed at the end of the last pass ---
    digitalOut = Next.digitalOut;  rwIO();

  //--- process next event message (if any) ---
    processMsg(eventsfd, (EventMsg *)buf, sizeof(buf));

  //--- update the current state and determine the next state ---
    updateCurState();  updateNextState();
    updateOutputState();  updateExternals();


    #define SharedApplLoopEnd
    #include <sharedAppl.c>
  }

}


//=============================================================================
//  list of application-specific commands that are normally entered via a
//  user shell
//=============================================================================
//void _task nsclSeriald(int argc, char **argv);

//--- add any application-specific commands here ---
//APPLCODE
const CmdInfo far applCmds[] = {
//  { "misc",        miscTest,       "..." },
  { "cmd",       cmd,         "Application-specific commands" },
  { "state",     showState,   "Show state of controller" }
};

int const NumApplCmds = ( sizeof(applCmds) / sizeof(CmdInfo) );

//-----------------------------------------------------------------------------

#endif
