//----- modbusRegs.c ----- 07/18/12 -----(05/08/06)-----

#ifndef _modbusRegsC_
#define _modbusRegsC_


#include <sys/modbus.h>

//-----------------------------------------------------------------------------
//  Application-specific code extracted from modbus.c, now included in
//  the application-specific source file at compile time.  This file is
//  generic, but the result of compiling it is dependant upon application-
//  specific definitions in the application source files.
//
//  See modbus.c and the application source files for details.
//-----------------------------------------------------------------------------
//  Change log (most recent first):
//
//
//  2012-07-18 - Mark Davis
//               - Changes to PV_Desc and related macros to support locking
//                 and PVs without a separate read buffer.
//
//  2012-03-27 - Mark Davis
//               - Began changes to support PV types defined by structures
//                 other then RegInfo.
//
//  2010-05-26 - Mark Davis
//               - Added values to RegInfo structure to support processing of
//                 EPICS CA/PV value change events.
//
//  20xx-xx-xx - Mark Davis
//               - ???
//
//  2006-05-08 - Mark Davis
//               - original vesion
//
//-----------------------------------------------------------------------------


#define RegDefs

//-----------------------------------------------------------------------------
//  The 1st inclusion of the application source code file declares storage
//  for register values and initialized structures with info on each register
//-----------------------------------------------------------------------------

#define Coil_1perChanx   0
#define Coil_1perChan_   OnePerChan

#define COIL0( name, mult, coilNum, wrFunc )  \
  uchar C0##mult##ValDef(name);  \
  extern bool _nearcall wrFunc(SetCoilFuncParams);  \
  CoilInfo name##_CoilInfo = \
   { C0##mult##ValPtr(name), coilNum-1, #name, wrFunc, \
     Coil_1perChan##mult | LogChanges };


#define R3_1perChanx   0
#define R3_1perChan_   OnePerChan

#define REG3( name, mult, reg, typ )  \
  typ R3##mult##ValDef(name);  \
  typ R3##mult##ValDef(name##_ChgEventVal);  \
  typ R3##mult##ValDef(name##_MinChg);  \
  RegInfo name##_RegInfo = \
   { (S16 *)R3##mult##ValPtr(name), (reg)-30001, #name, NULL, DT_##typ, \
     NR_##typ, 0, R3_1perChan##mult | LogChanges, 0, 0, \
     R3##mult##ValPtr(name##_ChgEventVal), R3##mult##ValPtr(name##_MinChg) };


#define R4_1perChanx   0
#define R4_1perChan_   OnePerChan

#define REG4( name, mult, reg, typ, rw, wrFunc )  \
  typ R4##mult##ValDef(name);  \
  typ R4##mult##ValDef(name##_ChgEventVal);  \
  typ R4##mult##ValDef(name##_MinChg);  \
  extern bool _nearcall wrFunc(SetRegFuncParams);  \
  RegInfo name##_RegInfo = \
   { (S16 *)R4##mult##ValPtr(name), (reg)-40001, #name, wrFunc, DT_##typ, \
     NR_##typ, REG_##rw, R4_1perChan##mult | LogChanges, 0, 0, \
     R4##mult##ValPtr(name##_ChgEventVal), R4##mult##ValPtr(name##_MinChg) };


#define RegMacros
#include  ApplicSource

//------------------------------------------------------------------

#define  PV_DECLS
  #include ApplicSource

//------------------------------------------------------------------

#define  PV_DEFS

#define PV_STRING(name, flags, updVal, lockVal, sendVal, rcvVal) \
                       name##_IDX,

#define PV_ARRAY(name, val, type, flags, updVal, lockVal, sendVal, rcvVal) \
                       name##_IDX,

enum PV_IDXS {
  #include ApplicSource

  NUM_EXT_PV_IDXS
};


//------------------------------------------------------------------


#define  PV_DEFS


#undef PV_STRING
#define PV_STRING(name, flags, updVal, lockVal, sendVal, rcvVal)  \
  int _nearcall updVal(PV_OP_Params);   \
  int _nearcall lockVal(PV_Lock_Params);   \
  int _nearcall sendVal(PV_RW_Params);   \
  int _nearcall rcvVal(PV_RW_Params);


#undef PV_ARRAY
#define PV_ARRAY(name, val, type, flags, updVal, lockVal, sendVal, rcvVal)  \
  int _nearcall updVal(PV_OP_Params);   \
  int _nearcall lockVal(PV_Lock_Params);   \
  int _nearcall sendVal(PV_RW_Params);   \
  int _nearcall rcvVal(PV_RW_Params);


#include ApplicSource



//-----------------------------------------------------------------------------
//  The 2nd inclusion of the application source code file declares storage
//  creates arrays (one per register group) of pointers to register info
//  structures.  There is one pointer for each word (ModBus register).  For
//  values that require multiple words, the pointer corresponding to each
//  register will point to the same info structure.
//-----------------------------------------------------------------------------

#define REG_S16( name )  &( name##_RegInfo ),
#define REG_U16( name )  &( name##_RegInfo ),
#define REG_S32( name )  &( name##_RegInfo ), &( name##_RegInfo ),
#define REG_U32( name )  &( name##_RegInfo ), &( name##_RegInfo ),
#define REG_F32( name )  &( name##_RegInfo ), &( name##_RegInfo ),


//-------------------------------
#undef COIL0
#define COIL0( name, mult, reg, wrFunc )      &( name##_CoilInfo ),

CoilInfo *c0Coils[] = {
  #include  ApplicSource
  0
};


//-------------------------------
#undef REG3
#define REG3( name, mult, reg, typ )      REG_##typ( name )

RegInfo *r3Regs[] = {
  #include  ApplicSource
  0
};


//-------------------------------
#undef REG4
#define REG4( name, mult, reg, typ, rw, wrFunc )      REG_##typ( name )

RegInfo *r4Regs[] = {
  #include  ApplicSource
  0
};

//----------------------------------------------------------
#define  PV_DEFS

#undef PV_STRING
#define PV_STRING(name, flags, updVal, lockVal, sendVal, rcvVal)  { \
     { #name, DT_STR, \
       sizeof(name), sizeof(name), 1,  \
       (flags)|PV_SEND_RAW },  \
                 (void far *)(name), sizeof(name), 0, 0, \
                 updVal, lockVal, sendVal, rcvVal },

#undef PV_ARRAY
#define PV_ARRAY(name, val, type, flags, updVal, lockVal, sendVal, rcvVal)  { \
     { #name, DT_##type, \
       sizeof(val), sizeof(type), sizeof(val)/sizeof(type), \
       (flags)|PV_SEND_RAW },  \
            (void far *)(val), sizeof(val), 0, 0, \
            updVal, lockVal, sendVal, rcvVal },

PV_Desc  nonRegPVs[] = {
  #include ApplicSource

  { { NULL, 0, 0, 0, 0, 0 }, NULL, 0, 0, 0, NULL, NULL, NULL, NULL }
};


#define PV_Macros
#include  ApplicSource

//-----------------------------------------------------------------------------

uint  C0ChanMult, NumC0Chans, R3ChanMult, NumR3Chans, R4ChanMult, NumR4Chans;


//-----------------------------------------------------------------------------

#undef RegDefs


#endif
