//----- PhaseCtlr.c ----- 04/30/08 -----(03/26/06)-----

//-----------------------------------------------------------------------------
//  Update log (most recent first)
//
//  2008-04-30 - Mark Davis
//               - fixes for port F outputs
//
//  2008-04-17 - Mark Davis
//               - changes for prototype 3 (Nathan's 2nd board):
//                 The logic 1 output voltage level for the Rabbit is too high
//                 for the PROG FPGA output, so PE1 (DO-14) is set to INPUT
//                 mode for a logic 1 and output mode for a logic 0.
//-----------------------------------------------------------------------------

#include <common.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <sys/backup.h>

#include <sys/spi.h>
#include <sys/hdwDefs.h>

#include <busybox.h>

#include <Rabbit.h>
#include <dcdefs.h>

#include <stcpip.h>

#include <sys/socket.h>
#include <sys/srfs.h>
#include <sys/flashFile.h>
#include <sys/asflash.h>

#include <sys/farprintf.h>


//--- see hdwInit() for Digital I/O pin usage ---

const char ArchName[] = { "PhaseCtlr" };

#define  NUM_ADC  1
#define  NUM_DAC  1

#define  MAX_DIN   14
#define  MAX_DOUT  24


//--- these values can be set LOWER by the application ---
const uint  NumHdwADC = NUM_ADC;
const uint  NumHdwDAC = NUM_DAC;

const uint  MaxDIN =  MAX_DIN;
const uint  MaxDOUT = MAX_DOUT;


unsigned char  __brdInitFlag;

#include <hdwDefs.c>


//=============================================================================
_nearcall float adcGetRaw(uint ui)  { return (float) adcRaw[ui]; }

_nearcall float adcMinRaw(uint ui)  { return 0.0; }
_nearcall float adcMaxRaw(uint ui)  { return (float)0xffff; }

_nearcall void adcDefaults(uint ui, float *gain, float *offset)
{ *gain = 0.0003046;  *offset = 32786.0; }

_nearcall void adcCalibPoints(uint ui, float *low, float *high)
{ *low = -9.5;  *high = 9.5; }

_nearcall char *adcUnits(uint ui, char *buf)  { return "V"; }

//=============================================================================
_nearcall float dacGetRaw(uint ui)  { return (float) dacRaw[ui]; }

_nearcall float dacMinRaw(uint ui)  { return 0.0; }
_nearcall float dacMaxRaw(uint ui)  { return (float)0xffffL; }

_nearcall void dacDefaults(uint ui, float *gain, float *offset)
{  *gain = 0.000311;  *offset = 32768.0;  }

_nearcall void dacCalibPoints(uint ui, float *low, float *high)
  {  *low = -9.5;  *high = 9.5; }

_nearcall char *dacUnits(uint ui, char *buf)  { return "V"; }

//=============================================================================


//-----------------------------------------------------------------------------

/* START FUNCTION DESCRIPTION ********************************************
Function: int _readExtioPort ( int port );

Note:
This function is setup to provide additional hold time when
accessing external I/O addresses 0000 and 0002. This routine
must stay         and not be changed, for the additional
address hold-time to be valid.

Improved by Softools to not require a copy to RAM because
real tools have align and other stuff for real programmers.
************************************************************/
/*
unsigned readExtIO (unsigned port)      // Port is 0 or 2
{
#asm
    push    ip
    ipset   3
    ld      a,l
    and     a,0x03
    jr      z,..read0
    jr      ..skip
    .align  4           ; Align to clear A0 & A1
..read0:
    nop                 ; A0 = 0, A1 = 0
    nop                 ; A0 = 1, A1 = 0
    ioe     ld l,(hl)   ; A0 = 0, A1 = 1
    pop     ip          ; A0 = 0, A1 = 0
    jr      ..exit
    .align  4           ; Align to clear A0 & A1
..skip:
    ioe     ld l,(hl)   ; A0 = 0, A1 = 0 (2 bytes)
    pop     ip          ; A0 = 0, A1 = 1
..exit:
#endasm
}
*/

//-----------------------------------------------------------------------------
//  Determine the device ID based on external hardware settings
//
//  On the Digital Phase Meter, the ID switches are attached to the FPGA and
//  the state of the switches are read by issuing a command to the FPGA
//-----------------------------------------------------------------------------
//extern bool useLastID;  // to allow temp disabling of "reading" the device ID

ulong ReadDeviceID()
{
  uchar  inA;
  static uchar  devID;


//  if (useLastID)  return (ulong) devID;

  inA = RdPortI(PADR);

//--- device ID comes from 8-bit switch input ----
  devID = ((inA & 0x01) << 7)
        | ((inA & 0x02) << 5)
        | ((inA & 0x04) << 3)
        | ((inA & 0x08) << 1)
        | ((inA & 0x10) >> 1)
        | ((inA & 0x20) >> 3)
        | ((inA & 0x40) >> 5)
        | ((inA & 0x80) >> 7);
  devID = ~devID;

  return (ulong) devID;
}

//-----------------------------------------------------------------------------
//  return the current state of the digital inputs
//-----------------------------------------------------------------------------
ulong ReadDigInputs(void)
{
  uchar  inA, inD, inE, inF;
  ulong  inBits;


//--- read state of each of the digital inputs we need bits from ---
  inA = RdPortI(PADR);
  inD = RdPortI(PDDR);
  inE = RdPortI(PEDR);
  inF = RdPortI(PFDR);

  inBits = 0xfffc000L;

//--- PortA: ####-####  ==>  inBits: xxxx-xxxx xxxx-xxxx ####-####
  inBits |= (ulong)(inA);

//--- PortD: xx##-xxxx  ==>  inBits: xxxx-xxxx xxxx-xx## xxxx-xxxx
  inBits |= (ulong)(((uint)inD & 0x30) << 4);

//--- PortE: xxx#-#xxx  ==>  inBits: xxxx-xxxx xxxx-##xx xxxx-xxxx
  inBits |= (ulong)(((uint)inE & 0x18) << 7);

//--- PortF: xxx#-#xxx  ==>  inBits: xxxx-xxxx xx##-xxxx xxxx-xxxx
  inBits |= (ulong)(((uint) inF & 0x18) << 9);

  inBits = (inBits & ~forcedDigInMask) | forcedDigInBits;

  return inBits;
}


//-----------------------------------------------------------------------------
//  set the state of the digital outputs
//-----------------------------------------------------------------------------
void WriteDigOutputs(ulong digOut_)
{
  uchar  byt;
  ulong  digOut;


  digOut = (digOut_ & ~forcedDigOutMask) | forcedDigOutBits;

//--- digOut: xxxx-xxxx xxxx-xxxx ####-####  ==>  PortG: ####-####
  WrPortI(PGDR, &PGDRShadow, (uchar)digOut);

//--- digOut: xxxx-xxxx xx##-#### xxxx-xxxx  ==>  PortB: ####-##xx
  WrPortI(PBDR, &PBDRShadow,
    (PBDRShadow & 0x03) | ((uchar)(digOut >> 6) & 0xFC) );

//--- digOut: xxxx-xxx# ##xx-xxxx xxxx-xxxx  ==>  PortE: ##xx-xx#x
//--- NOTE:  DO-14 (PE1) must be either forced low, or at a high-impedance
//---        state (which gets pulled high by an external pull-up resistor).
//---        To manage this, we set it to output mode for logic 0 and input
//---        mode for a logic 1.
  if ((uint)digOut & 0x004000)  // logic 1 for PE1, so set it to INPUT mode
    WrPortI(PEDDR, &PEDDRShadow, (PEDDRShadow|0xC2) & ~0x1A); //dir:  OOxI-IxIx
  else                          // logic 0 for PE1, so set it to OUTPUT mode
    WrPortI(PEDDR, &PEDDRShadow, (PEDDRShadow|0xC2) & ~0x18); //dir:  OOxI-IxOx
  WrPortI(PEDR, &PEDRShadow, (PEDRShadow & 0x3D)
      | ((uchar)(digOut >> 9) & 0xC0) | ((uchar)(digOut >> 13) & 0x20) );

//--- digOut: ###x-#x#x xxxx-xxxx xxxx-xxxx  ==>  PortF: ###x-x#x#
  byt = (uchar)(digOut >> 16);
  WrPortI(PFDR, &PFDRShadow, (PFDRShadow & 0x1A)
          | (byt & 0xE0) | ((byt >> 1) & 0x05) );
}


//-----------------------------------------------------------------------------
void readAnalogInputs(ulong mask)
{
}


//-----------------------------------------------------------------------------
//  Update the state of the AD5542 DAC chips
//-----------------------------------------------------------------------------
void writeAnalogOutputs(ulong mask)
{
}

//-----------------------------------------------------------------------------
/*
float ADCtoVolts(uchar chan, uint rawADC)
{
  return (ADCOffset(chan) - (float)(rawADC)) * ADCGain(chan);
}
*/

//-----------------------------------------------------------------------------
//  Set an External 2/4-wire RS-485 adapter to transmit mode
//-----------------------------------------------------------------------------
void setExt485Tx(void)
{
}
//-----------------------------------------------------------------------------
//  Set an External 2/4-wire RS-485 adapter to receive mode
//-----------------------------------------------------------------------------
void setExt485Rx(void)
{
}

//-----------------------------------------------------------------------------
//  functions to enable/disable the transmitter for an RS-485 connection
//-----------------------------------------------------------------------------
void _nearcall SerSetDirA(int enableXmit)  {  }
void _nearcall SerSetDirB(int enableXmit)  {  }
void _nearcall SerSetDirC(int enableXmit)  {  }
void _nearcall SerSetDirD(int enableXmit)  {  }


//-----------------------------------------------------------------------------
void hdwInit(ulong digOutState)
{
  hdwType = HT_PHASE_CTLR;

  __brdInitFlag = TRUE;  // useLastID = false;

//--- Port A --- (*must be inputs due to bug in earlier Rabbit 3000's)
//  PA0 - I-00*
//  PA1 - I-01*
//  PA2 - I-02*
//  PA3 - I-03*
//  PA4 - I-04*
//  PA5 - I-05*
//  PA6 - I-06*
//  PA7 - I-07*
  WrPortI(SPCR, &SPCRShadow, SPCRShadow & ~0x0c);  // PortA: byte input

//--- Port B ---
//  PB0 - Serial-B (SPI) clock (output)
//  PB1 - Serial-A (Diag/Prog) clock (output)
//  PB2 - O-08
//  PB3 - O-09
//  PB4 - O-10
//  PB5 - O-11
//  PB6 - O-12
//  PB7 - O-13
  WrPortI(PBDDR, &PBDDRShadow, PBDDRShadow | 0xFC); // dir: OOOO-OOxx

//--- Port C ---
//  PC0 - Serial-D transmit
//  PC1 - Serial-D receive
//  PC2 - Serial-C (SPI) transmit
//  PC3 - Serial-C (SPI) receive
//  PC4 - Serial-B (SPI) transmit
//  PC5 - Serial-B (SPI) receive
//  PC6 - Serial-A (Diag/Prog) transmit
//  PC7 - Serial-A (Diag/Prog) receive
  WrPortI(PCFR, &PCFRShadow, 0x55);  // func: xAxA-xAxA

//--- Port D ---
//  PD0 - Ethernet Controller Reset (output)
//  PD1 - Not Avail (pulled high - otherwise not connected???)
//  PD2 - Ethernet Controller (TPOUT-)
//  PD3 - Ethernet Controller (TPOUT+)
//  PD4 - I-08
//  PD5 - I-09
//  PD6 - Ethernet Controller (TPIN-)
//  PD7 - Ethernet Controller (TPIN+)
  WrPortI(PDDDR, &PDDDRShadow, PDDDRShadow & ~0x30); // dir:  xxII-xxxx
  WrPortI(PDFR, &PDFRShadow, PDFRShadow & ~0x30);    // func: xxNN-xxxx

//--- Port E ---
//  PE0 - Ethernet Controller Interrupt (input)
//  PE1 - DO-14  *** must be high-impeadance when not pulled low ***
//  PE2 - Ethernet Controller Chip Select (output)
//  PE3 - I-10
//  PE4 - I-11
//  PE5 - External interrupt (input)
//  PE6 - O-15
//  PE7 - O-16
  WrPortI(PEDDR, &PEDDRShadow, (PEDDRShadow|0xC2) & ~0x1A); //dir:  OOxI-Ix*x
  WrPortI(PEFR, &PEFRShadow, PEFRShadow & ~0xDA);           //func: NNxN-NxNx

//--- Port F ---
//  PF0 - O-17
//  PF1 - Serial-C (EEPROM SPI) clock
//  PF2 - O-19
//  PF3 - I-12
//  PF4 - I-13
//  PF5 - O-21
//  PF6 - O-22
//  PF7 - O-23
  WrPortI(PFDDR, &PFDDRShadow, 0xE7);  // dir:  OOOI-IOOO
  WrPortI(PFFR, &PFFRShadow, 0x02);    // func: NNNN-NNAN

//--- Port G ---
//  PG0 - O-00
//  PG1 - O-01
//  PG2 - O-02
//  PG3 - O-03
//  PG4 - O-04
//  PG5 - O-05
//  PG6 - O-06
//  PG7 - O-07
  WrPortI(PGDDR, &PGDDRShadow, 0xFF);  // dir:  OOOO-OOOO
  WrPortI(PGFR, &PGFRShadow, 0x00);    // func: NNNN-NNNN


//--- all parallel port outputs driven hi/lo (no open-drain outputs) ---  `
  WrPortI(PDDCR, &PDDCRShadow, 0x00);
  WrPortI(PFDCR, &PFDCRShadow, 0x00);
  WrPortI(PGDCR, &PGDCRShadow, 0x00);


//--- set state of digital outputs ---
  digitalOut = digOutState;  WriteDigOutputs(digOutState);



  flashSize = 524288L;

  printf("\r\nInitialized %s hdw\r\n", ArchName);
}


//-----------------------------------------------------------------------------
//  Initialize and register hardware-dependent file systems
//-----------------------------------------------------------------------------
void registerFileSystems(void)
{
  socketInit_();
  srfsInit_();
  serialInit_();
#ifdef ASFLASH
  asflashInit_();
#endif

  flashFileInit_();
}

//-----------------------------------------------------------------------------
//  Initialize the network hardware
//-----------------------------------------------------------------------------
void netHdwInit()
{
//  Select ethernet hardware:
//  usingAsix, usingDevKit, usingRealtek, usingSMSC
//  usingAll() can be used to support all cmd core modules, but req's more code

  usingAsix();
}


//-----------------------------------------------------------------------------
//  allocate any resources that should be owned by the kernel process
//
//  To be called ONLY by the kernel process!!!!
//-----------------------------------------------------------------------------
void hdwInit2(void)
{
//  char  *argv_[3];
//  uint  argc_;

//  argv_[0] = "getty";  argv_[1] = "19200";  argv_[2] = "tty3";  argc_ = 3;
//  createProc(getty, 5, argc_, argv_);
}


//-----------------------------------------------------------------------------

