//----- SHD_DLM.c ----- 07/13/06 -----(xx/xx/xx)-----

//--- original code from SHDesigns ---

//--- modifications and reformatting by Mark Davis, NSCL ---


#include <stcpip.h>

#include "string.h"

#include <SHD_DLM.h>
#include <DLMiii.h>

#include <sys/farprintf.h>


static struct {
  unsigned udpdl_port;
  char * id_string;
  udp_Socket udp_dl_sock;
  char flags;
} _dl_data = {DNLD_UDP_PORT, ""};

extern unsigned long param_start;
int init_file_data(void);

#define CONFIG_KEY  0xFACE55aal
#define PRIMARY_KEY 0xFACE5533l
#define BACKUP_KEY  0xFACE55ccl


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int DLM_Init(char * str)
{
  _dl_data.flags = 0;
  _dl_data.id_string = str;

  if (str == NULL)  _dl_data.id_string = "Rabbit Board UDP Download";

  if (_dl_data.flags & DL_SOCKET_OPEN) {
    sock_close(&_dl_data.udp_dl_sock);  _dl_data.flags=0; }

  if (!udp_extopen(&_dl_data.udp_dl_sock, IF_ETH0, _dl_data.udpdl_port, -1L, -1, NULL, 0, 0))
  {
  #ifdef VERBOSE
    printf("UDPDL_Init - udp_open socket failed!\r\n");
  #endif
    return 1;
  }
  _dl_data.flags |= DL_SOCKET_OPEN; // tell future calls socket is open.

  init_file_data();  return 0;
}


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void UDPDL_SetPort(unsigned port)
{
  _dl_data.udpdl_port = port;
}


//-----------------------------------------------------------------------------
/* START FUNCTION DESCRIPTION ********************************************
int DLM_Reboot(int mode);                       <SHD_DLM.LIB>

PARAM: mode
UDPDNLD_MODE_RUN_DLM  - Run the resident DLM
UDPDNLD_MODE_RESTART  - Reboot and run Primary
UDPDNLD_MODE_RUN_BACKUP - Reboot and run Backup
UDPDNLD_MODE_RUN_OK   - Tell DM we started up properly
             (otherwise it will run backup on next reboot)

END DESCRIPTION **********************************************************/
//-----------------------------------------------------------------------------
extern unsigned long _xmemEnd;

int DLM_Reboot(int mode)
{
  unsigned long raddr = 0x80000l - 128l;
  int retval = UDPDNLD_MODE_RUN_PRIMARY;
  RUN_INFO run_info;

  fmemcpy(&run_info, (char far *)raddr, sizeof(RUN_INFO));
  if (run_info.Key == BACKUP_KEY)  retval = UDPDNLD_MODE_RUN_BACKUP;
  memset(&run_info, 0, sizeof(RUN_INFO));

  switch (mode)  {
    case UDPDNLD_MODE_RUN_DLM:
      run_info.Key = CONFIG_KEY;
      run_info.Port = _dl_data.udpdl_port;
      strcpy(run_info.IDString, _dl_data.id_string);
      run_info.IPAddress = htonl(gethostid());
      break;

    case UDPDNLD_MODE_RESTART: // 0 will just restart
      run_info.Key = 0; // will cause primary to run
      break;

    case UDPDNLD_MODE_RUN_BACKUP:
      run_info.Key = PRIMARY_KEY;
      break;
  }
  fmemcpy((char far *)raddr, &run_info, sizeof(RUN_INFO));

  #ifdef VERBOSE
    printf("Set key to %lX at %lx\n", run_info.Key, raddr);
  #endif
  if (mode != UDPDNLD_MODE_RUN_OK)  {
  ;
#asm
    ipset 3;
        ld  a,0x53
        ioi ld (WDTCR),a
        xor a
        ioi ld (WDTTR),a
wait_wdt:
        jr      wait_wdt
#endasm
  }
  return retval;
}


//-----------------------------------------------------------------------------
/* START FUNCTION DESCRIPTION ********************************************
int DLM_Tick();                       <SHD_DLM.LIB>

SYNTAX: int DLM_Tick();

PARAMETERS:

KEYWORDS: download

DESCRIPTION: Checks for download request and processes Queries.

RETURN VALUE: 0 = if no download request
              1 = if Download requested
Note: When this function returns 1 the user could could
shut down user functions like interrupts and threads

After this routine returns 1, at some later call, the DLM
will be executed and any user code may be lost. This return allows user
code to exit gracefully. I.e. flush buffers, close connections. Turn off
power to devices.

Note: downloads can not be run under the debugger, the DC IDE will get confused
 and reset the board.

END DESCRIPTION **********************************************************/
//-----------------------------------------------------------------------------
int DLM_Tick(void)
{
/*
  auto UDP_DBG_HEADER * up;
  auto int len;
  static word port;
  static unsigned long ip;
  auto char mac[6];
  auto char buff[1492]; // allow room for max code block

  up = (UDP_DBG_HEADER *)buff;

  if (_dl_data.flags & DL_RUN_PENDING) // no need to clear flags, we're history anyway.
    DLM_Reboot(UDPDNLD_MODE_RUN_DLM);

//  tcp_tick(&_dl_data.udp_dl_sock);
  len = udp_recvfrom(&_dl_data.udp_dl_sock, buff, sizeof(buff), &ip, &port);

  if (len > 0)  switch (up->cmd)  {

    case UDPDNLD_READ:
          ReadFlash((char far *)up->address, paddr(up->data), up->length);
      up->status = UDPDNLD_STATUS_ACK;
      udp_sendto(&_dl_data.udp_dl_sock, buff, UDP_DBG_HEADER_SIZE+up->length, ip, port);
#ifdef VERBOSE
      printf("Read from %lx, %d bytes\n", up->address, up->length);
#endif
      break;

    case UDPDNLD_QUERY3:
      if (up->length != 0) // check valid, no data
        return 0; // spurious packet
      len = strlen(_dl_data.id_string);
      if (len > 79) len = 79;
      memcpy(up->data, _dl_data.id_string, len);
      up->data[len] = '\0';
      len++;
      pd_getaddress(0, mac);
      sprintf(up->data+len, " %02x:%02x:%02x:%02x:%02x:%02x",
                 mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
      len = len + strlen(up->data+len);
      up->status = UDPDNLD_STATUS_USER_CODE; // tell downloader to send us loader
      up->address = ETH_MTU;  // used by downloader
      up->length = len+1;
      udp_sendto(&_dl_data.udp_dl_sock, buff, UDP_DBG_HEADER_SIZE+1+len, ip, port);
#ifdef VERBOSE
      printf("Query from %lx port %d\n", ip, port);
#endif
      return 0;

    // this is only called if loadein is not in FLASH
    case UDPDNLD_REBOOT: // off we go!
      _dl_data.flags |= DL_RUN_PENDING;
      return 1;

    case UDPDNLD_SET_SPECIFIC_IP:
      pd_getaddress(IF_ETH0, mac);
      if (memcmp(&up->data, mac, 6) != 0)
        return 0;
      // fall into set IP.

    case UDPDNLD_SET_IP:
      sethostid(up->address);
      // ip has changed so we need to close and reopen socket.
      sock_close(&_dl_data.udp_dl_sock);
      up->status = UDPDNLD_STATUS_ACK;
      udp_extopen(&_dl_data.udp_dl_sock, IF_ETH0, _dl_data.udpdl_port, -1L, -1, NULL, 0, 0);
      break;

    case UDPDNLD_SET_RTC:
      writeRTC(up->address);
      up->length = 0;
      up->status = UDPDNLD_STATUS_ACK;
      udp_sendto(&_dl_data.udp_dl_sock, buff, UDP_DBG_HEADER_SIZE, ip, port);
      break;

    case UDPDNLD_GET_PARAMS:
      if (!param_start)  init_file_data();
      up->length = 0;
      up->address = param_start;
      up->status = UDPDNLD_STATUS_ACK;
      udp_sendto(&_dl_data.udp_dl_sock, buff, UDP_DBG_HEADER_SIZE, ip, port);
      break;
  }
*/
  return 0;
}


//-----------------------------------------------------------------------------

