//----- applic.h ----- 12/05/12 -----(03/05/06)----


#ifndef _applicH_
#define _applicH_


#include <sys/types.h>

#include <busybox.h>


//-----------------------------------------------------------------------------
//  Update log (most recent first)
//
//
//  2012-07-31 - Mark Davis
//               - Changed several ms time values from 32 to to 16-bit to
//                 improve efficiency.
//
//  2011-05-04 - Mark Davis
//               - Added startupComplete that is set true just before the
//                 main application loop starts.  This was added to allow
//                 certain log messages to be supressed to minimize the
//                 liklihood of overflowing the system log buffer during
//                 startup.
//
//  2011-04-08 - Mark Davis
//               - Moved functions for tracking and managing averaged and noise
//                 values for analog inputs to the hdwDefs module.
//
//  2011-04-07 - Mark Davis
//               - Replaced resetRawMinMax() with a more comprehensive set of
//                 functions/macros for tracking the noise level of the ADC
//                 inputs.
//
//  2011-04-05 - Mark Davis
//               - Added fmtEng()
//
//  2011-02-25 - Mark Davis
//               - Added setGenericLabels() (which is implemented in the
//                 application-specific code modules).
//
//  2011-01-06 - Mark Davis
//               - Added allowSoftReset so individual applications can control
//                 when a soft reset is allowed.
//
//  2011-01-04 - Mark Davis
//               - Added application-specific ApplStateDataSize which provides
//                 the number of bytes needed to save the state of the
//                 application logic (used by the soft reset process).
//               - Added ApplStateDataHdr struct definition and stateDataBuf
//                 ptr (which points to ApplStateDataSize bytes to store the
//                 application state across a soft reset).
//
//  2010-12-06 - Mark Davis
//               - Added ctlrStateFile and saveApplState()
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// older style - to be eliminated...

typedef struct digFmtInfoStruct  {
  short  x, y;
  char  *str0, blink0;
  char  *str1, blink1;
} DigFmtInfo;

typedef struct analogFmtInfoStruct  {
  short  x, y;
  short  len, prec;
}  AnalogFmtInfo;

extern const DigFmtInfo digInFmt[], digOutFmt[];
extern const AnalogFmtInfo  adcFmt[],  dacFmt[];


//-----------------------------------------------------------------------------
//  preferred style...

typedef struct DispElem_  DispElem;

typedef char * (*FmtFunc)(DispElem far *elem, char *buf);

struct DispElem_ {
  uchar     x, y;
  void far *val;
  char     *attrs;
  FmtFunc   fmtFunc;
  int       width, prec;
};

typedef struct {
  uchar     x, y;
  ulong     mask;
  char     *str0, *attr0;
  char     *str1, *attr1;
}  DigDispElem;


#define ELEM_VAL(type)  ( *((type far *)elem->val) )


void displayElems(DispElem far *elems, bool blinkOff, char *buf);

void displayDigElems(ulong digBits, DigDispElem far *elems,
                     bool blinkOff, char *buf);

_nearcall char *fmtEng(DispElem far *elem, char *buf);
_nearcall char *fmtF32(DispElem far *elem, char *buf);

_nearcall char *fmtU16(DispElem far *elem, char *buf);
_nearcall char *fmtS16(DispElem far *elem, char *buf);

_nearcall char *fmtU32(DispElem far *elem, char *buf);
_nearcall char *fmtS32(DispElem far *elem, char *buf);

_nearcall char *fmtYN(DispElem far *elem, char *buf);

_nearcall char *fmtBool(DispElem far *elem, char *buf);

//--- values for elem->prec used by fmtBool() ---
#define  FMT_YES_NO        1
#define  FMT_ON_OFF        2
#define  FMT_CLOSED_OPEN   3
#define  FMT_ENA_DISABLED  4

//-----------------------------------------------------------------------------


extern const char far applName[];
extern const char ApplAbbrev[];  // abbreviated version of application name
extern char far applDTM[];

extern const ulong  DigOutInitState;


extern const uint  NumApplAIN, NumApplAOUT, NumApplDIN, NumApplDOUT,
                   NumExtraIn;

extern uint  ctlLoopTicks, ctlLoopTicksPerSec;
extern uint  lastCtlLoopTime, lastReadTime;
extern uint  readTicks, writeTicks, rwTicks;
extern U16  kernelLoopTime;

extern ulong  ADC_Mask, DAC_Mask;

extern const CmdInfo far applCmds[];
extern int const NumApplCmds;

extern ParamFile hdwCfgFile, devDataFile, ctlrStateFile;



typedef struct {
  uint   dataSize;
  uint   crc;
  ulong  resetTime;
} ApplStateDataHdr;

extern void far *stateDataBuf;  // BB RAM buf for saving/restoring appl state

extern const uint ApplStateDataSize;

extern bool  restoredApplState, allowSoftReset, startupComplete;



EVENT_DEF( RestoredParamFile )


_nearcall char *ainName(uint uidx, char *buf);
_nearcall char *aoutName(uint uidx, char *buf);
_nearcall char *dinName(uint uidx, char *buf);
_nearcall char *doutName(uint uidx, char *buf);

_nearcall uint logicalOrderAIN(uint uidx);
_nearcall uint logicalOrderAOUT(uint uidx);
_nearcall uint logicalOrderDIN(uint uidx);
_nearcall uint logicalOrderDOUT(uint uidx);

//=============================================================================
extern const uchar  LogicOrderAIn[], LogicOrderAOut[],
                    LogicOrderDIn[], LogicOrderDOut[];

CalibInfoGroup(adc)
CalibInfoGroup(dac)
CalibInfoGroup(read)
CalibInfoGroup(set)

//=============================================================================

void formatIO(bool blinkOff, char *msg);

int saveApplState(void);
int restoreApplState(void);

void setGenericLabels(void);



void setDefaultLabels(void);
void setDefaultConfig(void);


void _task applic(int argc, char **argv);


void  restoreHostName(void far *curVal, char *newVal, uint count);
void  restoredCalibs(void far *curVal, char *newVal, uint count);

void  setApplDTM(void);

//-----------------------------------------------------------------------------

#endif
