//----- busybox.h ----- 02/14/12 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//  UPDATE LOG (most recent first)
//
//
//  2012-02-14 - Mark Davis
//               - Changed CmdInfo.procMain from type ConstMain to far ProcMain
//                 (part of changes to get more stuff out of root memory)
//
//  2011-03-30 - Mark Davis
//               - Added nc command
//
//  2011-01-28 - Mark Davis
//               - Added route and ping commands
//
//  2010-02-15 - Mark Davis
//               - Moved readBufCh() from this module to the "common" module.
//
//-----------------------------------------------------------------------------

#ifndef _busyboxH_
#define _busyboxH_


#include <sys/types.h>

#include <sys/backup.h>



typedef struct {
  const char far  *const cmdName;
  const far ProcMain   procMain;
  const char far  *const descr;
}  CmdInfo;



typedef struct {
  float (*getRaw)(uint uidx);
  float (*minRaw)(uint uidx);
  float (*maxRaw)(uint uidx);
  void (*defaults)(uint uidx, float *gain, float *offset);
  void (*calibPoints)(uint uidx, float *low, float *high);
  char * (*units)(uint uidx, char *buf);
  char * (*getName)(uint uidx, char *buf);
  uint  (*order)(uint idx);
  void (*newCalib)(uint idx);
  void  (*setOutput)(uint idx, float val, bool clearForced);
  ParamFile *paramFile;
  float *gains, *offsets;
  const uint  *numValues;
  char *setDescr;
}  CalibInfo;


#define  CalibInfoGroup(group)  \
  _nearcall float group##GetRaw(uint ui);  \
  _nearcall float group##MinRaw(uint ui);  \
  _nearcall float group##MaxRaw(uint ui);  \
  _nearcall void  group##Defaults(uint ui, float *gain, float *offset);  \
  _nearcall void  group##CalibPoints(uint uidx, float *low, float *high);  \
  _nearcall char *group##Units(uint ui, char *buf);  \
  extern const CalibInfo far group##CalibInfo;




extern const CmdInfo far  commonCmds[];
extern int const NumCommonCmds;


//-----------------------------------------------------------------------------

//void _task getty(int argc, char **argv);

//-----------------------------------------------------------------------------

bool showHelp(int minArgs, int argc, char **argv);

void showDesc(char *cmdStr);

int fileInfoStr(char *fileName, char *descr);

void bootFileNames(char *primaryName, char *backupName, bool showNames);

char *fmtIdleTime(char *str, ulong idle);

char *fmtHMSUsed(char *str, ulong secs);

char *fmtCmd(char *str, int strLen, task_t *task);

char *fmtHexLine(char *str, ulong offset, char *buf, uchar count);

int  copyData(fd_t fromfd, int fromFlags, fd_t tofd, int toFlags,
              bool checkForETX, bool echo);

bool calibValue(CalibInfo far *ci, uint ui, char *temp, float egu1, float egu2);

void calibrate(uint argc, char **argv, CalibInfo far *ci);


int setDiagMsgs(int argc, char **argv);
int setDiagBits(int argc, char **argv);


void _task paramFileCmd(int argc, char **argv);


//-----------------------------------------------------------------------------

void _task _adccalib(int argc, char **argv);
void _task _daccalib(int argc, char **argv);
void _task _readcalib(int argc, char **argv);
void _task _setcalib(int argc, char **argv);

//-----------------------------------------------------------------------------

void _task _adc(int argc, char **argv);
void _task _anares(int argc, char **argv);
void _task _args(int argc, char **argv);
void _task _arp(int argc, char **argv);

void _task _bootimg(int argc, char **argv);
void _task _btof(int argc, char **argv);

void _task _cat(int argc, char **argv);
void _task _cp(int argc, char **argv);

void _task _dac(int argc, char **argv);
void _task _date(int argc, char **argv);
void _task _df(int argc, char **argv);
void _task _diag(int argc, char **argv);
void _task _dig(int argc, char **argv);
void _task _dumpPort(int argc, char **argv);

void _task _eng(int argc, char **argv);

void _task _fifoClient(int argc, char **argv);
void _task _fifoServ(int argc, char **argv);
void _task _float(int argc, char **argv);
void _task _free(int argc, char **argv);
void _task _fsck(int argc, char **argv);

void _task getty(int argc, char **argv);
void _task _grep(int argc, char **argv);

void _task _hang(int argc, char **argv);
void _task _help(int argc, char **argv);
void _task __hostname(int argc, char **argv);

void _task _ifconfig(int argc, char **argv);
void _task _inttab(int argc, char **argv);
void _task _ios(int argc, char **argv);

void _task _kill(int argc, char **argv);

void _task _label(int argc, char **argv);
void _task _ls(int argc, char **argv);
void _task _lsof(int argc, char **argv);
void _task _ltest(int argc, char **argv);

void _task _more(int argc, char **argv);
void _task _mv(int argc, char **argv);

void _task _nc(int argc, char **argv);
void _task _netstat(int argc, char **argv);

void _task _od(int argc, char **argv);

void _task _ping_(int argc, char **argv);
void _task _ps(int argc, char **argv);

void _task _reboot(int argc, char **argv);
void _task _reg(int argc, char **argv);
void _task _renice(int argc, char **argv);
void _task _rdflash(int argc, char **argv);
void _task _rdmem(int argc, char **argv);
void _task _rm(int argc, char **argv);
void _task _route(int argc, char **argv);

void _task _scope(int argc, char **argv);
void _task _sh(int argc, char **argv);
void _task _shutdown(int argc, char **argv);
void _task _srfs(int argc, char **argv);
void _task _stat(int argc, char **argv);
void _task _sysinfo(int argc, char **argv);
void _task _syslog(int argc, char **argv);

void _task _testgets(int argc, char **argv);
void _task _testOut(int argc, char **argv);
void _task _tip(int argc, char **argv);
void _task _top(int argc, char **argv);

void _task _uname(int argc, char **argv);
void _task _update(int argc, char **argv);

void _task _watch(int argc, char **argv);


//-----------------------------------------------------------------------------

#endif
