//----- _farst1.c ----- 06/17/10 -----(06/17/10)-----

//-----------------------------------------------------------------------------
//  A version of _farst1() that always uses an even address to avoid single
//  write operations that span a 4K boundary
//-----------------------------------------------------------------------------
//  Version log (most recent first):
//
//  2010-06-17 - Mark Davis
//               - First version of replacement for Softools _farst1().  This
//                 version is compatible with hardware write protection, which
//                 the Softools version at the time of this writing was not.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#asm
    .cseg

__farst1::   ;;  target addr = C[3-0]:DE,  value = A

    push   hl         ;; preserve HL

    ex     de,hl      ;; copy addr to IY
    ld     iy,hl
    ex     de,hl

    ld     b,a        ;; save value to write in B

    ld     a,c        ;; copy most sig nibble of addr to A

    bit    0,e        ;; is the addr even or odd?
    jr     z,evenAddr

;; The passed addr is an odd #, so the read/write could span a 4K boundary.
;; We handle this by decrementing the address and copying the value to H
;; rather than L before writing HL to memory
    dec    iy
    ldp    hl,(iy)
    ld     h,b
    jr     writeVal

;; The passed addr is an even #, so the read/write will not span a 4K boundary
;; and we need to copy the value to L before writing HL to memory
evenAddr:
    ldp    hl,(iy)
    ld     l,b

writeVal:
    ldp    (iy),hl

    ld     a,b        ;; restore value to A

    pop    hl         ;; restore HL

    ret

#endasm

//-----------------------------------------------------------------------------

