//----- applCommon.c ----- 04/05/11 -----(10/05/06)-----

//-----------------------------------------------------------------------------
//  Generic functions used by most application-specific code
//-----------------------------------------------------------------------------
//
//  2011-04-05 - Mark Davis
//               - Added fmtEng()
//
//  2009-06-09 - Mark Davis -
//               - added fmtS16() and fmtS32()
//
//  2006-10-05 - Mark Davis -
//               - ported from old Dynamic-C code
//
//-----------------------------------------------------------------------------
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <signal.h>
#include <errno.h>

#include <string.h>

#include <sys/backup.h>

#include <sys/hdwDefs.h>

#include <applic.h>


//-----------------------------------------------------------------------------
//  call the formatting function for a list of display elements and display
//  the resulting string using the display attributes specified in the list
//-----------------------------------------------------------------------------
void displayElems(DispElem far *elems, bool blinkOff, char *buf)
{
  DispElem far  *elem;
  char  *str;

  for (elem=elems; elem->val; ++elem)  {
    str = (elem->fmtFunc)(elem, buf);
    DispStrAttr(elem->x, elem->y, elem->attrs, str, false);
  }
}

//-----------------------------------------------------------------------------
//  function to handle generic formatting/display/translation of I/O values
//
//  blinkOff specifies whether strings that should be blinking should be
//  visiable or not.
//  *msg must point to a buffer of at least 160 bytes
//-----------------------------------------------------------------------------
void displayDigElems(ulong digBits, DigDispElem far *elems,
                     bool blinkOff, char *buf)
{
  DigDispElem far  *elem;
  char  *str, *attrs;

  for (elem=elems; elem->mask; ++elem)  {
    if (digBits & elem->mask)  {
      str = elem->str1;  attrs = elem->attr1;  }
    else  {
      str = elem->str0;  attrs = elem->attr0;  }
    DispStrAttr(elem->x, elem->y, attrs, str, false);
  }
}


//-----------------------------------------------------------------------------
_nearcall char *fmtEng(DispElem far *elem, char *buf)
{
  return Engineering(ELEM_VAL(float), elem->width, buf);
}

//-----------------------------------------------------------------------------
_nearcall char *fmtF32(DispElem far *elem, char *buf)
{
  return fmtFloat(ELEM_VAL(float), elem->width, elem->prec, buf);
}

//-----------------------------------------------------------------------------
_nearcall char *fmtU16(DispElem far *elem, char *buf)
{
  sprintf(buf, "%*u", elem->width, ELEM_VAL(uint) );  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *fmtS16(DispElem far *elem, char *buf)
{
  sprintf(buf, "%*d", elem->width, ELEM_VAL(int) );  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *fmtU32(DispElem far *elem, char *buf)
{
  sprintf(buf, "%*lu", elem->width, ELEM_VAL(ulong) );  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *fmtS32(DispElem far *elem, char *buf)
{
  sprintf(buf, "%*ld", elem->width, ELEM_VAL(long) );  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *fmtYN(DispElem far *elem, char *buf)
{
  strcpy(buf, (ELEM_VAL(bool) ? "YES" : "NO ") );
  buf[elem->width] = 0;  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *fmtBool(DispElem far *elem, char *buf)
{
  bool  bval = ELEM_VAL(bool);
  char  *str;

  switch (elem->prec)  {
    case  FMT_YES_NO:  str = (bval ? "YES" : "NO ");  break;
    case  FMT_ON_OFF:  str = (bval ? "ON " : "OFF");  break;
    case  FMT_CLOSED_OPEN:  str = (bval ? "CLOSED" : "OPEN  ");  break;
    case  FMT_ENA_DISABLED:  str = (bval ? "ENABLED " : "DISABLED");  break;
    default:  str = "???";
  }

  strncpy(buf, str, elem->width);  buf[elem->width] = 0;  return buf;
}

//-----------------------------------------------------------------------------
//  function to handle generic formatting/display/translation of I/O values
//
//  blinkOff specifies whether strings that should be blinking should be
//  visiable or not.
//  *msg must point to a buffer of at least 160 bytes
//-----------------------------------------------------------------------------
//  OLDER non-preferred method - use above functions instead...
/*
void formatIO(bool blinkOff, char *msg)
{
  uint  i;
  ulong  mask;
  bool  blink;
  const DigFmtInfo  *dFmt;
  const AnalogFmtInfo  *aFmt;
  char  *str, *temp, *dispAttrs;


  temp = msg + 90;

//--- translate digitalIn bit values ---
  for (i=0,mask=1; ; ++i,mask<<=1)  {
    dFmt = digInFmt + i;    if (dFmt->x < 0)  break;
    str = (digitalIn & mask) ? dFmt->str1 : dFmt->str0;
    if (*str == 0)  continue;
    blink = (digitalIn & mask) ? dFmt->blink1 : dFmt->blink0;
    dispAttrs = blink ? DARK_RED : DARK_BLUE;
    DispStrAttr(dFmt->x, dFmt->y, dispAttrs, str, false);
  }


//--- translate digitalOut bit values ---
  for (i=0,mask=1; ; ++i,mask<<=1)  {
    dFmt = digOutFmt + i;    if (dFmt->x < 0)  break;
    str = (digitalOut & mask) ? dFmt->str1 : dFmt->str0;
    if (*str == 0)  continue;
    blink =  (digitalOut & mask) ? dFmt->blink1 : dFmt->blink0;
    dispAttrs = blink ? DARK_RED : DARK_BLUE;
    DispStrAttr(dFmt->x, dFmt->y, dispAttrs, str, false);
  }


//--- display Analog Input values ---
  for (i=0; ; ++i)  {
    aFmt = adcFmt + i;  if (aFmt->x < 0)  break;
    if (aFmt->len == 0)  continue;
    DispStrAttr(aFmt->x, aFmt->y, DARK_BLUE,
           fmtFloat(anaIn[i], aFmt->len, aFmt->prec, temp), false);
  }


//--- display Analog Output values ---
  for (i=0; ; ++i)  {
    aFmt = dacFmt + i;  if (aFmt->x < 0)  break;
    if (aFmt->len == 0)  continue;
    DispStrAttr(aFmt->x, aFmt->y, DARK_BLUE,
      fmtFloat(anaOut[i], aFmt->len, aFmt->prec, temp), false);
  }

}
*/

//-----------------------------------------------------------------------------
_nearcall char *ainName(uint uidx, char *buf)
{
  if (uidx >= NumApplAIN)
    sprintf(buf, "ai%02u", uidx);
  else
    strcpyfn(buf, ainLabel[uidx]);

  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *aoutName(uint uidx, char *buf)
{
  if (uidx >= NumApplAOUT)
    sprintf(buf, "ao%02u", uidx);
  else
    strcpyfn(buf, aoutLabel[uidx]);

  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *dinName(uint uidx, char *buf)
{
  if (uidx >= NumApplDIN)
    sprintf(buf, "di%02u", uidx);
  else
    strcpyfn(buf, dinLabel[uidx]);

  return buf;
}

//-----------------------------------------------------------------------------
_nearcall char *doutName(uint uidx, char *buf)
{
  if (uidx >= NumApplDOUT)
    sprintf(buf, "do%02u", uidx);
  else
    strcpyfn(buf, doutLabel[uidx]);

  return buf;
}

//-----------------------------------------------------------------------------
_nearcall uint logicalOrderAIN(uint uidx)  {
   return (uidx < NumApplAIN) ? LogicOrderAIn[uidx] : uidx; }

_nearcall uint logicalOrderAOUT(uint uidx)  {
   return (uidx < NumApplAOUT) ? LogicOrderAOut[uidx] : uidx; }

_nearcall uint logicalOrderDIN(uint uidx)  {
   return (uidx < NumApplDIN) ? LogicOrderDIn[uidx] : uidx; }

_nearcall uint logicalOrderDOUT(uint uidx)  {
   return (uidx < NumApplDOUT) ? LogicOrderDOut[uidx] : uidx; }

//-----------------------------------------------------------------------------

