//----- checkValue.c ----- 08/02/10 -----(08/02/10)-----

//-----------------------------------------------------------------------------
//  Module to support a generic set of logic for checking on the state of a
//  value.
//-----------------------------------------------------------------------------
//  Change log (most recent first):
//
//
//  2010-08-02 - Mark Davis
//               - Original version
//
//-----------------------------------------------------------------------------
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <signal.h>
#include <errno.h>

#include <string.h>

#include <sys/backup.h>

#include <sys/hdwDefs.h>

#include <applic.h>

typedef struct  {
  float  *val;              // value to evaluate
  bool    useABS;           // use abs(val)

  float  *errLevelStart;    // enters Error state if value >=
  float  *errLevelEnd;      // leaves Error state if value <=

  ulong   errStartTime;     // time error state started (in ms)
  ulong  *maxErrPeriod;     // max time in Error state (in ms)

  float   maxErrVal;        // max value achieved since errStartTime
  ulong   maxErrTime;       // first time this error state val was == maxErrVal

  ulong  *minDecrTime;      // delay from maxErrTime to check for minDecrRate
  float  *minDecrRate;      // if not 0, min rate of improvement req

}  ValueCheck;

//-----------------------------------------------------------------------------
//  Check the state of a value and log messages as appropriate.
//
//  This function compares the referenced value to the start and end values
//  to determine if the value is in an "error state".
//
//  The parameters that drive this function are all stored in a structure:
//
//    *val:
//         The floating-point value to be monitored
//    useABS:
//         If not 0, then fabs(*val) is used for all operations.
//
//    *errLevelStart:
//         The value above which *val is considered to be in an error state.
//    *errLevelEnd:
//         The value below which *val is NOT considered to be in an error state.
//
//    errStartTime:
//         The time (in ms) when the value last entered the error state.  This
//         should be initialized to 0 at startup.  Thereafter, it is maintained
//         by checkValue().
//    *maxErrPeriod:
//         The max time the value can be in an error state before the error
//         is considered "fatal" (i.e. the device is not operating properly).
//         If this is NULL, or the value pointed to is 0, then the error state
//         is always considered to be fatal.
//
//    maxErrVal:
//         Maintained checkValue(), this is the maximum value that *val
//         was seen at by checkValue() since *val last entered an error state.
//    maxErrTime:
//         Maintained checkValue(), this is the time (in ms) at which *val
//         was equal to maxErrVal.
//
//    *minDecrTime:
//         The minimum # of ms to wait after maxErrTime
//    *minDecrRate:
//         If not NULL, then this is used to determine if the average rate of
//         change is sufficient that an on-going error state is tolerated
//         the
//
//  Returns a value indicating the state:
//    < 0:  There was a problem checking the value (invalid pointer, or the
//          value was not a valid number, etc)
//      0:  The value is within normal operating range
//      1:  The value is in an error state and is not (known to be) improving
//      2:  The value is in an Error state but is improving
//      3:  The value has been in an Error state longer than allowed
//
//-----------------------------------------------------------------------------
int checkValue(ValueData *vd)
{


  return -1;
}

//-----------------------------------------------------------------------------


