//----- stricmp.c ----- 04/30/12 -----(04/27/12)-----

//-----------------------------------------------------------------------------
//  A version of stricmp() that works (the Softools one doesn't produce
//  consistent results when comparing a non-alpha char to the same alpha
//  character in upper and lower case (e.g. 'A' < '_' but 'a' > '_').
//
//  The result is that you get different results based on the case of the
//  characters in your string (which rather defeats the whole purpose of a
//  case insensitve compariosn!)
//-----------------------------------------------------------------------------
//  Version log (most recent first):
//
//
//  2012-04-30 - Mark Davis
//               - First version of replacement for Softools stricmp().
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//int near stricmp (const unsigned char *str1, const unsigned char *str2)

#asm
    .cseg

_stricmp::

;; Return value is in HL

        push  ix
        ld    ix,0x0000
        add   ix,sp

        ld    hl,(ix+4)
        ex    de,hl      ;; de = str1
        ld    hl,(ix+6)  ;; hl = str2

 ;; start of loop
  @1:   ld    a,(de)     ;; ch1 = *str1

        cp    (hl)       ;; ch1 == *str2?
        jr    z,@4

 ;; fold ch1 to lower case if it is an upper-case alpha char
        cp    0x41       ;; ch1 < 'A'?
        jr    c,@2
        cp    0x5B       ;; ch1 >= '['?
        jr    nc,@2

        or    0x20       ;; ch1 is now lower case

 ;; fold ch2 to lower case if it is an upper-case alpha char
  @2:   ld    b,a        ;; save ch1
        ld    a,(hl)     ;; ch2 = *str2

        cp    0x41       ;; ch2 < 'A'?
        jr    c,@3
        cp    0x5B       ;; ch2 >= '['?
        jr    nc,@3

        or    0x20       ;; ch2 (A) is now lower case

 ;; ch1 and ch2 are now lower case if they were upper-case alpha
  @3:   cp    b          ;; compare ch2 to ch1
        jr    c,@5       ;; ch2 < ch1 ?
        jr    nz,@6      ;; ch2 > ch1 ?

 ;; ch1 == ch2 ignoring case: check for end of strings
  @4:   or    a          ;; test for 0
        inc   de         ;; ++str1
        inc   hl         ;; ++str2
        jr    nz,@1

 ;; reached the end of both strings; set hl = 0 and return
        ld    h,a
        ld    l,a
        jr    @8

 ;; *str1 > *str2
  @5:   ld    hl,1
        jr    @8

 ;; *str1 < *str2
  @6:   ld    hl,0xffff


 ;; return
  @8:   pop   ix
        ret


#endasm




/*

#define AND  &&

//-----------------------------------------------------------------------------
//  My own C version alternative
//-----------------------------------------------------------------------------
int near stricmp(const unsigned char *str1, const unsigned char *str2)
{
  unsigned char  c1, c2;

  while (*str1 AND *str2)  {
    if ((c1 = *str1) > 0x40)  if (c1 < 0x5B)  c1 |= 0x20;
    if ((c2 = *str2) > 0x40)  if (c2 < 0x5B)  c2 |= 0x20;
    if (c1 > c2)  return  1;
    if (c1 < c2)  return -1;
    ++str1;  ++str2;
  }

  if (*str1 > *str2)  return  1;
  if (*str1 < *str2)  return -1;

  return 0;
}


//-----------------------------------------------------------------------------
//  Another pretty efficient looking C code version from the web
//-----------------------------------------------------------------------------
int near stricmp(const unsigned char *s1, const unsigned char *s2)
{
  unsigned char  c1, c2;
  int  ret = 0;

  if (s1 AND s2)  for (;;)  {
    c1 = *s1++;  c2 = *s2++;

    if (c1 AND c2)  {   // both chars not == 0
      if (!(c1 ^ c2))  continue;  // my own (optional) addition
      c1 = tolower(c1);  c2 = tolower(c2);
      if (c1 < c2)  { ret = -1;  break; }
      if (c1 > c2)  { ret =  1;  break; }
    }
    else  {  // one or both chars == 0
      if (c1)  ret = 1;  else  if (c2)  ret = -1;
      break;
    }
  }

  return ret;
}

*/

//-----------------------------------------------------------------------------

