	.objcmt	Softools, Inc. ** Control Cross-C Globally Optimizing Compiler ** Version 2.18.00
	.objcmt	Copyright (c) 1989 - 2003, Softools, Inc.  All rights reserved.
	.debug	P 5
	.debug	X 11 10
	.debug	X 12 10
	.debug	N 'C:\work\dev\Rabbit_ST\rtos\drivers\srfs.c' 17 1 10 5 12 32
;1 //----- srfs.c ----- 02/08/12 -----(07/31/06)-----
;2 
;3 /*-------------------------------------------------------------------------
;4    Client interface for Simple Remote File System (SRFS), an inefficient, b
;5    dirt simple protocol for accessing files remotely over a TCP connection.
;6   -------------------------------------------------------------------------
;7 
;8 ===========================================================================
;9   For details on how this driver works, see detailed description following
;10   the Update log.
;11 ===========================================================================
;12 
;13  Update log (most recent first):
;14 
;15 
;16   2012-02-08 - Mark Davis
;17                - errno is now set to EBADF rather than ENXIO when a functio
;18                  is passed an invalid device pointer.
;19 
;20   2011-10-25 - Mark Davis
;21                - The BYE command is now sent just before closing the shared
;22                  socket connection if no error has occured on the connectio
;23 
;24   2011-03-23 - Mark Davis
;25                - After MANY interruptions for other projects, finally a
;26                  clean, stable version of the single-socket changes.
;27 
;28   2010-10-25 - Mark Davis
;29                - Began work on changes to use a single socket connection to
;30                  SRFS server, shared by all tasks and protected by a semaph
;31 
;32   2010-02-05 - Mark Davis (*)
;33                - Added fatalError flag and set it if the conversation with 
;34                  SRFS server fails at any point.  Since failing to read or
;35                  write the expected # of bytes means the conversation is
;36                  basically out of sync, this means the connection is no
;37                  longer usable, so an EIO error is returned for the call th
;38                  had the problem and all subsequent calls.  Prior to this f
;39                  the results of a failed read or write on the socket connec
;40                  could mean all kinds of bizarre behavior following the err
;41                - remfd now initialized to -1 and checked in the close call
;42                  to avoid sending a close command for a remote file that wa
;43                  never opened.  This gets rid of all the "Bad file number"
;44                  errors in the srfsd.log on the server.
;45 
;46 
;47   2010-01-22 - Mark Davis (*)
;48                - Changed all read()/write() calls to readBytes()/writeBytes
;49                  with a timeout.
;50 
;51   2009-04-15 - Mark Davis
;52                - Fixed unlink() so it works again (the pathnames it is call
;53                  with now include the "/srfs/" prefix, which I believe wher
;54                  removed by an earlier version of fcntl.c
;55 
;56   2009-04-13 - Mark Davis
;57                - Fixed rename() so it works again (the pathnames it is call
;58                  with now includes the "/srfs/" prefix, which I believe was
;59                  removed by an earlier version of fcntl.c
;60 
;61   2009-03-24 - Mark Davis
;62                - added _srfsEntry_() function for getting info about SRFS f
;63 
;64   2007-10-05 - Mark Davis
;65                - added support for unlink() and lseek() calls
;66 
;67 ===========================================================================
;68  Calls made by the client:
;69 
;70   int  srfsOpen(const char *path, int oflag, int mode);
;71   int  srfsClose(int fd);
;72   int  srfsRead(int fd, void *buf, int nbyte);
;73   int  srfsWrite(int fd, void *buf, int nbyte);
;74   long srfsLseek(int fd, long offset, int whence);
;75   int  srfsUnlink(const char *pathname);
;76   int  srfsRename(const char *oldName, const char *newName);
;77 
;78   The protocol is kept very simple:
;79 
;80   - All parameters are sent in network (Most Significant 1st) byte order
;81 
;82   - Each request from a client consists of a 2-byte Func #, a 2-byte Size
;83     value, then Size # of bytes (if any):
;84     - 2 byte func #
;85     - 2 byte size
;86     - size # bytes
;87 
;88   - Each response from the server starts with 6 result bytes:
;89     - 2 byte return value
;90     - 2 byte errno value
;91     - 2 byte data len
;92     - data len # bytes
;93 
;94     The 6 result bytes are followed by "len" bytes of data (if any)
;95 
;96     For the lseek function, the 4-byte return value is in the data bytes
;97 
;98 ===========================================================================
;99  Notes on how this driver operates (newer, single shared socket approach):
;100 
;101     The newer version of this driver now uses a single socket connection
;102     shared by all tasks.
;103 
;104     This eliminates several of the issues with the older one socket per ope
;105     file approach:
;106 
;107       - Only one TCP socket resource is required, regardless of the number
;108         of files open on the server.
;109 
;110       - Opening a remote file does not require 2 free process fds (the olde
;111         version used an additional one for the socket connection that
;112         applied only to a single open file)
;113 
;114       - Connections are not constantly being opened and closed, so the serv
;115         is no longer stuck with hundreds of old closed connections in the
;116         one of the XXX_WAIT states.
;117 
;118 
;119  One connection to rule them all:
;120 
;121     Overview of how the revamped SRFS driver works:
;122 
;123        - A single connection is used by all tasks to communicate with the
;124          server.
;125 
;126        - The shared connection to the SRFS server is opened whenever a
;127          task needs to open a file on the server and there isn't currently
;128          an open connection to the server.
;129 
;130        - Only ONE task can be using the connection at a time.  Once
;131          started, a read/write/etc call must complete before another task
;132          can use the connection.
;133 
;134        - A semaphore is used as a global SRFS driver lock to insure that
;135          only one task is trying to use the shared connection at a time.
;136 
;137        - Any error on the shared connection means we are probably out of
;138          sync with the server, so the connection is closed and all files
;139          currently open by the SRFS driver are marked as bad.  (While it
;140          would be possible to modify the protocol so it could - at least
;141          in theory - recover and resync, the liklihood is that the
;142          occurance of an error that the TCP/IP layers could not recover
;143          from means the connection is almost certainly no longer usable).
;144 
;145 
;146    A more detailed description of how all this is done:
;147 
;148       The first time a file on the server is opened, the SRFS driver
;149       will open a connection to the server and leave it open.  A pointer
;150       to the FDEV structure (effectively a global fd) for the connection
;151       is saved in sockFDEV and used by all tasks.
;152 
;153       Calls to read(), write(), etc for remote files requires sending
;154       one or more messages to the server and receiving one or more
;155       messages in return.  To insure that messages (or even portions
;156       of messages) for different files are not mixed together, a global
;157       lock is used to insure only one operation for one task is using
;158       the connection at any time.
;159 
;160       So, each read/write/etc does this:
;161 
;162         - Get the global SRFS lock (or return an error if it can't
;163           get the lock)
;164 
;165         - Send the request and any data for the request to the
;166           server.
;167 
;168         - Read the response from the server (often 2 reads: one to
;169           get back a header with the status, and one to read any
;170           data that is part of the response).
;171 
;172         - Release the global SRFS lock so other tasks can use the
;173           connection.
;174 
;175       If an error occurs when writing to or reading from the shared
;176       connection, then there is a good chance that the client and server
;177       are no longer in sync.
;178 
;179         Remember that "successful" writes to an fd do NOT guarantee that
;180         the data made it all the way to its final destination - only that
;181         the driver accepted the data.
;182 
;183         Even getting a response back from the server for a write operation
;184         does not mean that the data was actually written to the file.  For
;185         all we know the server could  have crashed before it could update
;186         the disk with data that was accepted, acknowledged, and cached.
;187 
;188         This means that if any error occurs on the shared connection, all
;189         the fds for files that were using it are now unusable.
;190 
;191       So, whenever an error occurs, all the currently open SRFS files
;192       are marked as bad and the connection is closed.  Any tasks blocked
;193       on a pending operation will eventually get the global lock, find
;194       that the connection they were using was closed, and return an
;195       error.
;196 
;197       As with any error of this kind, recovery is up to the tasks that
;198       were using the files.  Only they know the context in which the
;199       file was being used and how to recover from the error.  Typical
;200       responses include trying again, or simply reporting the error
;201       and exiting.
;202 
;203 
;204    Sharing a connection across tasks:
;205 
;206       Opening the shared connection is currently done using the standard
;207       BSD-style socket library calls, which results in a process-specific
;208       fd being allocated for the task that opened the connection.
;209 
;210       Since this fd is usable only by the task that opened the connection,
;211       it is simply freed up, leaving the FDEV structure that it pointed to
;212       (and the socket driver-level structures IT points to).  It is the
;213       FDEV structure that is used by the SRFS driver functions to read and
;214       write to the shared connection.
;215 
;216       One consequence of freeing up the process-specific fd is that the
;217       FDEV structure it pointed to is now effectively an "orphan" (i.e.
;218       there are no direct references to it by any of the process level
;219       fds).  To avoid any cleanup or scanning routines from interpreting
;220       this as an error, a new flag was added to the FDEV structure that
;221       can be set to indicate that the FDEV is SUPPOSED to be an orphan.
;222 
;223       One of the advantages of this approach is that the only part of
;224       the code that is dependent on what type of connection is used is
;225       the srfsConnect function.  Everything else uses the generic API
;226       that the fcntl layer uses for all the drivers.  This means it
;227       would be fairly simple to add support for different connection
;228       types.
;229 
;230       Another advantage is that, with no pfd references to it, the
;231       shared FDEV will not be closed even when all the SRFS files have
;232       been closed.  This means that, unless an error occurs, the
;233       connection to the server should always be up and ready after it
;234       is first opened.
;235 
;236 
;237    Blocking - The fcntl layer vs the driver layer:
;238 
;239       From the point of view of the fcntl layer, drivers come in 2
;240       varieties:  Those that handle blocking on their own and those
;241       that don't.
;242 
;243       For the later group, the driver's read and write functions
;244       never block:  They simply read or write however bytes they can
;245       each time they are called.  It is up to the fcntl layer to handle
;246       the difference between blocking and non-blocking calls.
;247 
;248       For non-blocking reads and writes, the fcntl layer simply
;249       returns whatever the driver supplies.
;250 
;251       For blocking calls, it will call the driver's read or write
;252       function repeatedly until the specified # of bytes are processed
;253       or an error occurs.  Between calls to the driver, the task is
;254       suspended using the specified wait queue.  The more intelligent
;255       drivers provide their own wait queues which allows them to wake
;256       up tasks as soon as (and only when) there is a good chance the
;257       driver can process more bytes.  This improves response time and
;258       minimizes the number of CPU cycles wasted calling the driver
;259       when it cannot process more bytes.
;260 
;261 
;262       The SRFS driver is one of the ones that handles blocking on its
;263       own.  The only difference between the 2 modes is whether or not
;264       the task blocks if the global SRFS lock is in use.  After that,
;265       the two modes are the same, because once a conversation with the
;266       server is started, it has to be finished.
;267 
;268 
;269   Locking and wait queues:
;270 
;271      As noted above, a lock is used to insure that only one task at a time
;272      is using the shared connection.  To avoid the possibility that packets
;273      from different conversations can get mixed together, once a task owns
;274      the lock, it must not release it until it finishes a conversation.
;275      This means it sent a command and received a complete response, or an
;276      error occured (in which case no more conversations are allowed on the
;277      connection).
;278 
;279      The lock for the shared connection is also used to protect most of the
;280      global values used in this module.  Because the locking mechanism used
;281      is a semaphore, when a task releases the lock, the next task waiting
;282      for it (if any) will be woken up and given the semaphore.
;283 
;284 
;285   Open Questions:
;286 
;287       The blocking/non-blocking mode currently is NOT passed on to the
;288       server, so it opens all files in blocking mode.  But even so,
;289       is it possible that the server would NOT read or write the
;290       specified number of bytes (except when EOF is reached for a
;291       read, or a signal/etc interrupted the process)?
;292 
;293       If this CAN happen, then I the SRFS driver would need to
;294       return wait queues (even if just the sleepQueue) so the fcntl
;295       layer would do its thing (e.g. read, suspend, read, suspend, ...)
;296       to make sure all the bytes are processed for blocking calls.
;297 
;298 
;299 ===========================================================================
;300 
;301 
;302 #include <common.h>
;303 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\time.h' 15 48 18 10 12 22
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\string.h' 26 27 14 2 6 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stddef.h' 21 20 12 5 6 22
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\math.h' 19 23 18 21 9 21
	.debug	I
	.debug	I
;304 #include <unistd.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\signal.h' 6 21 8 17 3 29
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\list.h' 21 49 15 26 7 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\dummy.h' 25 58 21 12 10 27
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\list.h' 21 49 15 26 7 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stdio.h' 22 16 12 9 4 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\SoftoolsIDblock.h' 18 52 15 4 6 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\FlashBlock.h' 5 10 16 30 11 24
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Flash.h' 0 55 21 17 7 26
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Stdlib.h' 2 30 16 6 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\dcdefs.h' 18 42 23 19 7 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\SoftoolsIDBlock.h' 18 52 15 4 6 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errors.h' 25 8 15 18 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stdlib.h' 2 30 16 6 7 24
	.debug	I
	.debug	# 109
_LIN2SEG	.macro

	.debug	# 110
add hl,hl

	.debug	# 111
rla

	.debug	# 112
add hl,hl

	.debug	# 113
rla

	.debug	# 114
add hl,hl

	.debug	# 115
rla

	.debug	# 116
add hl,hl

	.debug	# 117
rla

	.debug	# 118
sub 0x0E

	.debug	# 119
or a

	.debug	# 120
rr hl

	.debug	# 121
scf

	.debug	# 122
rr hl

	.debug	# 123
scf

	.debug	# 124
rr hl

	.debug	# 125
scf

	.debug	# 126
rr hl

	.debug	# 127
.endm

	.cseg
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\hdwdefs.h' 20 2 12 27 10 31
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\fcntl.h' 29 48 12 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\ioctl.h' 2 15 10 21 5 30
	.debug	I
	.debug	I
	.debug	I
;305 #include <sys/stat.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\stat.h' 0 48 10 31 3 29
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\time.h' 15 48 18 10 12 22
	.debug	I
	.debug	I
;306 #include <sys/socket.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\socket.h' 2 54 9 7 6 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\netinet\in.h' 15 56 20 12 3 27
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip.h' 10 19 7 30 6 29
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\dcdefs.h' 18 42 23 19 7 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\dcrtcp.h' 1 9 9 21 2 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\tcp_config.h' 9 28 14 7 1 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\neterrno.h' 26 24 16 30 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errors.h' 25 8 15 18 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\servlist.h' 7 29 16 30 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\net.h' 21 21 9 7 3 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ppp.h' 4 42 22 17 6 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ppplink.h' 18 1 2 19 9 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\chat.h' 12 7 15 6 10 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\net.h' 21 21 9 7 3 32
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\pktdrv.h' 16 19 17 14 3 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\arp.h' 21 56 14 30 7 24
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ip.h' 16 35 12 25 4 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ip.h' 16 35 12 25 4 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\udp.h' 23 23 20 4 3 30
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\icmp.h' 2 44 0 17 5 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\tcp.h' 14 5 18 19 4 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\bootp.h' 2 16 3 7 2 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\bsdname.h' 9 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\icmp.h' 2 44 0 17 5 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\dns.h' 18 15 17 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\servlist.h' 7 29 16 30 7 24
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\igmp.h' 10 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\smtp.h' 0 54 12 2 8 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\http.h' 28 52 16 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\zserver.h' 26 57 12 10 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\limits.h' 1 13 9 26 5 19
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\pool.h' 10 58 16 7 1 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errno.h' 16 4 11 6 2 29
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\mib.h' 28 3 18 15 4 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\snmp.h' 22 42 15 4 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ftp_client.h' 9 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ftp_server.h' 5 50 18 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\zserver.h' 26 57 12 10 1 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ethernet.h' 21 56 23 21 3 26
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I
;307 #include <fcntl.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\fcntl.h' 29 48 12 5 12 32
	.debug	I
;308 #include <arpa/inet.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\arpa\inet.h' 14 56 20 12 3 27
	.debug	I
;309 #include <semaphore.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I
;310 #include <sys/time.h>
;311 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\time.h' 10 37 11 8 3 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I
;312 #include <errno.h>
;313 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\errno.h' 22 0 10 11 8 30
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I
;314 #include <sys/farprintf.h>
;315 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\farprintf.h' 5 43 7 29 11 27
	.debug	I
;316 #include <services.h>
;317 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\services.h' 2 22 6 20 2 29
	.debug	I
;318 #include <linux/limits.h>
;319 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
;320 
;321 
;322 //#define  SRFS_DEBUG
;323 
;324 
;325 typedef struct {
;326   bool      inUse;     // non-0 if this entry is in use
;327   int       errNum;    // non-0 means connection failed
;328   int       remfd;     // server's descriptor for an open file
;329   char far *name;      // name of remote file
;330 } SRFS_File;
;331 
;332 
;333 char far srfsFileName[MaxSRFSFiles][NAME_MAX];
;334 
;335 
;336 int sendCmd(const char *cmd, int cmdLen, const char *data, int dataLen);
;337 
;338 
;339 IODEV_FUNCS(srfs)
	.defseg	CONST
	.seg	CONST
_srfsIOFuncs::
	.dw	word __srfsRead_
	.dw	word __srfsWrite_
	.dw	word __srfsClose_
	.dw	word __srfsLseek_
	.dw	word __srfsFstat_
	.dw	word __srfsioctl_
	.dw	word __srfsDescr_
	.dw	word __srfsGetQueues_
	.debug	G _srfsIOFuncs 32 0
;340 IODEV_FUNCS(srfs)
;341 FILESYS_FUNCS(srfs)
_srfsFSysFuncs::
	.dw	@1
	.dw	word _srfsIOFuncs
	.dw	word __srfsTick_
	.dw	word __srfsEntry_
	.dw	word __srfsOpen_
	.dw	word __srfsUnlink_
	.dw	word __srfsRename_
	.debug	G _srfsFSysFuncs 33 0
;342 FILESYS_FUNCS(srfs)
;343 
;344 static SRFS_File srfsFile[MaxSRFSFiles];
;345 
;346 
;347 #define SRFS  ((SRFS_File *)ioDev)
;348 
;349 //--- request IDs for SRFS function "calls" ---
;350 #define SRFS_KEEPALIVE  0
;351 #define SRFS_OPEN       1
;352 #define SRFS_CLOSE      2
;353 #define SRFS_READ       3
;354 #define SRFS_WRITE      4
;355 #define SRFS_LSEEK      5
;356 #define SRFS_FSTAT      6
;357 #define SRFS_UNLINK     7
;358 #define SRFS_RENAME     8
;359 #define SRFS_BYE       99  // close connection to server
;360 
;361 
;362 #define  ShowProcess  (DiagFlags & 0x00000001)
;363 
;364 ulong  srfsIPAddr, srfsAltAddr;
;365 
;366 #define  DEF_SRFS_ADDR   "192.168.4.34"  // icarus
;367 #define  DEF_ALT_ADDR    "192.168.4.33"  // epics3rd
;368 
;369 
;370 #define  MAX_RW_MS     10000   // max ms for r/w opers on shared connection
;371 
;372 
;373 sem_t srfsSem;          // semaphore used to protect SRFS driver's shared d
;374 #define  SRFS_LOCK(secs)  ( srfsGetLock(secs) )
;375 #ifdef SRFS_DEBUG
;378   #define  SRFS_UNLOCK()  ( sem_post(&srfsSem) )
;379 #endif
;380 
;381 
;382 FDEV far *sockFDEV;     // global ftable entry for connection to srfs serve
;383 ulong  lastSendTime;    // time of last xmit on sockFDEV
;384 bool  srfsConOK;
;385 
;386 struct sockaddr_in  srvAddr;  // address of server
;387 
;388 #define  LogErr(str)  
;389    fprintf(DiagLog, "%s(): %.32s: %ls\r\n", __func__, (str), strerror(errNu
;390 
;391 //-------------------------------------------------------------------------
;392 //  func called once by init process to initialize data structures
;393 //-------------------------------------------------------------------------
;394 void srfsInit_()
;395 {
	.defseg	FARCODE
	.seg	FARCODE
$srfsInit_::
	.debug	G $srfsInit_ 34 9
	.debug	{ $srfsInit_ 34 394 @4 @3
;396   BZERO(srfsFile);
	.debug	# 396
	ld	hl,0x36
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_srfsFile
	push	hl
	call	_memset
	add	sp,6
;397 
;398   inet_ptoh(AF_INET, DEF_SRFS_ADDR, &srfsIPAddr);
	.debug	# 398
	ld	hl,_srfsIPAddr
	push	hl
	ld	hl,@1+5
	push	hl
	ld	hl,0x2
	push	hl
	lcall	$inet_ptoh
	add	sp,6
;399   inet_ptoh(AF_INET, DEF_ALT_ADDR, &srfsAltAddr);
	.debug	# 399
	ld	hl,_srfsAltAddr
	push	hl
	ld	hl,@1+18
	push	hl
	ld	hl,0x2
	push	hl
	lcall	$inet_ptoh
	add	sp,6
;400 
;401   sockFDEV = NULL;  srfsConOK = false;
	.debug	# 401
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(_sockFDEV),de
	ld	(_sockFDEV+2),bc
	xor	a,a
	ld	(_srfsConOK),a
;402 
;403   if (sem_init(&srfsSem, 1, 1))  { perror(__func__);  return; }
	.debug	# 403
	ld	hl,0x1
	push	hl
	push	hl
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_init
	add	sp,6
	bool	hl
	jr	z,@5
@6:
	.debug	< @6 @7
	ld	de,word (phys @8)
	ld	c,phys @8 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	jr	@3
@7:
	.debug	>
;404 
;405   registerFileSys(&srfsFSysFuncs);
@5:
	.debug	# 405
	ld	hl,_srfsFSysFuncs
	push	hl
	lcall	$registerFileSys
	pop	de
;406 }
@3:
	.debug	} @3 406

@2	equ	0
	lret
@4	equ	0
;407 
;408 
;409 //-------------------------------------------------------------------------
;410 //  Returns true if provided pointer is a valid, in-use SRFS device.
;411 //  Returns false if is not.
;412 //-------------------------------------------------------------------------
;413 bool  validSRFS(void *ioDev)
;414 {
$validSRFS::
	.debug	G $validSRFS 35 9
	.debug	{ $validSRFS 35 413 @11 @10
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A ioDev 5 36
;415   uint  idx;
	.debug	L idx -2 7
;416 
;417 
;418   do  {
;419     if (!ioDev)  break;
@15:
	.debug	< @15 @16
	.debug	# 419
	ld	hl,(ix+5)
	bool	hl
	jr	z,@13
;420 
;421     if ((idx = SRFS - srfsFile) >= MaxSRFSFiles)  break;
	.debug	# 421
	ld	hl,(ix+5)
	ld	de,_srfsFile
	or	a,a
	sbc	hl,de
	ld	de,0x9
	call	__idiv##
	ld	(ix-2),hl
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	nc,@13
;422 
;423     if (SRFS != srfsFile + idx)  break;
	.debug	# 423
	ld	hl,(ix-2)
	ld	bc,0x9
	ex	de,hl
	mul
	ld	hl,_srfsFile
	add	hl,bc
	ex	de,hl
	ld	hl,(ix+5)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	nz,@13
;424 
;425     if (SRFS->inUse)  return true;
	.debug	# 425
	ld	h,d
	ld	l,e
	ld	a,(hl)
	or	a,a
	jr	z,@20
	ld	a,0x1
	jr	@10
;426   } while(0);
@20:
@16:
	.debug	>
	.debug	# 426
;427 
;428   return false;
@13:
	.debug	# 428
	xor	a,a
;429 }
@10:
	.debug	} @10 429

@9	equ	2
	ld	sp,ix
	pop	ix
	lret
@11	equ	4
;430 
;431 
;432 //-------------------------------------------------------------------------
;433 //  Wait up to the specified number of seconds for the global SRFS lock.
;434 //
;435 //  Returns 0 on success, -1 if an error or timeout
;436 //-------------------------------------------------------------------------
;437 int  srfsGetLock(int secs)
;438 {
$srfsGetLock::
	.debug	G $srfsGetLock 37 9
	.debug	{ $srfsGetLock 37 437 @23 @22
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-8
	.debug	A secs 5 6
;439   struct timespec  srfsTimeout;
	.debug	L srfsTimeout -8 38
;440 
;441 
;442   srfsTimeout.tv_sec = time(NULL) + secs;  srfsTimeout.tv_nsec = 0;
	.debug	# 442
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	ld	hl,(ix+5)
	push	bc
	push	de
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	call	__ladd##
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
	ld	de,0
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;443 
;444   if (sem_timedwait(&srfsSem, &srfsTimeout))  {
	.debug	# 444
	ld	hl,-8+@21
	add	hl,sp
	push	hl
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_timedwait
	add	sp,4
	bool	hl
	jr	z,@24
;445     fprintf(DiagLog, "\n*** Unable to get SRFS lock: %ls ***\n",  //tdebug
@25:
	.debug	< @25 @26
;446             strerror(errno));  //tdebug
	.debug	# 446
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strerror
	pop	hl
	push	bc
	push	de
	ld	de,word (phys @8+10)
	ld	c,phys @8+10 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,10
;447     return -1;
	.debug	# 447
	ld	hl,0xffff
	jr	@22
;448   }
@26:
	.debug	>
;449 
;450   #ifdef SRFS_DEBUG
;455   return 0;
@24:
	.debug	# 455
	bool	hl
	ld	l,h
;456 }
@22:
	.debug	} @22 456

@21	equ	8
	ld	sp,ix
	pop	ix
	lret
@23	equ	10
;457 
;458 #ifdef SRFS_DEBUG
;471 //-------------------------------------------------------------------------
;472 //  Establish a connection to the SRFS server if one not already establishe
;473 //
;474 //  The calling process MUST have the lock for the SRFS driver's shared dat
;475 //
;476 //  On success:
;477 //    - Returns a pointer to the FDEV for the shared socket connection
;478 //
;479 //  On failure (no existing connection and one cannot be established):
;480 //    - Returns NULL
;481 //-------------------------------------------------------------------------
;482 FDEV far *srfsConnect(void)
;483 {
$srfsConnect::
	.debug	G $srfsConnect 39 9
	.debug	{ $srfsConnect 39 482 @29 @28
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-92
;484   fd_t  sockfd;
	.debug	L sockfd -2 6
;485   char  buf[90];
	.debug	L buf -92 42
;486 
;487 
;488 //--- If we already have an open connection, then nothing to do ---
;489   if (sockFDEV)  return sockFDEV;
	.debug	# 489
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	ld	a,e
	or	a,d
	or	a,c
	jp	nz,@28
;490 
;491 //--- Otherwise we try to open a new connection ---
;492   #ifdef SRFS_DEBUG
;496   if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)  {
	.debug	# 496
	bool	hl
	ld	l,h
	push	hl
	ld	l,0x2
	push	hl
	push	hl
	lcall	$socket
	add	sp,6
	ld	(ix-2),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@31
;497     fprintf(DiagLog, "  srfsConnect(): socket() error: %ls\n",
@32:
	.debug	< @32 @33
;498                     strerror(errno) );
	.debug	# 498
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strerror
	pop	hl
	push	bc
	push	de
	ld	de,word (phys @8+49)
	ld	c,phys @8+49 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,10
;499     return NULL;
	.debug	# 499
	jp	@318
;500   }
@33:
	.debug	>
;501 
;502 //--- set addr info for remote end of connection ---
;503   srvAddr.sin_family = AF_INET;
@31:
	.debug	# 503
	ld	hl,0x2
	ld	(_srvAddr),hl
;504   srvAddr.sin_port   = htons(SRFS_PORT);
	.debug	# 504
	ld	hl,0x7d2
	ld	a,l
	ld	l,h
	ld	h,a
	ld	(_srvAddr+2),hl
;505   srvAddr.sin_addr   = htonl(srfsIPAddr);
	.debug	# 505
	ld	de,(_srfsIPAddr)
	ld	bc,(_srfsIPAddr+2)
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	ld	(_srvAddr+4),de
	ld	(_srvAddr+6),bc
;506 
;507 //--- and establish a connection to the SRFS server ---
;508   if (connect(sockfd, (struct sockaddr *) &srvAddr,
;509               sizeof(struct sockaddr_in) ) < 0)  {
	.debug	# 509
	ld	hl,0xe
	push	hl
	ld	hl,_srvAddr
	push	hl
	ld	hl,(ix-2)
	push	hl
	lcall	$connect
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@34
;510     fprintf(DiagLog, "  srfsConnect(): connect() error: [%d] %ls\n",
@35:
	.debug	< @35 @36
;511                     sockfd, strerror(errno) );
	.debug	# 511
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strerror
	pop	hl
	push	bc
	push	de
	ld	hl,(ix-2)
	push	hl
	ld	de,word (phys @8+87)
	ld	c,phys @8+87 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,12
;512     close(sockfd);  return NULL;
	.debug	# 512
	ld	hl,(ix-2)
	push	hl
	lcall	$close
	pop	de
@318:
	ld	de,0
	ld	c,e
	ld	b,e
	jp	@28
;513   }
@36:
	.debug	>
;514 
;515   KLOCK();  sockFDEV = _fdToOpenFile_(sockfd);  KUNLOCK();
@34:
	.debug	# 515
	call	_KLOCK
	ld	hl,(ix-2)
	push	hl
	lcall	$_fdToOpenFile_
	pop	hl
	ld	(_sockFDEV),de
	ld	(_sockFDEV+2),bc
	call	_KUNLOCK
;516 
;517   (sockFDEV->ioFuncs->descr)(sockFDEV->ioDev, buf, sizeof(buf));
	.debug	# 517
	ld	hl,0x5a
	push	hl
	ld	hl,-90+@27
	add	hl,sp
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	push	de
	ex	de,hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	bc,de
	ld	de,0xc
	ex	de,hl
	add	hl,bc
	ld	hl,(hl)
	ex	de,hl
	call	__callde##
	add	sp,6
;518   fprintf(DiagLog, "\r\n"
;519                    "*** Connected to SRFS server ***\r\n"
;520                    "  %.128s\r\n\n", buf);
	.debug	# 518
	ld	hl,-92+@27
	add	hl,sp
	push	hl
	ld	de,word (phys @8+131)
	ld	c,phys @8+131 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,8
;521 
;522 /*
;523    So we have a new connection, but the process-specific fd for it is prett
;524    useless because it IS process-specific.  So we just free it up and use
;525    the FDEV ptr which any task can use (as long as it has the global SRFS
;526    lock to avoid mixing messages for different tasks).
;527 */
;528   sockFDEV->shared = true;  // indicates this is SUPPOSED to be an orphan F
	.debug	# 528
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	ld	de,0x26
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	a,0x1
	call	__farst1##
;529   *((FDEV far **)current->FDSet + sockfd) = NULL;
	.debug	# 529
	ld	hl,(_current)
	ld	de,0xf
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	add	hl,de
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;530 
;531   srfsConOK = true;
	.debug	# 531
	ld	a,0x1
	ld	(_srfsConOK),a
;532 
;533   return sockFDEV;
	.debug	# 533
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
;534 }
@28:
	.debug	} @28 534

@27	equ	92
	ld	sp,ix
	pop	ix
	lret
@29	equ	94
;535 
;536 
;537 //-------------------------------------------------------------------------
;538 //  close the shared connection to the server
;539 //-------------------------------------------------------------------------
;540 void srfsDisconnect(void)
;541 {
$srfsDisconnect::
	.debug	G $srfsDisconnect 43 9
	.debug	{ $srfsDisconnect 43 540 @39 @38
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-92
;542   char  *ptr;
	.debug	L ptr -2 44
;543   char  buf[90];
	.debug	L buf -92 42
;544 
;545 
;546   KLOCK();
	.debug	# 546
	call	_KLOCK
;547 
;548   if (_validFDEV_(sockFDEV))  {
	.debug	# 548
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$_validFDEV_
	add	sp,4
	or	a,a
	jp	z,@40
;549     if (srfsConOK)  {
@41:
	.debug	< @41 @42
	.debug	# 549
	ld	a,(_srfsConOK)
	or	a,a
	jr	z,@43
;550       ptr = buf;  addShort(&ptr, SRFS_BYE);  addShort(&ptr, 0);
@44:
	.debug	< @44 @45
	.debug	# 550
	ld	hl,-92+@37
	add	hl,sp
	ld	(ix-2),hl
	ld	hl,0x63
	push	hl
	ld	hl,0+@37
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	bool	hl
	ld	l,h
	push	hl
	ld	hl,0+@37
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;551       sendCmd(buf, 4, NULL, 0);
	.debug	# 551
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	l,0x4
	push	hl
	ld	hl,-86+@37
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
;552     }
@45:
	.debug	>
;553     sockFDEV->openCount = 0;  // frees FDEV
@43:
	.debug	# 553
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	bool	hl
	ld	l,h
	call	__farst2##
;554     (sockFDEV->ioFuncs->descr)(sockFDEV->ioDev, buf, sizeof(buf));
	.debug	# 554
	ld	hl,0x5a
	push	hl
	ld	hl,-90+@37
	add	hl,sp
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	push	de
	ex	de,hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	bc,de
	ld	de,0xc
	ex	de,hl
	add	hl,bc
	ld	hl,(hl)
	ex	de,hl
	call	__callde##
	add	sp,6
;555     fprintf(DiagLog, "\r\n"
;556                      "*** Closing con to SRFS server ***\r\n"
;557                      "  %.128s\r\n\n", buf);
	.debug	# 555
	ld	hl,-92+@37
	add	hl,sp
	push	hl
	ld	de,word (phys @8+179)
	ld	c,phys @8+179 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,8
;558     (sockFDEV->ioFuncs->close)(sockFDEV->ioDev);
	.debug	# 558
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	push	de
	ex	de,hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	bc,hl
	ld	hl,0x4
	add	hl,de
	ld	hl,(hl)
	ld	de,bc
	ex	de,hl
	call	__callde##
	pop	de
;559   }
@42:
	.debug	>
;560 
;561   srfsConOK = false;  sockFDEV = NULL;
@40:
	.debug	# 561
	xor	a,a
	ld	(_srfsConOK),a
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(_sockFDEV),de
	ld	(_sockFDEV+2),bc
;562 
;563   KUNLOCK();
	.debug	# 563
	call	_KUNLOCK
;564 }
@38:
	.debug	} @38 564

@37	equ	92
	ld	sp,ix
	pop	ix
	lret
@39	equ	94
;565 
;566 //-------------------------------------------------------------------------
;567 //  The connection to the SRFS server failed, which means the connection fo
;568 //  all the currently open files is no longer usable so we close the fd for
;569 //  the shared connection for each one and mark them as failed.
;570 //
;571 //  The caller must have the global lock for the SRFS driver's shared data
;572 //-------------------------------------------------------------------------
;573 void _srfsConFailed_(void)
;574 {
$_srfsConFailed_::
	.debug	G $_srfsConFailed_ 43 9
	.debug	{ $_srfsConFailed_ 43 573 @48 @47
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-6
;575   uint  ui;
	.debug	L ui -2 7
;576   int  errNum;
	.debug	L errNum -4 6
;577   SRFS_File  *srfs;
	.debug	L srfs -6 45
;578 
;579 
;580   srfsConOK = false;
	.debug	# 580
	xor	a,a
	ld	(_srfsConOK),a
;581 
;582   fprintf(DiagLog, "*** Con to SRFS server failed ***\n");
	.debug	# 582
	ld	de,word (phys @8+229)
	ld	c,phys @8+229 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,6
;583 
;584   errNum = (errno ? errno : EIO);
	.debug	# 584
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	bool	hl
	jr	z,@49
	ld	hl,(_current)
	add	hl,de
	ld	hl,(hl)
	jr	@50
@49:
	ld	hl,0x5
@50:
	ld	(ix-4),hl
;585 
;586 /*
;587   Since any failure on the shared connection means none of the files
;588   currently open on it can reliably use the connection to continue
;589   interacting with the server, we set the error state for all of them.
;590 */
;591   for (ui=0,srfs=srfsFile; ui<MaxSRFSFiles; ++ui,++srfs)
@51:
	.debug	< @51 @52
	.debug	# 591
	bool	hl
	ld	l,h
	ld	(ix-2),hl
	ld	hl,_srfsFile
	ld	(ix-6),hl
@53:
	ld	hl,(ix-2)
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	nc,@54
;592     if (srfs->inUse)  srfs->errNum = errNum;
	.debug	# 592
	ld	hl,(ix-6)
	ld	a,(hl)
	or	a,a
	jr	z,@56
	inc	hl
	ex	de,hl
	ld	hl,(ix-4)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;593 
;594 //--- free up the FDEV and close the (failed) shared connection ---
;595   srfsDisconnect();
@56:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	ld	hl,(ix-6)
	ld	de,0x9
	add	hl,de
	ld	(ix-6),hl
	jr	@53
@54:
@52:
	.debug	>
	.debug	# 595
	lcall	$srfsDisconnect
;596 }
@47:
	.debug	} @47 596

@46	equ	6
	ld	sp,ix
	pop	ix
	lret
@48	equ	8
;597 
;598 
;599 //-------------------------------------------------------------------------
;600 //  Send a command and (optional) data to the SRFS server
;601 //
;602 //  The calling process MUST have the lock for the SRFS driver's shared dat
;603 //
;604 //  Parameters:
;605 //    - sockfd:    Process-specific fd for the shared connection to the ser
;606 //    - timeoutMS: Max time (in ms) for bytes to be accepted for transmissi
;607 //                 to the server (no guarantee they have been or will be
;608 //                 transmitted to or received by the server).
;609 //    - cmd:       Bytes that contain the command to the server
;610 //    - cmdLen:    # of cmd bytes
;611 //    - data:      Parameter values for the command
;612 //    - dataLen:   # of data bytes
;613 //
;614 //  On success (bytes accepted for transmission):
;615 //    - returns 0
;616 //
;617 //  On failure (unable to queue all the bytes for transmission):
;618 //    - returns -1
;619 //    - all open SRFS files marked as failed
;620 //
;621 //  NOTE: All files use a single connection, and each command and its respo
;622 //        constitutes a single (usually brief) conversation with the server
;623 //        These conversations are atomic (i.e. they cannot be concurrent),
;624 //        so any failure in any conversation means the connection itself ha
;625 //        failed, since any attempt to use the connection from that time on
;626 //        could result in the command and/or data bytes from the failed
;627 //        conversation getting mixed into subsequent ones.
;628 //-------------------------------------------------------------------------
;629 int sendCmd(const char *cmd, int cmdLen, const char *data, int dataLen )
;630 {
$sendCmd::
	.debug	G $sendCmd 47 9
	.debug	{ $sendCmd 47 629 @59 @58
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-1
	.debug	A cmd 5 44
	.debug	A cmdLen 7 6
	.debug	A data 9 44
	.debug	A dataLen 11 6
;631   bool  writeOK = false;
	.debug	L writeOK -1 3
	.debug	# 631
	xor	a,a
	ld	(ix-1),a
	.seg	FARCODE
;632 
;633   #ifdef SRFS_DEBUG
;640   do  {
;641     if (writeFDEV(sockFDEV, (void *)cmd, cmdLen, MAX_RW_MS) != cmdLen)
@63:
	.debug	< @63 @64
	.debug	# 641
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$writeFDEV
	add	sp,12
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	nz,@61
;642       break;
	.debug	# 642
;643     if (dataLen)
	.debug	# 643
	ld	hl,(ix+11)
	bool	hl
	jr	z,@66
;644       if (writeFDEV(sockFDEV, (void *)data, dataLen, MAX_RW_MS) != dataLen)
	.debug	# 644
	ld	de,0x2710
	ld	b,h
	ld	c,h
	push	bc
	push	de
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$writeFDEV
	add	sp,12
	ex	de,hl
	ld	hl,(ix+11)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	nz,@61
;645         break;
	.debug	# 645
;646     writeOK = true;
@66:
	.debug	# 646
	ld	(ix-1),0x1
;647   } while(0);
@64:
	.debug	>
	.debug	# 647
;648 
;649 //--- communications failure ---
;650   if (!writeOK)  {
@61:
	.debug	# 650
	ld	a,(ix-1)
	or	a,a
	jp	nz,@68
;651     if (errno == EAGAIN)  errno = ETIMEDOUT;
@69:
	.debug	< @69 @70
	.debug	# 651
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0xb
	or	a,a
	sbc	hl,de
	jr	nz,@71
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x6e
	ld	(hl),e
	inc	hl
	ld	(hl),d
;652     fprintf(DiagLog, "*** SRFS: sendCmd(%u): %ls ***\n",
@71:
;653             ntohs(*(const short *)cmd), strerror(errno) );
	.debug	# 653
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strerror
	pop	hl
	push	bc
	push	de
	ld	hl,(ix+5)
	ld	hl,(hl)
	ld	a,l
	ld	l,h
	ld	h,a
	push	hl
	ld	de,word (phys @8+264)
	ld	c,phys @8+264 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,12
;654     _srfsConFailed_();  return -1;
	.debug	# 654
	lcall	$_srfsConFailed_
	ld	hl,0xffff
	jp	@58
;655   }
@70:
	.debug	>
;656 
;657   lastSendTime = GetMS();
@68:
	.debug	# 657
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	(_lastSendTime),de
	ld	(_lastSendTime+2),bc
;658 
;659   return 0;
	.debug	# 659
	bool	hl
	ld	l,h
;660 }
@58:
	.debug	} @58 660

@57	equ	1
	ld	sp,ix
	pop	ix
	lret
@59	equ	3
;661 
;662 
;663 //-------------------------------------------------------------------------
;664 //  Read the response to the last cmd sent to the SRFS server
;665 //
;666 //  The calling process MUST have the lock for the SRFS driver's shared dat
;667 //
;668 //  On success:
;669 //    - returns 0
;670 //    - *stat = return value from requested func on the server
;671 //    - *errnum = errno value from requested func on server
;672 //    - *dataLen = length of any data returned by request
;673 //    - dataBuf contains any data returned by the request
;674 //
;675 //  On failure:
;676 //    - returns -1
;677 //    - *stat = -1
;678 //    - *errnum = local error
;679 //    - *dataLen = 0
;680 //    - dataBuf untouched
;681 //    - all open SRFS files marked as failed
;682 //
;683 //  NOTE: All files use a single connection, and each command and its respo
;684 //        constitutes a single (usually brief) conversation with the server
;685 //        These conversations are atomic (i.e. they cannot be concurrent),
;686 //        so any failure in any conversation means the connection itself ha
;687 //        failed, since any attempt to use the connection from that time on
;688 //        could result in the command and/or data bytes from the failed
;689 //        conversation getting mixed into subsequent ones.
;690 //-------------------------------------------------------------------------
;691 int readResp(int *stat, int *errnum, int *dataLen, char *dataBuf, int maxLe
;692 {
$readResp::
	.debug	G $readResp 48 9
	.debug	{ $readResp 48 691 @74 @73
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-8
	.debug	A stat 5 49
	.debug	A errnum 7 49
	.debug	A dataLen 9 49
	.debug	A dataBuf 11 44
	.debug	A maxLen 13 6
;693   char  buf[6];
	.debug	L buf -6 50
;694   char  *ptr;
	.debug	L ptr -8 44
;695 
;696 
;697   #ifdef SRFS_DEBUG
;703   if (readFDEV(sockFDEV, buf, 6, MAX_RW_MS) < 6)  {
	.debug	# 703
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	ld	hl,0x6
	push	hl
	ld	hl,0+@72
	add	hl,sp
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$readFDEV
	add	sp,12
	ld	de,0x8006
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@75
;704     if (errno == EAGAIN)  errno = ETIMEDOUT;
@76:
	.debug	< @76 @77
	.debug	# 704
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0xb
	or	a,a
	sbc	hl,de
	jr	nz,@78
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x6e
	ld	(hl),e
	inc	hl
	ld	(hl),d
;705     fprintf(DiagLog, "*** SRFS: readResp(): %ls\n", strerror(errno));
@78:
	.debug	# 705
	jp	@87
;706     _srfsConFailed_();
	.debug	# 706
;707     *stat = -1;  *errnum = errno;  *dataLen = 0;  return -1;
	.debug	# 707
;708   }
@77:
	.debug	>
;709 
;710   ptr = buf;
@75:
	.debug	# 710
	ld	hl,-6+@72
	add	hl,sp
	ld	(ix-8),hl
;711   *stat = getShort(&ptr);  *errnum = getShort(&ptr);  *dataLen = getShort(&
	.debug	# 711
	ld	hl,-8+@72
	add	hl,sp
	push	hl
	lcall	$getShort
	pop	de
	ex	de,hl
	ld	hl,(ix+5)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,-8+@72
	add	hl,sp
	push	hl
	lcall	$getShort
	pop	de
	ex	de,hl
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,-8+@72
	add	hl,sp
	push	hl
	lcall	$getShort
	pop	de
	ex	de,hl
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
;712 
;713   #ifdef SRFS_DEBUG
;719   if (*dataLen > maxLen)  {
	.debug	# 719
	ld	hl,(ix+9)
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix+13)
	ex	de,hl
	call	__icmp##
	jr	c,@79
	jr	z,@79
;720     if (ShowErrors)
@80:
	.debug	< @80 @81
	.debug	# 720
	ld	hl,(_current)
	ld	de,0xae
	add	hl,de
	ld	hl,(hl)
	ld	de,0x100
	and	hl,de
	jr	z,@82
;721       fprintf(DiagLog, "*** SRFS: readResp(): resp too large\r\n");
	.debug	# 721
	ld	de,word (phys @8+323)
	ld	c,phys @8+323 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,6
;722     _srfsConFailed_();
@82:
	.debug	# 722
	lcall	$_srfsConFailed_
;723     *stat = -1;  *errnum = ENOMEM;  *dataLen = 0;  return -1;
	.debug	# 723
	ld	hl,0xffff
	ex	de,hl
	ld	hl,(ix+5)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,0xc
	jp	@317
;724   }
@81:
	.debug	>
;725 
;726   if (*dataLen > 0)
@79:
	.debug	# 726
	ld	hl,(ix+9)
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@83
	jp	z,@83
;727     if (readFDEV(sockFDEV, dataBuf, *dataLen, MAX_RW_MS) != *dataLen)  {
	.debug	# 727
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix+11)
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$readFDEV
	add	sp,12
	ex	de,hl
	ld	hl,(ix+9)
	ld	hl,(hl)
	or	a,a
	sbc	hl,de
	jr	z,@84
;728       if (errno == EAGAIN)  errno = ETIMEDOUT;
@85:
	.debug	< @85 @86
	.debug	# 728
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0xb
	or	a,a
	sbc	hl,de
	jr	nz,@87
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x6e
	ld	(hl),e
	inc	hl
	ld	(hl),d
;729       fprintf(DiagLog, "*** SRFS: readResp(): %ls\n", strerror(errno));
@87:
	.debug	# 729
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strerror
	pop	hl
	push	bc
	push	de
	ld	de,word (phys @8+296)
	ld	c,phys @8+296 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,10
;730       _srfsConFailed_();
	.debug	# 730
	lcall	$_srfsConFailed_
;731       *stat = -1;  *errnum = errno;  *dataLen = 0;  return -1;
	.debug	# 731
	ld	hl,0xffff
	ex	de,hl
	ld	hl,(ix+5)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
@317:
	ex	de,hl
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	bool	hl
	ld	l,h
	ex	de,hl
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,0xffff
	jr	@73
;732     }
@86:
	.debug	>
;733 
;734   return 0;
@84:
@83:
	.debug	# 734
	bool	hl
	ld	l,h
;735 }
@73:
	.debug	} @73 735

@72	equ	8
	ld	sp,ix
	pop	ix
	lret
@74	equ	10
;736 
;737 
;738 //=========================================================================
;739 //  FileSysFuncs that do not pass an ioDev object
;740 //=========================================================================
;741 
;742 //-------------------------------------------------------------------------
;743 //  Open a a file on the SRFS server.  Note that, while the socket connecti
;744 //  to the server is shared by all open SRFS driver files, we keep a copy o
;745 //  the socket fd last used by each file so the next call to an SRFS driver
;746 //  function knows if the server fd applies to the current connection or a
;747 //  previous one.  In the later case, any further operations on the DEBROS 
;748 //  for the file will fail (since the only reason the socket connection for
;749 //  it would be closed is that an error occured).
;750 //-------------------------------------------------------------------------
;751 fd_t _srfsOpen_(const char *path, int flags, int mode)
;752 {
	.align	4
$$_srfsOpen_::
	ex	(sp),iy
$_srfsOpen_::
	.debug	G $_srfsOpen_ 51 9
	.debug	{ $_srfsOpen_ 51 751 @90 @89
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-32
	.debug	A path 5 44
	.debug	A flags 7 6
	.debug	A mode 9 6
;753   int  i, fd, nameLen, stat, errNum, respLen;
	.debug	L i -2 6
	.debug	L fd -4 6
	.debug	L nameLen -6 6
	.debug	L stat -8 6
	.debug	L errNum -10 6
	.debug	L respLen -12 6
;754   SRFS_File  *srfs;
	.debug	L srfs -14 45
;755   char  *ptr;
	.debug	L ptr -16 44
;756   char  buf[16];
	.debug	L buf -32 52
;757 
;758 
;759   #ifdef SRFS_DEBUG
;764 //--- ignore names that do not start with "/srfs/" ---
;765   if (strncmp(path, "/srfs/", 6))  { errno = ENXIO;  return -1; }
	.debug	# 765
	ld	hl,0x6
	push	hl
	ld	hl,@1+31
	push	hl
	ld	hl,(ix+5)
	push	hl
	call	_strncmp
	add	sp,6
	bool	hl
	jr	z,@91
@92:
	.debug	< @92 @93
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x6
	jp	@314
@93:
	.debug	>
;766   path += 6;  // skip over "/srfs/" portion
@91:
	.debug	# 766
	ld	hl,(ix+5)
	ld	de,0x6
	add	hl,de
	ld	(ix+5),hl
;767 
;768 //--- get lock for the SRFS driver's shared data ---
;769   if (SRFS_LOCK(30))  return -1;
	.debug	# 769
	ld	hl,0x1e
	push	hl
	lcall	$srfsGetLock
	pop	de
	bool	hl
	jp	nz,@313
;770 
;771 //--- locate an unused entry in the list ---
;772   for (i=0; i<MaxSRFSFiles; ++i)  if (! srfsFile[i].inUse)  break;
@95:
	.debug	< @95 @96
	.debug	# 772
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@97:
	ld	hl,(ix-2)
	ld	de,0x8006
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@98
	ld	hl,(ix-2)
	ld	bc,0x9
	ex	de,hl
	mul
	ld	hl,_srfsFile
	add	hl,bc
	ld	a,(hl)
	or	a,a
	jr	z,@98
;773   if (i == MaxSRFSFiles)  { SRFS_UNLOCK();  ERRNO = ENFILE;  return -1; }
	ld	h,d
	ld	l,e
	inc	hl
	ld	(ix-2),hl
	jr	@97
@98:
@96:
	.debug	>
	.debug	# 773
	ld	hl,(ix-2)
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	nz,@101
@102:
	.debug	< @102 @103
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x17
	jp	@314
@103:
	.debug	>
;774   srfs = srfsFile + i;  srfs->name = srfsFileName[(uint)i];
@101:
	.debug	# 774
	ld	hl,(ix-2)
	ld	bc,0x9
	ex	de,hl
	mul
	ld	hl,_srfsFile
	add	hl,bc
	ld	(ix-14),hl
	ld	h,d
	ld	l,e
	ld	bc,0
	push	bc
	push	de
	ld	de,0x18
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _srfsFileName)
	ld	c,phys _srfsFileName >> 16
	call	__ladd##
	ld	hl,(ix-14)
	push	de
	ld	de,0x5
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;775 
;776 //--- make sure the shared connection to the server is open ---
;777   if (! srfsConnect())  { SRFS_UNLOCK();  return -1; }
	.debug	# 777
	lcall	$srfsConnect
	ld	a,e
	or	a,d
	or	a,c
	jr	nz,@104
@105:
	.debug	< @105 @106
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	jp	@316
@106:
	.debug	>
;778 
;779   KLOCK();
@104:
	.debug	# 779
	call	_KLOCK
;780 
;781 //--- alloc a process fd and global FDEV for the SRFS driver's structure --
;782   if ((fd = _allocFile_(3, &srfsIOFuncs, srfs)) < 0)  return -1;
	.debug	# 782
	ld	hl,(ix-14)
	push	hl
	ld	hl,_srfsIOFuncs
	push	hl
	ld	hl,0x3
	push	hl
	lcall	$_allocFile_
	add	sp,6
	ld	(ix-4),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@313
;783 
;784   srfs->inUse = true;  srfs->remfd = -1;  srfs->errNum = 0;
	.debug	# 784
	ld	hl,(ix-14)
	ld	a,0x1
	ld	(hl),a
	inc	hl
	inc	hl
	inc	hl
	ld	de,0xffff
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix-14)
	inc	hl
	ld	de,0
	ld	(hl),e
	inc	hl
	ld	(hl),d
;785   fstrcpy(srfs->name, path);
	.debug	# 785
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-14)
	ld	de,0x5
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	call	_fstrcpy
	add	sp,8
;786 
;787   KUNLOCK();
	.debug	# 787
	call	_KUNLOCK
;788 
;789 //cur method for telling srfs server that we will send keepalive packets
;790 #define SRFS_O_TIMEOUT   0x8000   // abort if idle for too long
;791 
;792 //--- send OPEN request ---
;793   ptr = buf;  nameLen = strlen(path);
	.debug	# 793
	ld	hl,-32+@88
	add	hl,sp
	ld	(ix-16),hl
	ld	hl,(ix+5)
	push	hl
	call	_strlen
	pop	de
	ld	(ix-6),hl
;794   addShort(&ptr, SRFS_OPEN);  addShort(&ptr, 4+nameLen);
	.debug	# 794
	ld	hl,0x1
	push	hl
	ld	hl,-14+@88
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix-6)
	ld	de,0x4
	add	hl,de
	push	hl
	ld	hl,-14+@88
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;795   addShort(&ptr, flags | SRFS_O_TIMEOUT);  addShort(&ptr, mode);
	.debug	# 795
	ld	hl,(ix+7)
	set	7,h
	push	hl
	ld	hl,-14+@88
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix+9)
	push	hl
	ld	hl,-14+@88
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;796 
;797   if (sendCmd(buf, 8, (char *)path, nameLen))  {
	.debug	# 797
	ld	hl,(ix-6)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x8
	push	hl
	ld	hl,-26+@88
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
	bool	hl
	jr	nz,@315
;798     SRFS_UNLOCK();  close(fd);  return -1;  }
@109:
	.debug	< @109 @110
	.debug	# 798
@110:
	.debug	>
;799 
;800 //--- wait for response to OPEN request ---
;801   if (readResp(&stat, &errNum, &respLen, buf, sizeof(buf)))  {
	.debug	# 801
	ld	hl,0x10
	push	hl
	ld	hl,-30+@88
	add	hl,sp
	push	hl
	ld	hl,-8+@88
	add	hl,sp
	push	hl
	ld	hl,-4+@88
	add	hl,sp
	push	hl
	ld	hl,0+@88
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jr	z,@111
;802     SRFS_UNLOCK();  close(fd);  return -1; }
@112:
	.debug	< @112 @113
	.debug	# 802
@315:
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-4)
	push	hl
	lcall	$close
@316:
	pop	de
	jr	@313
@113:
	.debug	>
;803 
;804 //--- server was unable to open the specified file ---
;805   if (stat < 0)  { SRFS_UNLOCK();  close(fd);  errno = errNum;  return -1; 
@111:
	.debug	# 805
	ld	hl,(ix-8)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@114
@115:
	.debug	< @115 @116
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-4)
	push	hl
	lcall	$close
	pop	de
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-10)
	ex	de,hl
@314:
	ld	(hl),e
	inc	hl
	ld	(hl),d
@313:
	ld	hl,0xffff
	jr	@89
@116:
	.debug	>
;806 
;807 //--- file was opened: save the remote fd provided for the file ---
;808   srfs->remfd = stat;
@114:
	.debug	# 808
	ld	hl,(ix-14)
	inc	hl
	inc	hl
	inc	hl
	ex	de,hl
	ld	hl,(ix-8)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;809 
;810   #ifdef SRFS_DEBUG
;815 //--- everything OK - return the local fd assigned for the remote file ---
;816   SRFS_UNLOCK();  return fd;
	.debug	# 816
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-4)
;817 }
@89:
	.debug	} @89 817

@88	equ	32
	ld	sp,ix
	pop	ix
	lret
@90	equ	34
;818 
;819 
;820 //-------------------------------------------------------------------------
;821 //  func called each kernel tick to provide the driver with CPU cycles
;822 //  Should ONLY be called by the kernel task!
;823 //-------------------------------------------------------------------------
;824 void _srfsTick_(void)
;825 {
	.align	4
$$_srfsTick_::
	ex	(sp),iy
$_srfsTick_::
	.debug	G $_srfsTick_ 43 9
	.debug	{ $_srfsTick_ 43 824 @119 @118
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-10
;826   ushort  u;
	.debug	L u -2 5
;827   char  *ptr;
	.debug	L ptr -4 44
;828   SRFS_File  *ioDev;
	.debug	L ioDev -6 45
;829   char  buf[4];
	.debug	L buf -10 53
;830   static uint  tickCount;
	.bseg
@120:
	.ds	2
	.debug	S tickCount @120 7
	.seg	FARCODE
;831 
;832 
;833   if (current != kernelTask)  return;
	.debug	# 833
	ld	hl,(_kernelTask)
	ld	de,(_current)
	or	a,a
	sbc	hl,de
	jp	nz,@118
;834 
;835 //--- should we close the shared connection in preperation for a shutdown? 
;836   if (doShutdown AND sockFDEV)  {
	.debug	# 836
	ld	a,(_doShutdown)
	or	a,a
	jr	z,@123
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@123
;837     for (u=0,ioDev=srfsFile; u<MaxSRFSFiles; ++u,++ioDev)
@124:
	.debug	< @124 @125
@126:
	.debug	< @126 @127
	.debug	# 837
	bool	hl
	ld	l,h
	ld	(ix-2),hl
	ld	hl,_srfsFile
	ld	(ix-6),hl
@128:
	ld	hl,(ix-2)
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	nc,@129
;838       if (ioDev->inUse)  break;
	.debug	# 838
	ld	hl,(ix-6)
	ld	a,(hl)
	or	a,a
	jr	nz,@129
;839     if (u == MaxSRFSFiles)  srfsDisconnect();  // not in use, safe to close
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	ld	hl,(ix-6)
	ld	e,0x9
	add	hl,de
	ld	(ix-6),hl
	jr	@128
@129:
@127:
	.debug	>
	.debug	# 839
	ld	hl,(ix-2)
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	nz,@132
	lcall	$srfsDisconnect
;840   }
@132:
@125:
	.debug	>
;841 
;842 
;843   if ((++tickCount & 0x3F) == 0)  return;  // don't need to check very ofte
@123:
	.debug	# 843
	ld	hl,(@120)
	inc	hl
	ld	(@120),hl
	ld	de,0x3f
	and	hl,de
	jp	z,@118
;844 
;845 //===== Send a "keepalive" packet if the connection to the SRFS server ====
;846 //===== is open, not busy, and has been idle for at least 10 seconds =====
;847 
;848 //--- If another process already has the lock for the shared data, then ---
;849 //--- a keepalive packet isn't needed (since the link is in use). ---
;850 //--- Besides, we CANNOT allow the kernel task to block indefinitely! ---
;851   if (sem_trywait(&srfsSem))  return;
	.debug	# 851
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_trywait
	pop	de
	bool	hl
	jp	nz,@118
;852 
;853 //--- If we have an open connection and it has been idle, send a KEEPALIVE 
;854 //--- packet to the server so it knows we are still alive and well ---
;855   if (sockFDEV AND  (msSince(lastSendTime) >= 10000))  {
	.debug	# 855
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	ld	a,e
	or	a,d
	or	a,c
	jp	z,@136
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	de,(_lastSendTime)
	ld	bc,(_lastSendTime+2)
	call	__lsub##
	push	bc
	push	de
	ld	de,0x2710
	ld	bc,0
	call	__lsub##
	jr	c,@136
;856   //--- send KEEPALIVE "cmd" (no response sent by server) ---
;857     ptr = buf;  addShort(&ptr, SRFS_KEEPALIVE);  addShort(&ptr, 0);
@137:
	.debug	< @137 @138
	.debug	# 857
	ld	hl,-10+@117
	add	hl,sp
	ld	(ix-4),hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-2+@117
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-2+@117
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;858     sendCmd(buf, 4, NULL, 0);
	.debug	# 858
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	l,0x4
	push	hl
	ld	hl,-4+@117
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
;859   }
@138:
	.debug	>
;860 
;861   SRFS_UNLOCK();  return;
@136:
	.debug	# 861
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
;862 }
@118:
	.debug	} @118 862

@117	equ	10
	ld	sp,ix
	pop	ix
	lret
@119	equ	12
;863 
;864 
;865 //-------------------------------------------------------------------------
;866 //  Return the state of the specified ioDev entry or NULL if not a valid
;867 //  entry #.
;868 //
;869 //  For valid entrys, the entryState struct is filled in
;870 //-------------------------------------------------------------------------
;871 void *_srfsEntry_(uint idx, EntryState *entryState)
;872 {
	.align	4
$$_srfsEntry_::
	ex	(sp),iy
$_srfsEntry_::
	.debug	G $_srfsEntry_ 54 9
	.debug	{ $_srfsEntry_ 54 871 @141 @140
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A idx 5 7
	.debug	A entryState 7 55
;873   SRFS_File  *srfs;
	.debug	L srfs -2 45
;874 
;875   if (idx >= MaxSRFSFiles)  return NULL;
	.debug	# 875
	ld	hl,(ix+5)
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	c,@142
	bool	hl
	ld	l,h
	jr	@140
;876 
;877   srfs = srfsFile + idx;
@142:
	.debug	# 877
	ld	hl,(ix+5)
	ld	bc,0x9
	ex	de,hl
	mul
	ld	hl,_srfsFile
	add	hl,bc
	ld	(ix-2),hl
;878 
;879   entryState->openCount = (srfs->inUse ? 1 : 0);
	.debug	# 879
	ld	a,(hl)
	or	a,a
	jr	z,@143
	ld	hl,0x1
	jr	@144
@143:
	bool	hl
	ld	l,h
@144:
	ex	de,hl
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
;880   entryState->unused = (entryState->openCount == 0);
	.debug	# 880
	ld	hl,(ix+7)
	ld	hl,(hl)
	bool	hl
	ld	l,h
	jr	nz,@145
	inc	l
@145:
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	inc	de
	inc	de
	ex	de,hl
	ld	(hl),e
;881 
;882   return srfs;
	.debug	# 882
	ld	hl,(ix-2)
;883 }
@140:
	.debug	} @140 883

@139	equ	2
	ld	sp,ix
	pop	ix
	lret
@141	equ	4
;884 
;885 
;886 //-------------------------------------------------------------------------
;887 //  Attempt to delete a remote file via the SRFS protocol
;888 //
;889 //  Should only be called from the fcntl.c/unlink() function!
;890 //
;891 //  RETURN VALUEs:
;892 //  - if successful: 0
;893 //  - if an error: -1 and ERRNO set accordingly
;894 //-------------------------------------------------------------------------
;895 int _srfsUnlink_(const char *path)
;896 {
	.align	4
$$_srfsUnlink_::
	ex	(sp),iy
$_srfsUnlink_::
	.debug	G $_srfsUnlink_ 57 9
	.debug	{ $_srfsUnlink_ 57 895 @148 @147
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-14
	.debug	A path 5 44
;897   char  localBuf[4];
	.debug	L localBuf -4 53
;898   int   stat, respLen, errNum, pathLen;
	.debug	L stat -6 6
	.debug	L respLen -8 6
	.debug	L errNum -10 6
	.debug	L pathLen -12 6
;899   char  *ptr;
	.debug	L ptr -14 44
;900 
;901 
;902   #ifdef SRFS_DEBUG
;906 //--- ignore names that do not start with "/srfs/" ---
;907   if (strncmp(path, "/srfs/", 6))  { errno = ENXIO;  return -1; }
	.debug	# 907
	ld	hl,0x6
	push	hl
	ld	hl,@1+31
	push	hl
	ld	hl,(ix+5)
	push	hl
	call	_strncmp
	add	sp,6
	bool	hl
	jr	z,@149
@150:
	.debug	< @150 @151
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x6
	ld	(hl),e
	inc	hl
	ld	(hl),d
	jr	@312
@151:
	.debug	>
;908   path += 6;  // skip over "/srfs/" portion
@149:
	.debug	# 908
	ld	hl,(ix+5)
	ld	de,0x6
	add	hl,de
	ld	(ix+5),hl
;909 
;910 //--- get lock for the SRFS driver's shared data ---
;911   if (SRFS_LOCK(30))  return -1;
	.debug	# 911
	ld	hl,0x1e
	push	hl
	lcall	$srfsGetLock
	pop	de
	bool	hl
	jr	nz,@312
;912 
;913 //--- make sure the shared connection to the server is open ---
;914   if (! srfsConnect())  { SRFS_UNLOCK();  return -1; }
	.debug	# 914
	lcall	$srfsConnect
	ld	a,e
	or	a,d
	or	a,c
	jr	nz,@153
@154:
	.debug	< @154 @155
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
@312:
	ld	hl,0xffff
	jp	@147
@155:
	.debug	>
;915 
;916   pathLen = strlen(path) + 1;  stat = errNum = 0;
@153:
	.debug	# 916
	ld	hl,(ix+5)
	push	hl
	call	_strlen
	pop	de
	inc	hl
	ld	(ix-12),hl
	bool	hl
	ld	l,h
	ld	(ix-10),hl
	ld	(ix-6),hl
;917 
;918 //--- send UNLINK request and get the response ---
;919   ptr = localBuf;  addShort(&ptr, SRFS_UNLINK);  addShort(&ptr, pathLen);
	.debug	# 919
	ld	hl,-4+@146
	add	hl,sp
	ld	(ix-14),hl
	ld	hl,0x7
	push	hl
	ld	hl,-12+@146
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix-12)
	push	hl
	ld	hl,-12+@146
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;920   if (sendCmd(localBuf, 4, path, pathLen))
	.debug	# 920
	ld	hl,(ix-12)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x4
	push	hl
	ld	hl,2+@146
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
	bool	hl
	jr	z,@156
;921     stat = -1;  else
	.debug	# 921
	ld	hl,0xffff
	ld	(ix-6),hl
;922   if (readResp(&stat, &errNum, &respLen, localBuf, sizeof(localBuf)))
	jr	@157
@156:
	.debug	# 922
	ld	hl,0x4
	push	hl
	ld	hl,-2+@146
	add	hl,sp
	push	hl
	ld	hl,-4+@146
	add	hl,sp
	push	hl
	ld	hl,-4+@146
	add	hl,sp
	push	hl
	ld	hl,2+@146
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jr	z,@158
;923     stat = -1;  else
	.debug	# 923
	ld	hl,0xffff
	ld	(ix-6),hl
;924   if (errNum)  ERRNO = errNum;
	jr	@159
@158:
	.debug	# 924
	ld	hl,(ix-10)
	bool	hl
	jr	z,@160
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-10)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;925 
;926   SRFS_UNLOCK();  return stat;
@160:
@159:
@157:
	.debug	# 926
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-6)
;927 }
@147:
	.debug	} @147 927

@146	equ	14
	ld	sp,ix
	pop	ix
	lret
@148	equ	16
;928 
;929 
;930 //-------------------------------------------------------------------------
;931 //  Attempt to rename a remote file via the SRFS protocol
;932 //
;933 //  Should only be called from the fcntl.c/rename() function!
;934 //
;935 //  RETURN VALUEs:
;936 //  - if successful: 0
;937 //  - if an error: -1 and ERRNO set accordingly
;938 //-------------------------------------------------------------------------
;939 int _srfsRename_(const char *oldPath, const char *newPath)
;940 {
	.align	4
$$_srfsRename_::
	ex	(sp),iy
$_srfsRename_::
	.debug	G $_srfsRename_ 58 9
	.debug	{ $_srfsRename_ 58 939 @163 @162
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-17
	.debug	A oldPath 5 44
	.debug	A newPath 7 44
;941   bool  writeOK;
	.debug	L writeOK -1 3
;942   char  localBuf[4];
	.debug	L localBuf -5 53
;943   int   stat, respLen, errNum, oldPathLen, newPathLen;
	.debug	L stat -7 6
	.debug	L respLen -9 6
	.debug	L errNum -11 6
	.debug	L oldPathLen -13 6
	.debug	L newPathLen -15 6
;944   char  *ptr;
	.debug	L ptr -17 44
;945 
;946 
;947   #ifdef SRFS_DEBUG
;952 //--- ignore names that do not start with "/srfs/" ---
;953   if (strncmp(oldPath, "/srfs/", 6))  { errno = ENXIO;  return -1; }
	.debug	# 953
	ld	hl,0x6
	push	hl
	ld	hl,@1+31
	push	hl
	ld	hl,(ix+5)
	push	hl
	call	_strncmp
	add	sp,6
	bool	hl
	jr	z,@164
@165:
	.debug	< @165 @166
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x6
	jr	@311
@166:
	.debug	>
;954   if (strncmp(newPath, "/srfs/", 6))  { errno = ENXIO;  return -1; }
@164:
	.debug	# 954
	ld	hl,0x6
	push	hl
	ld	hl,@1+31
	push	hl
	ld	hl,(ix+7)
	push	hl
	call	_strncmp
	add	sp,6
	bool	hl
	jr	z,@167
@168:
	.debug	< @168 @169
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x6
@311:
	ld	(hl),e
	inc	hl
	ld	(hl),d
	jp	@310
@169:
	.debug	>
;955   oldPath += 6;  newPath += 6;  // skip over "/srfs/" portion
@167:
	.debug	# 955
	ld	hl,(ix+5)
	ld	de,0x6
	add	hl,de
	ld	(ix+5),hl
	ld	hl,(ix+7)
	add	hl,de
	ld	(ix+7),hl
;956 
;957 
;958   oldPathLen = strlen(oldPath) + 1;  newPathLen = strlen(newPath) + 1;
	.debug	# 958
	ld	hl,(ix+5)
	push	hl
	call	_strlen
	pop	de
	inc	hl
	ld	(ix-13),hl
	ld	hl,(ix+7)
	push	hl
	call	_strlen
	pop	de
	inc	hl
	ld	(ix-15),hl
;959 
;960 //--- get lock for the SRFS driver's shared data ---
;961   if (SRFS_LOCK(30))  return -1;
	.debug	# 961
	ld	hl,0x1e
	push	hl
	lcall	$srfsGetLock
	pop	de
	bool	hl
	jp	nz,@310
;962 
;963 //--- make sure the shared connection to the server is open ---
;964   if (! srfsConnect())  { SRFS_UNLOCK();  return -1; }
	.debug	# 964
	lcall	$srfsConnect
	ld	a,e
	or	a,d
	or	a,c
	jp	z,@309
@172:
	.debug	< @172 @173
@173:
	.debug	>
;965 
;966 //--- send WRITE request and the data ---
;967   ptr = localBuf;  addShort(&ptr, SRFS_RENAME);
	.debug	# 967
	ld	hl,-5+@161
	add	hl,sp
	ld	(ix-17),hl
	ld	hl,0x8
	push	hl
	ld	hl,-15+@161
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;968   addShort(&ptr, oldPathLen + newPathLen);
	.debug	# 968
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-15)
	ex	de,hl
	add	hl,de
	push	hl
	ld	hl,-15+@161
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;969   do  {
;970     writeOK = true;
@177:
	.debug	< @177 @178
	.debug	# 970
	ld	a,0x1
	ld	(ix-1),a
;971     if (writeFDEV(sockFDEV, localBuf, 4, MAX_RW_MS) != 4)
	.debug	# 971
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	ld	hl,0x4
	push	hl
	ld	hl,1+@161
	add	hl,sp
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$writeFDEV
	add	sp,12
	ld	de,0x4
	or	a,a
	sbc	hl,de
	jr	z,@179
;972       writeOK = false;  else
	.debug	# 972
	xor	a,a
	ld	(ix-1),a
;973     if (writeFDEV(sockFDEV, (char *)oldPath, oldPathLen, MAX_RW_MS) != oldP
	jr	@180
@179:
	.debug	# 973
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-13)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$writeFDEV
	add	sp,12
	ex	de,hl
	ld	hl,(ix-13)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@181
;974       writeOK = false;  else
	.debug	# 974
	xor	a,a
	ld	(ix-1),a
;975     if (writeFDEV(sockFDEV, (char *)newPath, newPathLen, MAX_RW_MS) != newP
	jr	@182
@181:
	.debug	# 975
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-15)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	de,(_sockFDEV)
	ld	bc,(_sockFDEV+2)
	push	bc
	push	de
	lcall	$writeFDEV
	add	sp,12
	ex	de,hl
	ld	hl,(ix-15)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@183
;976       writeOK = false;
	.debug	# 976
	xor	a,a
	ld	(ix-1),a
;977   } while(0);
@183:
@182:
@180:
@178:
	.debug	>
	.debug	# 977
;978 
;979   if (!writeOK)  {
	.debug	# 979
	ld	a,(ix-1)
	or	a,a
	jr	nz,@184
;980     fprintf(DiagLog, "_srfsRename_(): %ls\n", strerror(errno) );
@185:
	.debug	< @185 @186
	.debug	# 980
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strerror
	pop	hl
	push	bc
	push	de
	ld	de,word (phys @8+362)
	ld	c,phys @8+362 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,10
;981     _srfsConFailed_();  SRFS_UNLOCK();  return -1;
	.debug	# 981
	lcall	$_srfsConFailed_
@309:
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
@310:
	ld	hl,0xffff
	jr	@162
;982   }
@186:
	.debug	>
;983 
;984 //--- wait for response to RENAME request ---
;985   if (readResp(&stat, &errNum, &respLen, localBuf, sizeof(localBuf)))
@184:
	.debug	# 985
	ld	hl,0x4
	push	hl
	ld	hl,-3+@161
	add	hl,sp
	push	hl
	ld	hl,-5+@161
	add	hl,sp
	push	hl
	ld	hl,-5+@161
	add	hl,sp
	push	hl
	ld	hl,1+@161
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jr	z,@187
;986     stat = -1;
	.debug	# 986
	ld	hl,0xffff
	ld	(ix-7),hl
;987   else
;988     if (errNum)  ERRNO = errNum;
	jr	@188
@187:
	.debug	# 988
	ld	hl,(ix-11)
	bool	hl
	jr	z,@189
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-11)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;989 
;990   SRFS_UNLOCK();  return stat;
@189:
@188:
	.debug	# 990
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-7)
;991 }
@162:
	.debug	} @162 991

@161	equ	17
	ld	sp,ix
	pop	ix
	lret
@163	equ	19
;992 
;993 
;994 
;995 //=========================================================================
;996 //  IODev interface functions (for use by fcntl module)
;997 //=========================================================================
;998 
;999 //-------------------------------------------------------------------------
;1000 //  The only difference between blocking and non-blocking mode for the SRFS
;1001 //  driver files is whether or not the call waits for the global SRFS lock
;1002 //  if it is not available immediately.  So there is no point in having the
;1003 //  fcntl layer attempt a read or write multiple times.
;1004 //-------------------------------------------------------------------------
;1005 int _srfsGetQueues_(void *ioDev, WaitQueues far *queues)
;1006 {
	.align	4
$$_srfsGetQueues_::
	ex	(sp),iy
$_srfsGetQueues_::
	.debug	G $_srfsGetQueues_ 59 9
	.debug	{ $_srfsGetQueues_ 59 1005 @192 @191
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ioDev 5 36
	.debug	A queues 7 60
;1007   queues->readQueue = queues->writeQueue = NULL;  //&sleepQueue;
	.debug	# 1007
	ld	hl,(ix+7)
	ex	de,hl
	ld	hl,(ix+9)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	bool	hl
	ld	l,h
	call	__farst2##
	ex	de,hl
	ld	hl,(ix+9)
	ld	bc,hl
	ld	hl,(ix+7)
	ex	de,hl
	call	__farst2##
;1008 
;1009   return 0;
	.debug	# 1009
	bool	hl
	ld	l,h
;1010 }
@191:
	.debug	} @191 1010

@190	equ	0
	pop	ix
	lret
@192	equ	2
;1011 
;1012 //-------------------------------------------------------------------------
;1013 //  close an SRFS connection to a remote file
;1014 //-------------------------------------------------------------------------
;1015 int _srfsClose_(void *ioDev)
;1016 {
	.align	4
$$_srfsClose_::
	ex	(sp),iy
$_srfsClose_::
	.debug	G $_srfsClose_ 62 9
	.debug	{ $_srfsClose_ 62 1015 @195 @194
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-24
	.debug	A ioDev 5 36
;1017   int  stat, respLen, errNum;
	.debug	L stat -2 6
	.debug	L respLen -4 6
	.debug	L errNum -6 6
;1018   char  *ptr;
	.debug	L ptr -8 44
;1019   char  localBuf[16];
	.debug	L localBuf -24 52
;1020 
;1021 
;1022   if (!validSRFS(ioDev))  { ERRNO = EBADF;  return -1; }
	.debug	# 1022
	ld	hl,(ix+5)
	push	hl
	lcall	$validSRFS
	pop	hl
	or	a,a
	jr	nz,@196
@197:
	.debug	< @197 @198
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x9
	ld	(hl),e
	inc	hl
	ld	(hl),d
	jr	@308
@198:
	.debug	>
;1023 
;1024   #ifdef SRFS_DEBUG
;1029 
;1030 //--- get lock for the SRFS driver's shared data ---
;1031   if (SRFS_LOCK(30))  return -1;
@196:
	.debug	# 1031
	ld	hl,0x1e
	push	hl
	lcall	$srfsGetLock
	pop	de
	bool	hl
	jr	nz,@308
;1032 
;1033 //--- previous error on the server connection we were using? ---
;1034   if ((SRFS->remfd < 0) OR SRFS->errNum)  {
	.debug	# 1034
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@200
	ld	hl,(ix+5)
	inc	hl
	ld	hl,(hl)
	bool	hl
	jr	z,@201
@200:
;1035     ERRNO = SRFS->errNum;  SRFS->inUse = false;
@202:
	.debug	< @202 @203
	.debug	# 1035
	ld	hl,(ix+5)
	inc	hl
	ld	de,(_current)
	ld	bc,hl
	ld	hl,0xb9
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix+5)
	xor	a,a
	ld	(hl),a
;1036     SRFS_UNLOCK();  return -1;
	.debug	# 1036
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
@308:
	ld	hl,0xffff
	jp	@194
;1037   }
@203:
	.debug	>
;1038 
;1039 //--- Send CLOSE cmd and wait for the response ---
;1040   ptr = localBuf;  addShort(&ptr, SRFS_CLOSE);
@201:
	.debug	# 1040
	ld	hl,-24+@193
	add	hl,sp
	ld	(ix-8),hl
	ld	hl,0x2
	push	hl
	ld	hl,-6+@193
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1041   addShort(&ptr, 2);  addShort(&ptr, SRFS->remfd);
	.debug	# 1041
	ld	hl,0x2
	push	hl
	ld	hl,-6+@193
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,-6+@193
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1042 
;1043   stat = 0;
	.debug	# 1043
	bool	hl
	ld	l,h
	ld	(ix-2),hl
;1044   if (sendCmd(localBuf, 6, NULL, 0))
	.debug	# 1044
	push	hl
	push	hl
	ld	l,0x6
	push	hl
	ld	hl,-18+@193
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
	bool	hl
	jr	z,@204
;1045     stat = -1;  else
	.debug	# 1045
	ld	hl,0xffff
	ld	(ix-2),hl
;1046   if (readResp(&stat, &errNum, &respLen, localBuf, sizeof(localBuf)))
	jr	@205
@204:
	.debug	# 1046
	ld	hl,0x10
	push	hl
	ld	hl,-22+@193
	add	hl,sp
	push	hl
	ld	hl,0+@193
	add	hl,sp
	push	hl
	ld	hl,0+@193
	add	hl,sp
	push	hl
	ld	hl,6+@193
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jr	z,@206
;1047     stat = -1;  else
	.debug	# 1047
	ld	hl,0xffff
	ld	(ix-2),hl
;1048   if (errNum)  ERRNO = errNum;
	jr	@207
@206:
	.debug	# 1048
	ld	hl,(ix-6)
	bool	hl
	jr	z,@208
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-6)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1049 
;1050 
;1051 //--- clear the driver's ioDev for the remote file so it can be reused ---
;1052   SRFS->remfd = -1;  SRFS->inUse = false;
@208:
@207:
@205:
	.debug	# 1052
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	de,0xffff
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix+5)
	xor	a,a
	ld	(hl),a
;1053 
;1054 
;1055   SRFS_UNLOCK();  return stat;
	.debug	# 1055
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-2)
;1056 }
@194:
	.debug	} @194 1056

@193	equ	24
	ld	sp,ix
	pop	ix
	lret
@195	equ	26
;1057 
;1058 //-------------------------------------------------------------------------
;1059 //  Attempt to read bytes from a remote file via the SRFS protocol
;1060 //
;1061 //  Should only be called from the fcntl.c/read() function!
;1062 //
;1063 //  RETURN VALUEs:
;1064 //  - if bytes read: # of bytes read
;1065 //  - if no bytes read:
;1066 //    - if more possible in the future: -1 and ERRNO == EAGAIN
;1067 //    - if no more possible (EOF):  0
;1068 //-------------------------------------------------------------------------
;1069 int _srfsRead_(void *ioDev, char *buf, int numBytes, bool dontBlock)
;1070 {
	.align	4
$$_srfsRead_::
	ex	(sp),iy
$_srfsRead_::
	.debug	G $_srfsRead_ 63 9
	.debug	{ $_srfsRead_ 63 1069 @211 @210
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-24
	.debug	A ioDev 5 36
	.debug	A buf 7 44
	.debug	A numBytes 9 6
	.debug	A dontBlock 11 3
;1071   char  localBuf[16];
	.debug	L localBuf -16 52
;1072   int  stat, respLen, errNum;
	.debug	L stat -18 6
	.debug	L respLen -20 6
	.debug	L errNum -22 6
;1073   char  *ptr;
	.debug	L ptr -24 44
;1074 
;1075 
;1076   #ifdef SRFS_DEBUG
;1080   if (!validSRFS(ioDev))  { ERRNO = EBADF;  return -1; }
	.debug	# 1080
	ld	hl,(ix+5)
	push	hl
	lcall	$validSRFS
	pop	hl
	or	a,a
	jr	nz,@212
@213:
	.debug	< @213 @214
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x9
	ld	(hl),e
	inc	hl
	ld	(hl),d
	jr	@307
@214:
	.debug	>
;1081 
;1082 //--- Attempt to get the global lock (with no timeout if non-blocking mode)
;1083   if (dontBlock ? sem_trywait(&srfsSem) : sem_wait(&srfsSem))  return -1;
@212:
	.debug	# 1083
	ld	a,(ix+11)
	or	a,a
	jr	z,@215
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_trywait
	pop	de
	jr	@216
@215:
	ld	de,0xffff
	ld	bc,0x7fff
	push	bc
	push	de
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_wait_
	add	sp,6
@216:
	bool	hl
	jr	nz,@307
;1084 
;1085 //--- previous error on the server connection we were using? ---
;1086   if ((SRFS->remfd < 0) OR SRFS->errNum)  {
	.debug	# 1086
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@218
	ld	hl,(ix+5)
	inc	hl
	ld	hl,(hl)
	bool	hl
	jr	z,@219
@218:
;1087     ERRNO = SRFS->errNum;  SRFS_UNLOCK();  return -1; }
@220:
	.debug	< @220 @221
	.debug	# 1087
	ld	hl,(ix+5)
	inc	hl
	ld	de,(_current)
	ld	bc,hl
	ld	hl,0xb9
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
@307:
	ld	hl,0xffff
	jp	@210
@221:
	.debug	>
;1088 
;1089 
;1090 //--- Send READ request and read parameters ---
;1091   ptr = localBuf;
@219:
	.debug	# 1091
	ld	hl,-16+@209
	add	hl,sp
	ld	(ix-24),hl
;1092   addShort(&ptr, SRFS_READ);  addShort(&ptr, 4);
	.debug	# 1092
	ld	hl,0x3
	push	hl
	ld	hl,-22+@209
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,0x4
	push	hl
	ld	hl,-22+@209
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1093   addShort(&ptr, SRFS->remfd);  addShort(&ptr, numBytes);
	.debug	# 1093
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,-22+@209
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix+9)
	push	hl
	ld	hl,-22+@209
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1094 
;1095   stat = 0;
	.debug	# 1095
	bool	hl
	ld	l,h
	ld	(ix-18),hl
;1096   if (sendCmd(localBuf, 8, NULL, 0))
	.debug	# 1096
	push	hl
	push	hl
	ld	l,0x8
	push	hl
	ld	hl,-10+@209
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
	bool	hl
	jr	z,@222
;1097     stat = -1;  else
	.debug	# 1097
	ld	hl,0xffff
	ld	(ix-18),hl
;1098   if (readResp(&stat, &errNum, &respLen, buf, numBytes))
	jr	@223
@222:
	.debug	# 1098
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,-16+@209
	add	hl,sp
	push	hl
	ld	hl,-16+@209
	add	hl,sp
	push	hl
	ld	hl,-10+@209
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jr	z,@224
;1099     stat = -1;
	.debug	# 1099
	ld	hl,0xffff
	ld	(ix-18),hl
;1100   else  {
	jr	@225
@224:
;1101     if (errNum)  errno = errNum;
@226:
	.debug	< @226 @227
	.debug	# 1101
	ld	hl,(ix-22)
	bool	hl
	jr	z,@228
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-22)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1102     if (errno == EAGAIN)  errno = EIO;
@228:
	.debug	# 1102
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0xb
	or	a,a
	sbc	hl,de
	jr	nz,@229
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x5
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1103   }
@229:
@227:
	.debug	>
;1104 
;1105 //fprintf(DiagLog, "srfsd-read returned: %d/%d\r\n", stat, errno);  //tdebu
;1106 
;1107   SRFS_UNLOCK();  return stat;
@225:
@223:
	.debug	# 1107
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-18)
;1108 }
@210:
	.debug	} @210 1108

@209	equ	24
	ld	sp,ix
	pop	ix
	lret
@211	equ	26
;1109 
;1110 //-------------------------------------------------------------------------
;1111 //  Attempt to write bytes to remote file via the SRFS protocol
;1112 //
;1113 //  Should only be called from the fcntl.c/write() function!
;1114 //
;1115 //  RETURN VALUEs:
;1116 //  - if bytes written: # of bytes written
;1117 //  - if no bytes written:
;1118 //    - if more possible in the future: -1 and ERRNO == EAGAIN
;1119 //    - if no more possible (EOF):  0
;1120 //-------------------------------------------------------------------------
;1121 int _srfsWrite_(void *ioDev, char *buf, int numBytes, bool dontBlock)
;1122 {
	.align	4
$$_srfsWrite_::
	ex	(sp),iy
$_srfsWrite_::
	.debug	G $_srfsWrite_ 63 9
	.debug	{ $_srfsWrite_ 63 1121 @232 @231
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-24
	.debug	A ioDev 5 36
	.debug	A buf 7 44
	.debug	A numBytes 9 6
	.debug	A dontBlock 11 3
;1123   char  localBuf[16];
	.debug	L localBuf -16 52
;1124   int   stat, respLen, errNum;
	.debug	L stat -18 6
	.debug	L respLen -20 6
	.debug	L errNum -22 6
;1125   char  *ptr;
	.debug	L ptr -24 44
;1126 
;1127 
;1128   #ifdef SRFS_DEBUG
;1134   if (!validSRFS(ioDev))  { ERRNO = EBADF;  return -1; }
	.debug	# 1134
	ld	hl,(ix+5)
	push	hl
	lcall	$validSRFS
	pop	hl
	or	a,a
	jr	nz,@233
@234:
	.debug	< @234 @235
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x9
	ld	(hl),e
	inc	hl
	ld	(hl),d
	jr	@306
@235:
	.debug	>
;1135 
;1136 
;1137 //--- Attempt to get the global lock (with no timeout if non-blocking mode)
;1138   if (dontBlock ? sem_trywait(&srfsSem) : sem_wait(&srfsSem))  return -1;
@233:
	.debug	# 1138
	ld	a,(ix+11)
	or	a,a
	jr	z,@236
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_trywait
	pop	de
	jr	@237
@236:
	ld	de,0xffff
	ld	bc,0x7fff
	push	bc
	push	de
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_wait_
	add	sp,6
@237:
	bool	hl
	jr	nz,@306
;1139 
;1140 //--- previous error on the server connection we were using? ---
;1141   if ((SRFS->remfd < 0) OR SRFS->errNum)  {
	.debug	# 1141
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@239
	ld	hl,(ix+5)
	inc	hl
	ld	hl,(hl)
	bool	hl
	jr	z,@240
@239:
;1142     ERRNO = SRFS->errNum;  SRFS_UNLOCK();  return -1; }
@241:
	.debug	< @241 @242
	.debug	# 1142
	ld	hl,(ix+5)
	inc	hl
	ld	de,(_current)
	ld	bc,hl
	ld	hl,0xb9
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
@306:
	ld	hl,0xffff
	jp	@231
@242:
	.debug	>
;1143 
;1144 
;1145 //--- Send WRITE request and wait for the response ---
;1146   ptr = localBuf;
@240:
	.debug	# 1146
	ld	hl,-16+@230
	add	hl,sp
	ld	(ix-24),hl
;1147   addShort(&ptr, SRFS_WRITE);  addShort(&ptr, 4+numBytes);
	.debug	# 1147
	ld	hl,0x4
	push	hl
	ld	hl,-22+@230
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix+9)
	ld	de,0x4
	add	hl,de
	push	hl
	ld	hl,-22+@230
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1148   addShort(&ptr, SRFS->remfd);  addShort(&ptr, numBytes);
	.debug	# 1148
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,-22+@230
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix+9)
	push	hl
	ld	hl,-22+@230
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1149 
;1150   stat = 0;
	.debug	# 1150
	bool	hl
	ld	l,h
	ld	(ix-18),hl
;1151   if (sendCmd(localBuf, 8, buf, numBytes))
	.debug	# 1151
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,0x8
	push	hl
	ld	hl,-10+@230
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
	bool	hl
	jr	z,@243
;1152     stat = -1;  else
	.debug	# 1152
	ld	hl,0xffff
	ld	(ix-18),hl
;1153   if (readResp(&stat, &errNum, &respLen, localBuf, sizeof(localBuf)))
	jr	@244
@243:
	.debug	# 1153
	ld	hl,0x10
	push	hl
	ld	hl,-14+@230
	add	hl,sp
	push	hl
	ld	hl,-16+@230
	add	hl,sp
	push	hl
	ld	hl,-16+@230
	add	hl,sp
	push	hl
	ld	hl,-10+@230
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jr	z,@245
;1154     stat = -1;
	.debug	# 1154
	ld	hl,0xffff
	ld	(ix-18),hl
;1155   else  {
	jr	@246
@245:
;1156     if (errNum)  errno = errNum;
@247:
	.debug	< @247 @248
	.debug	# 1156
	ld	hl,(ix-22)
	bool	hl
	jr	z,@249
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-22)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1157     if (errno == EAGAIN)  errno = EIO;
@249:
	.debug	# 1157
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0xb
	or	a,a
	sbc	hl,de
	jr	nz,@250
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x5
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1158   }
@250:
@248:
	.debug	>
;1159 
;1160 //fprintf(DiagLog, "srfsd-write returned: %d/%d\r\n", stat, errno);  //tdeb
;1161 
;1162   SRFS_UNLOCK();  return stat;
@246:
@244:
	.debug	# 1162
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix-18)
;1163 }
@231:
	.debug	} @231 1163

@230	equ	24
	ld	sp,ix
	pop	ix
	lret
@232	equ	26
;1164 
;1165 
;1166 //-------------------------------------------------------------------------
;1167 //  Set the current offset for a remote file using the SRFS protocol
;1168 //-------------------------------------------------------------------------
;1169 off_t  _srfsLseek_(void *ioDev, off_t offset, int whence)
;1170 {
	.align	4
$$_srfsLseek_::
	ex	(sp),iy
$_srfsLseek_::
	.debug	G $_srfsLseek_ 64 9
	.debug	{ $_srfsLseek_ 64 1169 @253 @252
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-24
	.debug	A ioDev 5 36
	.debug	A offset 7 9
	.debug	A whence 11 6
;1171   char  localBuf[16];
	.debug	L localBuf -16 52
;1172   int    stat, respLen, errNum;
	.debug	L stat -18 6
	.debug	L respLen -20 6
	.debug	L errNum -22 6
;1173   char  *ptr;
	.debug	L ptr -24 44
;1174 
;1175 
;1176   #ifdef SRFS_DEBUG
;1180   if (!validSRFS(ioDev))  { ERRNO = EBADF;  return -1; }
	.debug	# 1180
	ld	hl,(ix+5)
	push	hl
	lcall	$validSRFS
	pop	hl
	or	a,a
	jr	nz,@254
@255:
	.debug	< @255 @256
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x9
	ld	(hl),e
	inc	hl
	ld	(hl),d
	jr	@305
@256:
	.debug	>
;1181 
;1182 //--- Attempt to get the global lock ---
;1183   if (SRFS_LOCK(15))  return -1;
@254:
	.debug	# 1183
	ld	hl,0xf
	push	hl
	lcall	$srfsGetLock
	pop	de
	bool	hl
	jr	nz,@305
;1184 
;1185 //--- previous error on the server connection we were using? ---
;1186   if ((SRFS->remfd < 0) OR SRFS->errNum)  {
	.debug	# 1186
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@258
	ld	hl,(ix+5)
	inc	hl
	ld	hl,(hl)
	bool	hl
	jr	z,@259
@258:
;1187     ERRNO = SRFS->errNum;  SRFS_UNLOCK();  return -1; }
@260:
	.debug	< @260 @261
	.debug	# 1187
	ld	hl,(ix+5)
	inc	hl
	ld	de,(_current)
	ld	bc,hl
	ld	hl,0xb9
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
@305:
	ld	de,0xffff
	ld	c,e
	ld	b,e
	jp	@252
@261:
	.debug	>
;1188 
;1189 
;1190 //--- send LSEEK request and wait for the response ---
;1191   ptr = localBuf;
@259:
	.debug	# 1191
	ld	hl,-16+@251
	add	hl,sp
	ld	(ix-24),hl
;1192   addShort(&ptr, SRFS_LSEEK);  addShort(&ptr, 8);
	.debug	# 1192
	ld	hl,0x5
	push	hl
	ld	hl,-22+@251
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,0x8
	push	hl
	ld	hl,-22+@251
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1193   addShort(&ptr, SRFS->remfd);
	.debug	# 1193
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,-22+@251
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1194   addLong(&ptr, offset);  addShort(&ptr, whence);
	.debug	# 1194
	ld	hl,(ix+7)
	ex	de,hl
	ld	hl,(ix+9)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,-20+@251
	add	hl,sp
	push	hl
	lcall	$addLong
	add	sp,6
	ld	hl,(ix+11)
	push	hl
	ld	hl,-22+@251
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1195 
;1196   stat = 0;
	.debug	# 1196
	bool	hl
	ld	l,h
	ld	(ix-18),hl
;1197   if (sendCmd(localBuf, 12, NULL, 0))
	.debug	# 1197
	push	hl
	push	hl
	ld	l,0xc
	push	hl
	ld	hl,-10+@251
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
	bool	hl
	jr	z,@262
;1198     offset = (off_t)(-1);  else
	.debug	# 1198
	ld	de,0xffff
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix+9),hl
	ex	de,hl
	ld	(ix+7),hl
	ex	de,hl
;1199   if (readResp(&stat, &errNum, &respLen, localBuf, sizeof(localBuf)))
	jr	@263
@262:
	.debug	# 1199
	ld	hl,0x10
	push	hl
	ld	hl,-14+@251
	add	hl,sp
	push	hl
	ld	hl,-16+@251
	add	hl,sp
	push	hl
	ld	hl,-16+@251
	add	hl,sp
	push	hl
	ld	hl,-10+@251
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jr	z,@264
;1200     offset = (off_t)(-1);
	.debug	# 1200
	ld	de,0xffff
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix+9),hl
	ex	de,hl
	ld	(ix+7),hl
	ex	de,hl
;1201   else  {
	jr	@265
@264:
;1202     if (errNum)  ERRNO = errNum;
@266:
	.debug	< @266 @267
	.debug	# 1202
	ld	hl,(ix-22)
	bool	hl
	jr	z,@268
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-22)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1203     if (errno == EAGAIN)  errno = EIO;
@268:
	.debug	# 1203
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0xb
	or	a,a
	sbc	hl,de
	jr	nz,@269
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x5
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1204     ptr = localBuf;  offset = getLong(&ptr);
@269:
	.debug	# 1204
	ld	hl,-16+@251
	add	hl,sp
	ld	(ix-24),hl
	ld	hl,-24+@251
	add	hl,sp
	push	hl
	lcall	$getLong
	pop	hl
	ld	hl,bc
	ld	(ix+9),hl
	ex	de,hl
	ld	(ix+7),hl
	ex	de,hl
;1205   }
@267:
	.debug	>
;1206 
;1207 //fprintf(DiagLog, "srfsd-write returned: %d/%d\r\n", stat, errno);  //tdeb
;1208 
;1209   SRFS_UNLOCK();  return offset;
@265:
@263:
	.debug	# 1209
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,(ix+7)
	ex	de,hl
	ld	hl,(ix+9)
	ld	bc,hl
;1210 }
@252:
	.debug	} @252 1210

@251	equ	24
	ld	sp,ix
	pop	ix
	lret
@253	equ	26
;1211 
;1212 
;1213 //-------------------------------------------------------------------------
;1214 //  Get information about a remote file using the SRFS protocol
;1215 //-------------------------------------------------------------------------
;1216 int  _srfsFstat_(void *ioDev, struct stat *fsbuf)
;1217 {
	.align	4
$$_srfsFstat_::
	ex	(sp),iy
$_srfsFstat_::
	.debug	G $_srfsFstat_ 65 9
	.debug	{ $_srfsFstat_ 65 1216 @272 @271
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-50
	.debug	A ioDev 5 36
	.debug	A fsbuf 7 66
;1218   char  buf[sizeof(struct stat)];
	.debug	L buf -42 68
;1219   int   respStat, respLen, errNum;
	.debug	L respStat -44 6
	.debug	L respLen -46 6
	.debug	L errNum -48 6
;1220   char  *ptr;
	.debug	L ptr -50 44
;1221 
;1222 
;1223   #ifdef SRFS_DEBUG
;1227   if (!validSRFS(ioDev))  { ERRNO = EBADF;  return -1; }
	.debug	# 1227
	ld	hl,(ix+5)
	push	hl
	lcall	$validSRFS
	pop	hl
	or	a,a
	jr	nz,@273
@274:
	.debug	< @274 @275
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x9
	ld	(hl),e
	inc	hl
	ld	(hl),d
	jp	@304
@275:
	.debug	>
;1228 
;1229 
;1230 //--- fill buffer with FSTAT request ---
;1231   ptr = buf;
@273:
	.debug	# 1231
	ld	hl,-42+@270
	add	hl,sp
	ld	(ix-50),hl
;1232   addShort(&ptr, SRFS_FSTAT);  addShort(&ptr, 4);
	.debug	# 1232
	ld	hl,0x6
	push	hl
	ld	hl,-48+@270
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,0x4
	push	hl
	ld	hl,-48+@270
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1233   addShort(&ptr, SRFS->remfd);  addShort(&ptr, sizeof(buf));
	.debug	# 1233
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,-48+@270
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,0x2a
	push	hl
	ld	hl,-48+@270
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;1234 
;1235 //--- Try to get the lock for the shared connection ---
;1236   if (SRFS_LOCK(15))  return -1;
	.debug	# 1236
	ld	hl,0xf
	push	hl
	lcall	$srfsGetLock
	pop	de
	bool	hl
	jr	nz,@304
;1237 
;1238 //--- previous error on the server connection we were using? ---
;1239   if ((SRFS->remfd < 0) OR SRFS->errNum)  {
	.debug	# 1239
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@277
	ld	hl,(ix+5)
	inc	hl
	ld	hl,(hl)
	bool	hl
	jr	z,@278
@277:
;1240     ERRNO = SRFS->errNum;  SRFS_UNLOCK();  return -1; }
@279:
	.debug	< @279 @280
	.debug	# 1240
	ld	hl,(ix+5)
	inc	hl
	ld	de,(_current)
	ld	bc,hl
	ld	hl,0xb9
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
@304:
	ld	hl,0xffff
	jp	@271
@280:
	.debug	>
;1241 
;1242 
;1243 //--- send cmd and wait for response ---
;1244   do  {
@278:
;1245     if (sendCmd(buf, 8, NULL, 0))  break;
@284:
	.debug	< @284 @285
	.debug	# 1245
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	l,0x8
	push	hl
	ld	hl,-36+@270
	add	hl,sp
	push	hl
	lcall	$sendCmd
	add	sp,8
	bool	hl
	jp	nz,@282
;1246     if (readResp(&respStat, &errNum, &respLen, buf, sizeof(buf)))  break;
	.debug	# 1246
	ld	l,0x2a
	push	hl
	ld	hl,-40+@270
	add	hl,sp
	push	hl
	ld	hl,-42+@270
	add	hl,sp
	push	hl
	ld	hl,-42+@270
	add	hl,sp
	push	hl
	ld	hl,-36+@270
	add	hl,sp
	push	hl
	lcall	$readResp
	add	sp,10
	bool	hl
	jp	nz,@282
;1247 
;1248     SRFS_UNLOCK();
	.debug	# 1248
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
;1249 
;1250     if (errNum)  errno = errNum;
	.debug	# 1250
	ld	hl,(ix-48)
	bool	hl
	jr	z,@288
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ex	de,hl
	ld	hl,(ix-48)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1251     if (errno == EAGAIN)  errno = EIO;
@288:
	.debug	# 1251
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0xb
	or	a,a
	sbc	hl,de
	jr	nz,@289
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x5
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1252 
;1253     if (respStat == 0)  {
@289:
	.debug	# 1253
	ld	hl,(ix-44)
	bool	hl
	jp	nz,@290
;1254       ptr = (char *)buf;
@291:
	.debug	< @291 @292
	.debug	# 1254
	ld	hl,-42+@270
	add	hl,sp
	ld	(ix-50),hl
;1255       fsbuf->st_mode  = getLong(&ptr);  // protection and file type bits
	.debug	# 1255
	ld	hl,-50+@270
	add	hl,sp
	push	hl
	lcall	$getLong
	pop	hl
	ld	hl,(ix+7)
	push	de
	ld	de,0x8
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1256       fsbuf->st_rdev  = getLong(&ptr);  // device type
	.debug	# 1256
	ld	hl,-50+@270
	add	hl,sp
	push	hl
	lcall	$getLong
	pop	hl
	ld	hl,(ix+7)
	push	de
	ld	de,0x12
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1257       fsbuf->st_size  = getLong(&ptr);  // total size in bytes
	.debug	# 1257
	ld	hl,-50+@270
	add	hl,sp
	push	hl
	lcall	$getLong
	pop	hl
	ld	hl,(ix+7)
	push	de
	ld	de,0x16
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1258       fsbuf->st_atime = getLong(&ptr);  // time of last access
	.debug	# 1258
	ld	hl,-50+@270
	add	hl,sp
	push	hl
	lcall	$getLong
	pop	hl
	ld	hl,(ix+7)
	push	de
	ld	de,0x1e
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1259       fsbuf->st_mtime = getLong(&ptr);  // time of last modification
	.debug	# 1259
	ld	hl,-50+@270
	add	hl,sp
	push	hl
	lcall	$getLong
	pop	hl
	ld	hl,(ix+7)
	push	de
	ld	de,0x22
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1260       fsbuf->st_ctime = getLong(&ptr);  // time of last status change
	.debug	# 1260
	ld	hl,-50+@270
	add	hl,sp
	push	hl
	lcall	$getLong
	pop	hl
	ld	hl,(ix+7)
	push	de
	ld	de,0x26
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1261     }
@292:
	.debug	>
;1262 
;1263     return respStat;
@290:
	.debug	# 1263
	ld	hl,(ix-44)
	jr	@271
;1264   } while(0);
@285:
	.debug	>
;1265 
;1266   SRFS_UNLOCK();  return -1;
@282:
	.debug	# 1266
	ld	hl,_srfsSem
	push	hl
	lcall	$sem_post
	pop	de
	ld	hl,0xffff
;1267 }
@271:
	.debug	} @271 1267

@270	equ	50
	ld	sp,ix
	pop	ix
	lret
@272	equ	52
;1268 
;1269 //-------------------------------------------------------------------------
;1270 //  Kernel-space function to handle ioctl calls for srfs ports
;1271 //-------------------------------------------------------------------------
;1272 int _srfsioctl_(void *ioDev, int request, void *arg)
;1273 {
	.align	4
$$_srfsioctl_::
	ex	(sp),iy
$_srfsioctl_::
	.debug	G $_srfsioctl_ 69 9
	.debug	{ $_srfsioctl_ 69 1272 @295 @294
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ioDev 5 36
	.debug	A request 7 6
	.debug	A arg 9 36
;1274   if (!validSRFS(ioDev))  { ERRNO = EBADF;  return -1; }
	.debug	# 1274
	ld	hl,(ix+5)
	push	hl
	lcall	$validSRFS
	pop	hl
	or	a,a
	jr	nz,@296
@297:
	.debug	< @297 @298
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x9
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,0xffff
	jr	@294
@298:
	.debug	>
;1275 
;1276  //finish:  need to implement this...
;1277 
;1278   ERRNO = EINVAL;  return -1;
@296:
	.debug	# 1278
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x16
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,0xffff
;1279 }
@294:
	.debug	} @294 1279

@293	equ	0
	pop	ix
	lret
@295	equ	2
;1280 
;1281 
;1282 //-------------------------------------------------------------------------
;1283 //  Kernel-space function to return a description of the srfs port
;1284 //
;1285 //  WARNING!  To be called ONLY by KERNELFUNC functions (or functions calle
;1286 //            KERNELFUNC functions) !!!!
;1287 //-------------------------------------------------------------------------
;1288 char *_srfsDescr_(void *ioDev, char *buf, int bufSize)
;1289 {
	.align	4
$$_srfsDescr_::
	ex	(sp),iy
$_srfsDescr_::
	.debug	G $_srfsDescr_ 70 9
	.debug	{ $_srfsDescr_ 70 1288 @301 @300
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A ioDev 5 36
	.debug	A buf 7 44
	.debug	A bufSize 9 6
;1290   char  *ptr;
	.debug	L ptr -2 44
;1291 
;1292 
;1293   sprintf(buf, "srfs 0x%04lX ", (ulong)ioDev);
	.debug	# 1293
	ld	hl,(ix+5)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,word (phys @8+383)
	ld	c,phys @8+383 >> 16
	push	bc
	push	de
	ld	hl,(ix+7)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,12
;1294 
;1295   ptr = buf + strlen(buf);
	.debug	# 1295
	ld	hl,(ix+7)
	push	hl
	call	_strlen
	pop	de
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	(ix-2),hl
;1296 
;1297 //finish: limit output to bufSize
;1298 
;1299   KLOCK();
	.debug	# 1299
	call	_KLOCK
;1300 
;1301   if (validSRFS(ioDev))
	.debug	# 1301
	ld	hl,(ix+5)
	push	hl
	lcall	$validSRFS
	pop	hl
	or	a,a
	jr	z,@302
;1302     sprintf(ptr, "srfs[%d] %ls", (int)(SRFS - srfsFile), SRFS->name);
	.debug	# 1302
	ld	hl,(ix+5)
	ld	de,0x5
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix+5)
	ld	de,_srfsFile
	or	a,a
	sbc	hl,de
	ld	de,0x9
	call	__idiv##
	push	hl
	ld	de,word (phys @8+397)
	ld	c,phys @8+397 >> 16
	push	bc
	push	de
	ld	hl,(ix-2)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,14
;1303   else
;1304     strcpy(ptr, "bad bad");
	jr	@303
@302:
	.debug	# 1304
	ld	hl,@1+38
	push	hl
	ld	hl,(ix-2)
	push	hl
	call	_strcpy
	add	sp,4
;1305 
;1306   KUNLOCK();
@303:
	.debug	# 1306
	call	_KUNLOCK
;1307 
;1308   return buf;
	.debug	# 1308
	ld	hl,(ix+7)
;1309 }
@300:
	.debug	} @300 1309

@299	equ	2
	ld	sp,ix
	pop	ix
	lret
@301	equ	4
;1310 
;1311 
;1312 //-------------------------------------------------------------------------
;1313 
	.extern	$time 71
	.extern	_memset 72
	.extern	_strcpy 73
	.extern	_strlen 57
	.extern	_strncmp 74
	.extern	_fstrcpy 75
	.extern	$addShort 76
	.extern	$addLong 77
	.extern	$getShort 78
	.extern	$getLong 79
	.extern	$inet_ptoh 80
	.extern	_current 81
	.extern	_kernelTask 81
	.extern	$farfprintf 83
	.extern	$farsprintf 84
	.extern	__TimerMilliSeconds 85
	.extern	$registerFileSys 86
	.extern	$_validFDEV_ 87
	.extern	$_allocFile_ 88
	.extern	$_fdToOpenFile_ 89
	.extern	$writeFDEV 90
	.extern	$readFDEV 90
	.extern	$close 37
	.extern	_KLOCK 43
	.extern	_KUNLOCK 43
	.extern	_doShutdown 3
	.extern	_diagLog 91
	.extern	$connect 92
	.extern	$socket 93
	.extern	$sem_wait_ 94
	.extern	$sem_init 95
	.extern	$sem_timedwait 96
	.extern	$sem_trywait 97
	.extern	$sem_post 97
	.extern	$perror 98
	.extern	$strerror 99
	.defseg	FARBSS
	.seg	FARBSS
_srfsFileName::
	.ds	144
	.debug	G _srfsFileName 101 8
	.cseg
__srfsRead_:
	pop	iy
	lcall	$$_srfsRead_
__srfsWrite_:
	pop	iy
	lcall	$$_srfsWrite_
__srfsClose_:
	pop	iy
	lcall	$$_srfsClose_
__srfsLseek_:
	pop	iy
	lcall	$$_srfsLseek_
__srfsFstat_:
	pop	iy
	lcall	$$_srfsFstat_
__srfsioctl_:
	pop	iy
	lcall	$$_srfsioctl_
__srfsDescr_:
	pop	iy
	lcall	$$_srfsDescr_
__srfsGetQueues_:
	pop	iy
	lcall	$$_srfsGetQueues_
__srfsTick_:
	pop	iy
	lcall	$$_srfsTick_
__srfsEntry_:
	pop	iy
	lcall	$$_srfsEntry_
__srfsOpen_:
	pop	iy
	lcall	$$_srfsOpen_
__srfsUnlink_:
	pop	iy
	lcall	$$_srfsUnlink_
__srfsRename_:
	pop	iy
	lcall	$$_srfsRename_
	.bseg
_srfsFile:
	.ds	54
	.debug	F _srfsFile 103 0
_srfsIPAddr::
	.ds	4
	.debug	G _srfsIPAddr 9 0
_srfsAltAddr::
	.ds	4
	.debug	G _srfsAltAddr 9 0
_srfsSem::
	.ds	22
	.debug	G _srfsSem 104 0
_sockFDEV::
	.ds	4
	.debug	G _sockFDEV 40 0
_lastSendTime::
	.ds	4
	.debug	G _lastSendTime 9 0
_srfsConOK::
	.ds	1
	.debug	G _srfsConOK 3 0
_srvAddr::
	.ds	14
	.debug	G _srvAddr 105 0
	.debug	T	64000320001000080001006A0000000572656164006A0002000677
	.debug	T	72697465006B00040006636C6F7365006C000600066C736565
	.debug	T	6B006D000800066673746174006E000A0006696F63746C006F
	.debug	T	000C000664657363720070000E000A67657451756575657300
	.debug	T	57000321000E00070001002C000000056E616D6500710002000869
	.debug	T	6F46756E63730072000400057469636B007300060006656E74
	.debug	T	72790074000800056F70656E0075000A0007756E6C696E6B00
	.debug	T	76000C000772656E616D6500
	.debug	T	070002220000000100
	.debug	T	0B000223000100030024000000
	.debug	T	070000240002000100
	.debug	T	0B000225000100060006000000
	.debug	T	2A00032600080002000974696D657370656300090000000774765F
	.debug	T	73656300080004000874765F6E73656300
	.debug	T	0B000227000100280001000000
	.debug	T	070008280004002900
	.debug	T	0B01032900270013000100060000000A6F70656E436F756E740071
	.debug	T	00020008696F46756E6373002400040006696F446576003D00
	.debug	T	06000B776169745175657565730006000A0006666C61677300
	.debug	T	06000C00056D6F64650009000E000A72656164436F756E7400
	.debug	T	060012000D7264496E50726F67726573730006001400077264
	.debug	T	53746174000600160006726445727200030018000D646F6E74
	.debug	T	43616C6C5265616400090019000B7772697465436F756E7400
	.debug	T	06001D000D7772496E50726F67726573730006001F00077772
	.debug	T	53746174000600210006777245727200030023000E646F6E74
	.debug	T	43616C6C5772697465000300240004656F6600030025000669
	.debug	T	7374747900030026000773686172656400
	.debug	T	0900012A005A0003005A00
	.debug	T	0B00022B000100010001000000
	.debug	T	0700002C0002000300
	.debug	T	0700002D0002002E00
	.debug	T	3500032E000900040001000300000006696E557365000600010007
	.debug	T	6572724E756D00060003000672656D66640064000500056E61
	.debug	T	6D6500
	.debug	T	1700022F00040006002C000000060002002C00040006000600
	.debug	T	1B00023000050006003100000031000200310004002C0006000600
	.debug	T	0800
	.debug	T	070000310002000600
	.debug	T	0900013200060003000600
	.debug	T	1300023300030006002C0000000600020006000400
	.debug	T	0900013400100003001000
	.debug	T	0900013500040003000400
	.debug	T	0F00023600020024000700000037000200
	.debug	T	070000370002003800
	.debug	T	2400033800030002000100060000000A6F70656E436F756E740003
	.debug	T	00020007756E7573656400
	.debug	T	0B00023900010006002C000000
	.debug	T	0F00023A00020006002C0000002C000200
	.debug	T	0F00023B0002000600240000003C000200
	.debug	T	0700083C0004003D00
	.debug	T	2800033D00040002000100770000000A7265616451756575650077
	.debug	T	0002000B7772697465517565756500
	.debug	T	0B00023E000100060024000000
	.debug	T	1700023F0004000600240000002C0002000600040003000600
	.debug	T	130002400003000900240000000900020006000600
	.debug	T	0F00024100020006002400000042000200
	.debug	T	070000420002004300
	.debug	T	BB000343002A000D00057374617400090000000773745F64657600
	.debug	T	090004000773745F696E6F00090008000873745F6D6F646500
	.debug	T	06000C000973745F6E6C696E6B0006000E000773745F756964
	.debug	T	00060010000773745F67696400090012000873745F72646576
	.debug	T	00090016000873745F73697A650006001A000B73745F626C6B
	.debug	T	73697A650006001C000A73745F626C6F636B730009001E0009
	.debug	T	73745F6174696D6500090022000973745F6D74696D65000900
	.debug	T	26000973745F6374696D6500
	.debug	T	09000144002A0003002A00
	.debug	T	130002450003000600240000000600020024000400
	.debug	T	130002460003002C00240000002C00020006000400
	.debug	T	0B000247000100090079000000
	.debug	T	130002480003002400240000000600020007000400
	.debug	T	0F0002490002002C002C0000002C000200
	.debug	T	1300024A00030006002C0000002C00020007000400
	.debug	T	0F00024B00020001006400000064000400
	.debug	T	0F00024C00020001007A00000004000200
	.debug	T	0F00024D00020001007A00000008000200
	.debug	T	0B00024E00010004007A000000
	.debug	T	0B00024F00010008007A000000
	.debug	T	130002500003000600060000002C00020024000400
	.debug	T	070000510002005200
	.debug	T	7702035200E8002D000C7461736B5F737472756374007B00000006
	.debug	T	737461636B007B0002000C737461636B5F7374617274000700
	.debug	T	04000B737461636B5F73697A65000300060005737365670003
	.debug	T	0007000773746B6C6C7200030008000773746B686C72000300
	.debug	T	09000677707361720007000A0008777073616C68720003000C
	.debug	T	000677707362720007000D0008777073626C6872007C000F00
	.debug	T	064644536574000300110009757365436F756E740003001200
	.debug	T	08636C61696D65640007001300067374617465005100150007
	.debug	T	6D794164647200030017000A7A65726F416674657200060018
	.debug	T	000561726763007A001A000561726776007E001C0005617267
	.debug	T	730078009C0007776169745173000900A0000C7369675F7065
	.debug	T	6E64696E67000900A4000C7369675F626C6F636B6564000600
	.debug	T	A8000E707265656D70745F636F756E74000300AA0006696E55
	.debug	T	7365000300AB000565786974000300AC000A6C6F675374646F
	.debug	T	7574000300AD000B7265646972537464696F000900AE000A64
	.debug	T	696167466C616773007F00B2000970726F634D61696E000600
	.debug	T	B40004706964000600B6000570706964000300B80009707269
	.debug	T	6F72697479000600B900086C617374457272000900BB000873
	.debug	T	746172746564000900BF000C6C617374526573756D65640009
	.debug	T	00C3000C6C61737453746F70706564000900C7000C6C617374
	.debug	T	52756E6E696E67000900CB000D726573756D65645469636B73
	.debug	T	000900CF000A7469636B7355736564000900D300076D735573
	.debug	T	6564000300D700096F7574436F756E74006400D800076F7574
	.debug	T	507472000900DC000B736C6565705469636B73000900E0000D
	.debug	T	736C65657053746172746564000900E4000D74696D656F7574
	.debug	T	5469636B7300
	.debug	T	13000253000300060081000000640002000D000000
	.debug	T	13000254000300060064000000640004000D000000
	.debug	T	0900015500080009000200
	.debug	T	0B000256000100060082000000
	.debug	T	0B000257000100030028000000
	.debug	T	130002580003000600060000007100020024000400
	.debug	T	0B000259000100280006000000
	.debug	T	1700025A000400060028000000240004000600060008000800
	.debug	T	5400035B000C000600010083000000065F5F6F7574006B00020005
	.debug	T	5F5F696E0085000400085F5F77726974650085000600075F5F
	.debug	T	726561640006000800085F5F756E6765740024000A000B5F5F
	.debug	T	757365724461746100
	.debug	T	1300025C0003000600060000008700020006000400
	.debug	T	1300025D0003000600060000000600020006000400
	.debug	T	0F00025E00020006008800000009000200
	.debug	T	1300025F0003000600880000000600020007000400
	.debug	T	0F00026000020006008800000089000200
	.debug	T	0B000261000100060088000000
	.debug	T	0B000262000100010064000000
	.debug	T	0B000263000100640006000000
	.debug	T	070008640004000300
	.debug	T	0900016500900066000600
	.debug	T	0900016600180003001800
	.debug	T	090001670036002E000600
	.debug	T	680003680016000800010064000000056E616D650078000400076F
	.debug	T	70656E65640078000800076C6F636B65640078000C0008626C
	.debug	T	6F636B6564000300100009756E6C696E6B6564000300110006
	.debug	T	696E55736500060012000676616C75650024001400076D7941
	.debug	T	64647200
	.debug	T	41000369000E00040009736F636B61646472000500000007735F74
	.debug	T	797065000500020007735F706F7274000900040005735F6970
	.debug	T	003200080009735F73706172657300
	.debug	T	0700006A0002003F00
	.debug	T	0700006B0002003E00
	.debug	T	0700006C0002004000
	.debug	T	0700006D0002004100
	.debug	T	0700006E0002004500
	.debug	T	0700006F0002004600
	.debug	T	070000700002003B00
	.debug	T	070000710002002000
	.debug	T	070000720002002B00
	.debug	T	070000730002003600
	.debug	T	070000740002003300
	.debug	T	070000750002003900
	.debug	T	070000760002003A00
	.debug	T	070000770002007800
	.debug	T	280003780004000100125F5F776169745F71756575655F68656164
	.debug	T	008A000000096C6973744865616400
	.debug	T	070000790002000900
	.debug	T	0700007A0002002C00
	.debug	T	0700007B0002000700
	.debug	T	0700007C0002007D00
	.debug	T	0700087D0004000100
	.debug	T	0900017E00800003008000
	.debug	T	0700007F0002008000
	.debug	T	0F0002800002000100060000007A000200
	.debug	T	070000810002005B00
	.debug	T	070000820002002100
	.debug	T	070000830002008400
	.debug	T	0F00028400020006000300000024000200
	.debug	T	070000850002008600
	.debug	T	130002860003000800640000000800040024000800
	.debug	T	070000870002006900
	.debug	T	070000880002006800
	.debug	T	070000890002002600
	.debug	T	2600038A00040002000A6C6973745F68656164008B000000056E65
	.debug	T	7874008B000200057072657600
	.debug	T	0700008B0002008A00
	.seg	CONST
@1:
	.db	"srfs",0,"192.168.4.34",0,"192.168.4.33",0,"/srfs/",0,"bad bad",0
	.defseg	FARCONST
	.seg	FARCONST
@8:
	.db	"srfsInit_",0,10,"*** Unable to get SRFS lock: %ls ***",10,0,"  srfsConnect(): socket() error: %ls"
	.db	10,0,"  srfsConnect(): connect() error: [%d] %ls",10,0,13,10,"*** Connected to SRFS server ***",13,10,"  %"
	.db	".128s",13,10,10,0,13,10,"*** Closing con to SRFS server ***",13,10,"  %.128s",13,10,10,0,"*** Con to SRFS server fai"
	.db	"led ***",10,0,"*** SRFS: sendCmd(%u): %ls ***",10,0,"*** SRFS: readResp(): %ls",10,0,"*** SRFS: readRes"
	.db	"p(): resp too large",13,10,0,"_srfsRename_(): %ls",10,0,"srfs 0x%04lX ",0,"srfs[%d] %ls",0
	.end
