//----- anares.c ----- 06/03/09 -----(05/29/09)-----

//-----------------------------------------------------------------------------
//
//  2009-06-03 - Mark Davis
//               - All resolutions values are now shown as positive values
//
//  2009-05-29 - Mark Davis
//               - Original version of command to show the resolution of
//                 the analog readings and settings based on current
//                 calibration values.
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <applic.h>

#include <sys/farprintf.h>


//-----------------------------------------------------------------------------
//  show resolution of analog readings based on current calibration settings
//-----------------------------------------------------------------------------
void _task _anares(int argc, char **argv)
{
  uint  u, uidx;
  float  res;
  char  temp[128];


  if (showHelp(1, argc, argv))  { printf("\nusage: readres\r\n");  Exit(-1); }


  printf("\r\n Resolution of analog Readings:\r\n"
 "      --- Res ---   ------- Range -------   ------Name------\r\n");
//####:  -##.##E-##   -##.##E-## -##.##E-##   ################

  for (u=0; u<NumApplAIN; ++u)  {
    uidx = logicalOrderAIN(u);
    if (u >= NumHdwADC)
      printf("%4u:  -- n/a --   -- n/a --  -- n/a --   %.24s\r\n",
             uidx, ainName(uidx, temp) );
    else  {
      res = adcGain[uidx] * readGain[uidx];  res = FABS(res);
      printf("%4u:  %s   %s %s   %.24s\r\n",
              uidx,
              Engineering(res, 4, temp),
              Engineering(ReadMin(uidx), 4, temp+32),
              Engineering(ReadMax(uidx), 4, temp+48),
              ainName(uidx, temp+60)
            );
    }
  }


  printf("\r\n Resolution of analog Settings:\r\n"
         "      --- Res ---   ------- Range -------   ------Name------\r\n");
//####:  -##.####E-##      -##.##E-## -##.##E-##  ################

  for (u=0; u<NumApplAOUT; ++u)  {
    uidx = logicalOrderAOUT(u);
    if (u >= NumHdwDAC)
      printf("%4u:  --- n/a ---        -- n/a --  -- n/a --  %.24s\r\n",
             uidx, aoutName(uidx, temp) );
    else  {
      res = dacGain[uidx] * setGain[uidx];  res = FABS(res);
      printf("%4u:  %s   %s %s   %.24s\r\n",
              uidx,
              Engineering(res, 4, temp),
              Engineering(SetMin(uidx), 4, temp+32),
              Engineering(SetMax(uidx), 4, temp+48),
              aoutName(uidx, temp+60)
            );
    }
  }


  Exit(0);
}

//-----------------------------------------------------------------------------
