//----- dig.c ----- 06/21/09 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <applic.h>

#include <sys/farprintf.h>


//-----------------------------------------------------------------------------
//  commands to manipulate the digital outputs for testing
//-----------------------------------------------------------------------------
void _task _dig(int argc, char **argv)
{
  char  dir;
  uint  idx, numBits, numLabels;
  int  key;
  ulong  mask;
  bool  turnOn, wait, validCmd;
  ulong  *bits, *forcedMask, *forcedVals;
  char far  *labels;
  char  desc[64];


#define  DLABEL(idx)  \
  ((idx < numLabels) ? labels + (idx)*MAX_LABEL_LEN : (char far *)"")

  validCmd = false;

//===== process cmd for digital I/O bits =====
  if (StriEq(argv[1], "out"))  {
    bits = &digitalOut;  numBits = MaxDOUT;
    forcedMask = &forcedDigOutMask;  forcedVals = &forcedDigOutBits;
    labels = doutLabel[0];  numLabels = NumApplDOUT;  dir = 'O';
    validCmd = true;
  }  else
  if (StriEq(argv[1], "in"))  {
    bits = &digitalIn;  numBits = MaxDIN;
    forcedMask = &forcedDigInMask;  forcedVals = &forcedDigInBits;
    labels = dinLabel[0];  numLabels = NumApplDIN;  dir = 'I';
    validCmd = true;
  }
  if (StriEq(argv[1], "extra"))  {
    bits = &extraIn;  numBits = EXTRA_IN_BITS;
    forcedMask = &forcedExtraInMask;  forcedVals = &forcedExtraInBits;
    labels = extraInLabel[0];  numLabels = NumExtraIn;  dir = 'X';
    validCmd = true;
  }

  if (!validCmd OR showHelp(3, argc, argv))  {
    printf("\nusage: dig in|out|extra <cycle [wait]>|<#|all 0|1>\r\n");
    Exit(-1);
  }


//--- cycle through all bits, turning each one on for a short time ---
  if (stricmp(argv[2], "cycle") == 0)  {
    wait = ((argc == 4) AND (stricmp(argv[3], "wait") == 0));
    for (;;)  {
      for (idx=0; idx<numBits; ++idx)  {
        mask = 1L << idx;
        _EnterCritical_();
        *forcedMask = 0xffffffffL;  *bits = *forcedVals = mask;
        _ExitCritical_();
        printf("D%c-%02d (%ls)%s\r", dir, idx, DLABEL(idx), cllCmd);
        fflush(stdout);  CheckETX();  key = CheckETX();
        if (wait)
          while (key == -1)  { sleepMS(100);  key = CheckETX(); }
        else
          sleepMS(500);
      }
    }
  }

//--- change settings for a single or all bits ---
  if (StriEq(argv[2], "all"))  {
    mask = 0xffffffffL;  sprintf(desc, "All dig %s's", argv[1]);  }
  else  {
    if ((uint)(idx = atoi(argv[2])) >= 32)  Exit(-1);
    mask = 1L << idx;
    sprintf(desc, "D%c-%02d (%ls)", dir, idx, DLABEL(idx));
  }

//--- if no state specified: turn off forced bit for specified output ---
  if (argc <= 3)  {
    _EnterCritical_();
    *forcedMask &= ~mask;  *forcedVals &= ~mask;
    _ExitCritical_();
    printf("%s no longer forced\r\n", desc);
    Exit(0);
  }

//--- otherwise force specified output to the given state ---
  turnOn = (atoi(argv[3]) ? 1 : 0);
  _EnterCritical_();
  if (turnOn)  {
    *bits |= mask;  *forcedVals |= mask;  }
  else  {
    *bits &= ~mask;  *forcedVals &= ~mask;  }
  *forcedMask |= mask;
  _ExitCritical_();
  printf("%s forced to %d\r\n", desc, turnOn);


  Exit(0);
}

//-----------------------------------------------------------------------------
