//----- ios.c ----- 09/26/12 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//
//  2012-09-26 - Mark Davis
//               - The digital out bits now show any forced output states
//                 (again).  I believe what happened was that originally I
//                 set the global digitalOut to conform to the forced states,
//                 but that would mess up the application logic so I used
//                 the forced states only when actually setting the output
//                 signals (but forgot to re-enable the code here that shows
//                 the actual output states).
//                 NOTE that the bits of the global digitalIn ALWAYS includes
//                 the forced states (since not doing so would mean the code
//                 would not see the forced states, which would mean forcing
//                 an input would have no effect at all!)
//
//  2010-02-10 - Mark Davis
//               - Added option to show noise level for analog inputs
//                 (final tweaks on 02/15/2010)
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>

#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>
#include <math.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <tsh.h>

#include <applic.h>

#include <sys/farprintf.h>


//-----------------------------------------------------------------------------
//  show I/O status (analog and digital inputs and outputs)
//-----------------------------------------------------------------------------
void _task _ios(int argc, char **argv)
{
  uchar  maxLines, cmd, mode, stateCh, forcedFlag;
  bool  softBit, forcedBit, showNoise;
  uint  ui, uidx, x, rdTicks, wrTicks, ttl, diff;
  ulong  mask, digIn, digOut, lastMinMaxReset;
  float  bits,res, noise;
  char  temp[128];



  if (showHelp(1, argc, argv))  {
    printf("\nusage:  %s\r\n", argv[0]);
    printf("\n   press:  'S' for std display\r\n"
             "           'A' for Analog w/labels\r\n"
             "           'D' for Digital w/labels\r\n"
             "           'I' for Inputs w/labels\r\n"
             "           'N' for noise levels on Analog Inputs\r\n"
             "           'O' for Outputs w/labels\r\n"
          );
    Exit(-1);
  }

  printf(clsCmd);  mode = 'S';  cmd = 0;


  for (;;)  {
    if (mode == 'S')  {
      maxLines = NumHdwADC;
      if (NumHdwDAC > maxLines)  maxLines = NumHdwDAC;
      if (MaxDIN > maxLines)  maxLines = MaxDIN;
      if (MaxDOUT > maxLines)  maxLines = MaxDOUT;
    }
    else  {
      maxLines = NumApplAIN;
      if (NumApplAOUT > maxLines)  maxLines = NumApplAOUT;
      if (NumApplDIN > maxLines)  maxLines = NumApplDIN;
      if (NumApplDOUT > maxLines)  maxLines = NumApplDOUT;
    }

    cmd = toupper(cmd);
    if ((cmd == 'A') OR (cmd == 'D') OR (cmd == 'I') OR (cmd == 'N')
       OR (cmd == 'O') OR (cmd == 'S'))  {
//       for (ui=0; ui<NumHdwADC; ++ui)  resetRawMinMax(ui);
       lastMinMaxReset = GetMS();
       printf(clsCmd);  mode = cmd;
     }

    stdHeader();

    _EnterCritical_();
//    digIn = (digitalIn & ~forcedDigInMask) | forcedDigInBits;
    digIn = digitalIn;
    digOut = (digitalOut & ~forcedDigOutMask) | forcedDigOutBits;
    _ExitCritical_();

    _EnterCritical_();
    rdTicks = readTicks;  wrTicks = writeTicks;  ttl = rwTicks;
    _ExitCritical_();

#define  MSperTick  0.030517578125  // 32768 ticks/sec

    printf("ADC: %sms  DAC: %sms  RW+C: %sms  ",
           fmtFloat((float)rdTicks * MSperTick, 4,2, temp+40),
           fmtFloat((float)wrTicks * MSperTick, 4,2, temp+50),
           fmtFloat((float)ttl * MSperTick, 4,2, temp+60) );

    printf("Knob: %4d   ExtraIN: [%04X]  ", qdCount, extraIn);


  //--- Standard:  show all analog/digital signals w/#'s and std order
    if (mode == 'S')  {
      CursorTo(0, 3);   printf("------- ADC -------");
      for (ui=0,mask=1; ui<NumHdwADC; ++ui,mask<<=1)  {
        CursorTo(0, 4+ui);
        if ((mask & ADC_Mask) OR adcForced[ui])
          fmtFloat(anaIn[ui],8,4,temp);
        else
          strcpy(temp, " --OFF--");
        printf("%2d.%s (%04X) %c",
               ui, temp, adcRaw[ui], (adcForced[ui] ? '*' : ' ') );
      }

      CursorTo(23, 3);  printf("------- DAC -------");
      for (ui=0; ui<NumHdwDAC; ++ui)  {
        CursorTo(23, 4+ui);
        printf("%2d.%s (%04X) %c",
               ui, fmtFloat(anaOut[ui],8,4,temp),
               dacRaw[ui], (dacForced[ui] ? '*' : ' ') );
      }

      CursorTo(46, 3);  printf("-- DIN --");
      for (mask=1,ui=0; ui<MaxDIN; ++ui,mask<<=1)  {
        CursorTo(47, 4+ui);
        printf("%2d. %c%c", ui,
          (digIn & mask ? '1':'0'), (forcedDigInMask & mask ? '*':' ') );
      }

      CursorTo(60, 3);  printf("-- DOUT --");
      for (mask=1,ui=0; ui<MaxDOUT; ++ui,mask<<=1)  {
        CursorTo(61, 4+ui);
        printf("%2d. %c%c", ui,
            (digOut & mask ? '1':'0'), (forcedDigOutMask & mask ? '*':' ') );
      }
    }

  //--- show just Inputs/Outputs or Analog/Digital with labels
    else  {
      x = 0;

    //===== Analog Inputs =====
      if ((mode == 'A') OR (mode == 'I') OR (mode == 'N'))  {
        CursorTo(x, 3);  printf("----- Analog (ADC) Input Levels  -----");
        showNoise = false;
        if (mode == 'N')  {
          CursorTo(40, 3);  printf(" -------- Read Noise ---------");
          if (msSince(lastMinMaxReset) >= 3000)  showNoise = true;
        }
        for (ui=0; ui<NumApplAIN; ++ui)  {
          uidx = logicalOrderAIN(ui);
          CursorTo(x, 4+ui);
          if (uidx >= NumHdwADC)
            printf("%2d.  --N/A-- (----)  ", uidx);
          else
            printf("%2d. %s (%04X) %c", uidx,
                   fmtFloat(anaIn[uidx],8,4,temp), adcRaw[uidx],
                   (adcForced[uidx] ? '*' : ' ') );
          printf(" %s (%s)",
                 ainName(uidx, temp+16),  adcUnits(uidx, temp+64) );
        //--- show noise level for analog inputs? ---
          if (showNoise AND (uidx < NumHdwADC))  {
            diff = getNoiseADC(uidx, NULL);
            if (diff == 0)  bits = 0.0;  else
            if (diff == 1)  bits = 0.5;  else
                            bits = fmult(log(diff), 1.442695) + 0.01;
            res = fmult(adcGain[uidx], readGain[uidx]);
            noise = fmult(res, diff);  noise = FABS(noise);
            CursorTo(40, 4+ui);
            printf(" %5u (%s bits)  %s ",
                   diff, fmtFloat(bits, 4,1, temp),
                   Engineering(noise, 4, temp+6) );
          }
        }
        x += 40;
        if (showNoise)  lastMinMaxReset = GetMS();
      }

    //===== Analog Outputs =====
      if ((mode == 'A') OR (mode == 'O'))  {
        CursorTo(x, 3);  printf("----- Analog (DAC) Output Levels -----");
        for (ui=0; ui<NumApplAOUT; ++ui)  {
          uidx = logicalOrderAOUT(ui);
          CursorTo(x, 4+ui);
          if (uidx >= NumHdwDAC)
            printf("%2d.  --N/A-- (----)  ", uidx);
          else
            printf("%2d. %s (%04X) %c", uidx,
              fmtFloat(anaOut[uidx],8,4,temp), dacRaw[uidx],
              (dacForced[uidx] ? '*' : ' ') );
          printf(" %s (%s)",
              aoutName(uidx, temp+16),  dacUnits(uidx, temp+64) );
        }
        x += 40;
      }

    //===== Digital Inputs =====
      if ((mode == 'D') OR (mode == 'I'))  {
        CursorTo(x, 3);  printf("----- Digital Inputs -----");
        for (ui=0; ui<NumApplDIN; ++ui)  {
          uidx = logicalOrderDIN(ui);  mask = (1L << uidx);
          softBit = uidx >= MaxDIN;  forcedBit = BOOL(forcedDigInMask & mask);
          stateCh = (softBit AND !forcedBit ? '-' : (digIn & mask ? '1':'0'));
          forcedFlag = (forcedBit ? '*':' ');
          CursorTo(x, 4+ui);
          printf("%2d. %c%c  %s",
                 uidx, stateCh, forcedFlag, dinName(uidx, temp) );
       }
        x += 40;
      }

    //===== Digital Outputs =====
      if ((mode == 'D') OR (mode == 'O'))  {
        CursorTo(x, 3);  printf("----- Digital Outputs -----");
        for (ui=0; ui<NumApplDOUT; ++ui)  {
          uidx = logicalOrderDOUT(ui);  mask = (1L << uidx);
          stateCh = (uidx >= MaxDOUT ? '-' : (digOut & mask ? '1':'0'));
          forcedFlag = (forcedDigOutMask & mask ? '*':' ');
          CursorTo(x, 4+ui);
          printf("%2d. %c%c  %s",
                 uidx, stateCh, forcedFlag, doutName(uidx, temp) );
        }
        x += 40;
      }

    }

    CursorTo(0, maxLines+4);  fflush(stdout);  cmd = CheckETX();  sleepMS(500);
  }


}

//-----------------------------------------------------------------------------
