//----- netstat.c ----- 01/02/13-----(03/14/06)-----

//-----------------------------------------------------------------------------
//
//  2013-01-02 - Mark Davis
//               - Fixed -s option so it show correct % for retransmitions
//
//  2012-02-21 - Mark Davis
//               - Added -s option to show connection statistics
//
//  2011-01-26 - Mark Davis
//               - Added -r option to show routing table
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <tsh.h>

#include <applic.h>

#include <sys/farprintf.h>

//-----------------------------------------------------------------------------
void tcpShowStats(void)
{
  uint  u, numCons;
  SockInfo  *si;
  tcp_Socket  *tcp;
  ulong  rtbytes, rtsegments, txbytes, txsegments, timeouts, sendsoons,
         dupacks, txetherr;
  char  buf[80];


/*
typdef struct {
  ...
#ifdef TCP_STATS
  rtbytes;      // Total retransmitted bytes
  rtsegments;   // Number of retransmitted segments
  txbytes;      // Total tx bytes (new + retransmit)
  txsegments;   // Total tx segments (new + retransmit)
  timeouts;     // Total real timeouts
  sendsoons;    // Total sendsoon timeouts
  dupacks;      // Number of times dup ack processing triggered
  txetherr;     // Number of ethernet transmit errors
#endif
} tcp_Socket;
*/



  rtbytes = rtsegments = txbytes = txsegments = timeouts = sendsoons =
            dupacks = txetherr = 0;

  printf("\r\n"
         "TimeOuts  EthErrs   TotalXmit   TtlReXmit  ReX%%  lPort  Foreign Address\r\n");
//        ########  #######  ##########  ##########  ###%  #####  ###.###.###.###:#####
//        ########  #######  ##########  ##########  ###%  #####  ###.###.###.###:#####


  numCons = 0;  si=tcpSockets;
  for (u=0; u<MaxTCPSockets; ++u,++si)  {
    if (!si->inUse)  continue;
    if (stricmp(sockState(si), "Established"))  continue;

    tcp = (tcp_Socket *)si->sock;

    fmtAddrPort(buf, RemoteAddr(si), RemotePort(si));

    printf("%8lu  %7lu  %10lu  %10lu  %3u%%  %5u  %s\r\n",
           tcp->timeouts, tcp->txetherr,
           tcp->txbytes, tcp->rtbytes,
           txbytes ? (uint)((float)rtbytes * 100.0 / (float)txbytes) : 0,
           si->ourPort, buf );

    ++numCons;

    rtbytes += tcp->rtbytes;
    rtsegments += tcp->rtsegments;
    txbytes += tcp->txbytes;
    txsegments += tcp->txsegments;
    timeouts += tcp->timeouts;
    sendsoons += tcp->sendsoons;
    dupacks += tcp->dupacks;
    txetherr += tcp->txetherr;
  }

  printf("\n"
         "tcp:\r\n"
         "  %u established connections\r\n"
         "  %10lu segments sent out\r\n"
         "  %10lu segments retransmited\r\n"
         "  %10lu bytes transmitted\r\n"
         "  %10lu bytes retransmitted\r\n"
         "  %10lu real timeouts\r\n"
         "  %10lu sendsoon timeouts\r\n"
         "  %10lu duplicate acks\r\n"
         "  %10lu ethernet xmit errors\r\n",
         numCons, txsegments, rtsegments, txbytes, rtbytes,
         timeouts, sendsoons, dupacks, txetherr);
}


//-----------------------------------------------------------------------------
void _task _netstat(int argc, char **argv)
{
  uint  u;
  SockInfo  *si;
  char  buf[80];


  if (showHelp(1, argc, argv))  {
    printf("\nusage:  %s [-r|-s]\r\n", argv[0]);  Exit(-1); }

  if (argc > 1)  {
    if (StrEq(argv[1], "-r"))  { showRoutes();  Exit(0); }
    if (StrEq(argv[1], "-s"))  { tcpShowStats();  Exit(0); }
  }


  printf("\r\nInternet connections\r\n");
  printf("Proto  Local Address           Foreign Address\r\n");

  for (u=0,si=tcpSockets; u<MaxTCPSockets; ++u,++si)  {
    if (!si->inUse)  continue;
    strcpy(buf, "tcp    ");
    fmtAddrPort(buf+7, si->ourAddr, si->ourPort);  padStr(buf, 31);
    fmtAddrPort(buf+31, RemoteAddr(si), RemotePort(si));  padStr(buf, 53);
    sprintf(buf+53, " %s\r\n", sockState(si));
    printf(buf);
  }

  for (u=0,si=udpSockets; u<MaxUDPSockets; ++u,++si)  {
    if (!si->inUse)  continue;
    strcpy(buf, "udp    ");
    fmtAddrPort(buf+7, si->ourAddr, si->ourPort);  padStr(buf, 31);
    fmtAddrPort(buf+31, RemoteAddr(si), RemotePort(si));  padStr(buf, 53);
    sprintf(buf+53, " %s\r\n", sockState(si));
    printf(buf);
  }

  Exit(0);
}

//-----------------------------------------------------------------------------
