//----- scope.c ----- 12/29/11 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//
//  2011-12-28 - Mark Davis
//               - The scope command now cleanly exits if there is an error
//                 checking for a char on stdin.  Prior to this fix, if the
//                 connection was closed, the blocking read with a timeout
//                 would return immediately and the task would keep running
//                 but would use all available CPU cycles.
//
//  2009-10-26 - Mark Davis
//               - changes to work with register names ref via far ptr
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <sys/modbus.h>

#include <tsh.h>

#include <applic.h>

#include <sys/farprintf.h>

//-----------------------------------------------------------------------------
//  cmd to show changes to list of register values
//-----------------------------------------------------------------------------
void _task _scope(int argc, char **argv)
{
  bool  foundReg;
  int  count;
  uint  a, i, numRegs, numSecs, numSamples, maxSamples;
  ulong  bufSize, capCount;
  RegInfo  *reg, *lastReg, **regList;
  char  ch;
  char  name[64];


  if (showHelp(3, argc, argv))  {
    printf("\nusage: scope #secs regName [regName ...]\r\n");  Exit(-1);  }

  numRegs = 0;
  //chan = 0;  //finish - parse argv[] for subscript (e.g. "regname[2]")

  if ((numSecs = atoi(argv[1])) < 1)  numSecs = 1;

  numSamples = numSecs * 100;

//----- scan lists of registers for matches -----
  lastReg = NULL;  regList = r3Regs;
  bytesPerScopeSample = 8;  // for timestamp value incl with each sample
  for (a=2; (a < argc) AND (numRegs < MaxWatchSize); ++a)  {
    foundReg = false;
    for (regList=r3Regs; ; regList=r4Regs)  {
      for (i=0,reg=regList[0]; reg; ++i,reg=regList[i])  {
        if (reg == lastReg)  continue;
        lastReg = reg;
        strcpyfn(name, reg->name);  if (stricmp(argv[a], name))  continue;
        scopeList[numRegs] = reg;  ++numRegs;
        bytesPerScopeSample += reg->numRegs << 1;
        foundReg = true;
      }
      if (regList == r4Regs)  break;
    }
    if (!foundReg)  {
      printf("No such register: '%s'\r\n", argv[a]);  Exit(-1);  }
  }

  //finish: need a way to tell how much heap is actually free!
  maxSamples = (uint)(_farheapsize / bytesPerScopeSample);

  printf("\r\nBytes per sample: %u,  Max Secs: %u\r\n",
         bytesPerScopeSample, maxSamples/100);

  if (maxSamples < numSamples)  numSamples = maxSamples;

  bufSize = (ulong)bytesPerScopeSample * numSamples;
  scopeBuf = farmalloc(bufSize);

  if (! scopeBuf)  { printf("Unable to allocate a buffer\r\n");  Exit(-1); }

  printf("Allocated %lu bytes at %lu\r\n", bufSize, (ulong)scopeBuf);

  if ((ulong) scopeBuf > 0xfffffL)  {  //tdebug
     printf("Invalid pointer!\r\n");  Exit(-999); }

  numScopeRegs = numRegs;  captureIdx = 0;  scopeBufSampleSize = numSamples;

//--- start the capture process ---
  captureCount = 0;
  _EnterCritical_();  capturePtr = scopeBuf;  _ExitCritical_();


//--- periodically show # of samples captured ---
  printf("\r\n");
  for (;;)  {
    _EnterCritical_();  capCount = captureCount;  _ExitCritical_();
    printf("\r%lu", capCount);  fflush(stdout);
    count = read_(0, &ch, 1, 100);  // block for up to 100ms for a char
    if (count < 0)  if (errno != ETIME)  break;  // unexpected error
    if (count > 0)  if (ch == 0x03)  break;
  }


  _EnterCritical_();  capturePtr = NULL;  _ExitCritical_();

  printf("\r\n");  farfree(scopeBuf);


  Exit(0);
}

//-----------------------------------------------------------------------------
