//----- syslog.c ----- 03/31/10 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//
//  2010-03-31 - Mark Davis
//               - Added -x option to exit when the current end of of the log
//                 is reached (handy for use with commands like grep and more).
//
//  2009-06-16 - Mark Davis
//               - Changed to with work with diagMsg defined as a pointer
//                 rather than as an array.
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <sys/srfs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <sys/modbus.h>

#include <tsh.h>

#include <applic.h>

#include <sys/farprintf.h>


//-----------------------------------------------------------------------------
//  show current contents of the in-memory diagnostic/system log
//-----------------------------------------------------------------------------
void _task _syslog(int argc, char **argv)
{
  uint  fromIdx, toIdx;
  bool  wrapped, continuous = true;


  if (showHelp(1, argc, argv))  {
    printf("\nusage:  %s [-x]\r\n", argv[0]);  Exit(-1); }

  if (argc > 1)  if (StriEq(argv[1], "-x"))  continuous = false;

  _EnterCritical_();
  toIdx = diagLogInfo->toIdx;  wrapped = diagLogInfo->wrapped;
  _ExitCritical_();

  fromIdx = (wrapped ? toIdx + 1 : 0);
  if (fromIdx == DiagLogSize)  fromIdx = 0;

  for (;;)  {
    _EnterCritical_();  toIdx = diagLogInfo->toIdx;  _ExitCritical_();
    while (fromIdx != toIdx)  {
      putchar(diagLogBuf[fromIdx]);
      if (++fromIdx == DiagLogSize)  fromIdx = 0;
    }
    if (!continuous)  Exit(0);
    fflush(stdout);  CheckETX();  sleepMS(200);
  }

}

//-----------------------------------------------------------------------------
