//----- top.c ----- 04/27/12 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//  Update Log (most recent first):
//
//
//  2010-04-27 - Mark Davis
//               - Removed IP value from display.
//
//  2010-05-14 - Mark Davis
//               - Added call to CheckETX() at beginning to flush stdin
//
//  2010-03-05 - Mark Davis
//               - Added ability to change the sort order
//
//  2009-06-03 - Mark Davis
//               - updated to work with new SchedMS
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <sys/srfs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <sys/modbus.h>

#include <tsh.h>

#include <applic.h>

#include <sys/farprintf.h>

//-----------------------------------------------------------------------------
//  show relative CPU usage and state data for running processes
//-----------------------------------------------------------------------------
void _task _top(int argc, char **argv)
{
  int  ch, order;
  bool  pushDown;
  uint  i, j, pix;
  ulong  startTime, curTime, idle, secsUsed, elapsedMS, ticks;
  float  percentUsage;
  task_t  *task;
  char  *ptr;
  uint  used, msUsed[NUM_TASKS], taskIdx[NUM_TASKS];
  ulong  ttlms[NUM_TASKS], ttlmsLast[NUM_TASKS];
  char  msg[90];


  if (showHelp(1, argc, argv))  {
    printf("\nusage:  %s\r\n", argv[0]);  Exit(-1); }

  printf(clsCmd);

//--- initialize the values needed to determine CPU usage ---
  startTime = MS_TIMER;
  for (i=0; i<NUM_TASKS; ++i)  {
    task = tasks + i;
    _EnterCritical_();
    ttlmsLast[i] = (task->inUse ? task->msUsed : 0);
    _ExitCritical_();
  }
  sleepMS(1000);


  order = ' ';  CheckETX();

  for (;;)  {
    stdHeader();  printf("%s", cllCmd);

    _EnterCritical_();  ticks = rwTicks;  _ExitCritical_(); // 1 tick = 1/32768 sec
    printf("RW+C: %2u%% %s\r\n%s\n",
           (uint)(((ulong)ticks * 100000L / SchedMS) >> 15), cllCmd, cllCmd );

  //--- update usage value for each active process ---
    curTime = MS_TIMER;
    for (i=0; i<NUM_TASKS; ++i)  {
      taskIdx[i] = i;  task = tasks + i;
      _EnterCritical_();
      ttlms[i] = (task->inUse ? task->msUsed : 0);
      _ExitCritical_();
    }

  //--- determine # of ms each process used since last pass ---
    elapsedMS = msInterval(startTime, curTime);  startTime = curTime;
    for (i=0; i<NUM_TASKS; ++i)  {
      msUsed[i] = (ttlms[i] ? (uint)(ttlms[i] - ttlmsLast[i]) : 0);
      ttlmsLast[i] = ttlms[i];
    }

  //--- sort the list ---
    for (i=0; i<NUM_TASKS; ++i)
      for (j=0; j<NUM_TASKS-i-1; ++j)  {
        pushDown = false;

        switch (order)  {
          case 'p':  // in PID order
            pushDown = (tasks[taskIdx[j]].pid > tasks[taskIdx[j+1]].pid);
            break;

          case 'c':  // in Command name order
            pushDown =
              ( stricmp(tasks[taskIdx[j]].argv[0],
                        tasks[taskIdx[j+1]].argv[0]) > 0);
            break;

          default:  // % CPU usage
            pushDown = (msUsed[j] < msUsed[j+1]);
        }
        if (!pushDown)  continue;

        used = msUsed[j];  msUsed[j] = msUsed[j+1];    msUsed[j+1] = used;
        pix = taskIdx[j];  taskIdx[j] = taskIdx[j+1];  taskIdx[j+1] = pix;
      }


    printf(
  "  PID USER     PRI  STKAV  STATUS      IDLE   %%CPU        TIME COMMAND%s\r\n",
     cllCmd);
// ##### ######## ###  #####  MMMMMMMM  ####ms  ###.#  ####:##:## ############

/* Example from Linux:
top - 08:52:42 up 45 days, 20:41,  3 users,  load average: 0.00, 0.01, 0.00
Tasks:  76 total,   2 running,  74 sleeping,   0 stopped,   0 zombie
Cpu(s):  1.3% us,  0.7% sy,  0.0% ni, 98.0% id,  0.0% wa,  0.0% hi,  0.0% si
Mem:   1036524k total,   686680k used,   349844k free,   194412k buffers
Swap:  2431792k total,        0k used,  2431792k free,   253492k cached

  PID USER      PR  NI  VIRT  RES  SHR S %CPU %MEM    TIME+  COMMAND
 1142 epics     16   0 60236  10m  16m S  1.3  1.1 694:40.53 QtChan
  978 root      15   0  153m  14m 141m S  0.7  1.4 377:13.21 XFree86
    1 root      16   0  1504  512 1352 S  0.0  0.0   0:05.79 init
    2 root      34  19     0    0    0 S  0.0  0.0   0:00.00 ksoftirqd/0
*/

  //--- display the processes in sorted order ---
    for (i=0; i<NUM_TASKS; ++i)  {
      pix = taskIdx[i];  task = tasks + pix;
      if (!task->inUse)  continue;

      idle = msSince(task->lastRunning);

      secsUsed = task->msUsed / 1000;

      ptr = msg;

//      sprintf(ptr, "%5d root     %3d  %5d  %s  ",
      sprintf(ptr, "%5d root     %3d  %5d  %s%2d",
              task->pid, task->priority, _stackFree_(task->pid),
//              statusDescr(pix) );
                statusDescr(pix), task->preempt_count );
      while (*ptr) ++ptr;

      fmtIdleTime(ptr, idle);  while (*ptr) ++ptr;

      percentUsage = (float) ((ulong)msUsed[i] * 100) / elapsedMS;
      fmtFloat(percentUsage, 5,1, ptr);  while (*ptr) ++ptr;
      strcpy(ptr, "  ");  ptr += 2;

      fmtHMSUsed(ptr, secsUsed);  while (*ptr) ++ptr;

      sprintf(ptr, "%.15s%s\r\n", task->argv[0], cllCmd);

      printf(msg);
    }

    printf(clpCmd);

    fflush(stdout);  sleepMS(2000);

    ch = CheckETX();  if (ch AND (ch != -1))  order = tolower(ch);
  }

}

//-----------------------------------------------------------------------------
