//----- watch.c ----- 04/06/11 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//
//  2011-04-06 - Mark Davis
//               - fixed interpretation of OnePerChan flag
//
//  2009-10-26 - Mark Davis
//               - changes to work with register names ref via far ptr
//
//  2009-06-02 - Mark Davis
//               - Fixed so it works properly with a single register
//               - Added display of "standard" header
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <sys/modbus.h>

#include <tsh.h>

#include <applic.h>

#include <sys/farprintf.h>


//-----------------------------------------------------------------------------
//  monitor the values of one or more Modbus register values
//-----------------------------------------------------------------------------
void _task _watch(int argc, char **argv)
{
  RegInfo  *reg, *lastReg;
  CoilInfo  *coil, *lastCoil;
  uint  i, n, chan;
  bool  foundMatches;
  char  temp[64], name[64];


  if (showHelp(2, argc, argv))  {
    printf("\nusage:  watch regName [regName ...]\r\n");  Exit(-1); }

  lastCoil = NULL;  lastReg = NULL;

  chan = 0;  //finish - parse argv[1] for subscript (e.g. "regname[2]")

  n = 1;

  foundMatches = false;

  printf("%s", clsCmd);

  for (;;)  {
    CursorTo(0,0);  stdHeader();  printf("\r\n");

  //----- coils (0xxxx) -----
    for (i=0,coil=c0Coils[0]; coil; ++i,coil=c0Coils[i])  {
      if (coil == lastCoil)  continue;
      strcpyfn(name, coil->name);
      for (n=1; n<argc; ++n)  {
        if (!strstri(name, argv[n]))  continue;
        printf("%05u %-16s", coil->coilNum+1, name);
        for (chan=0; chan<NumC0Chans; ++chan)  {
          printf("  %d", *coil->coilVal);
          if (!(coil->flags & OnePerChan))  break;
        }
        printf("\r\n");  lastCoil = coil;  foundMatches = true;
      }
    }

  //----- read-only registers (3xxxx) -----
    for (i=0,reg=r3Regs[0]; reg; ++i,reg=r3Regs[i])  {
      if (reg == lastReg)  continue;
      strcpyfn(name, reg->name);
      for (n=1; n<argc; ++n)  {
        if (!strstri(name, argv[n]))  continue;
        printf("3%04u#%s %-16s ", reg->regNum+1, typeDesc[reg->dataType], name);
        for (chan=0; chan<NumR3Chans; ++chan)  {
          if (chan)  printf("\r\n                           ");
          printf("  %s", fmtRegVal(reg, NULL, chan, temp));
          if (!(reg->flags & OnePerChan))  break;
        }
        printf("\r\n");  lastReg = reg;  foundMatches = true;
      }
    }

  //----- R/W registers (4xxxx) -----
    for (i=0,reg=r4Regs[0]; reg; ++i,reg=r4Regs[i])  {
      if (reg == lastReg)  continue;
      strcpyfn(name, reg->name);
      for (n=1; n<argc; ++n)  {
        if (!strstri(name, argv[n]))  continue;
        printf("4%04u#%s %-16s ", reg->regNum+1, typeDesc[reg->dataType], name);
        for (chan=0; chan<NumR4Chans; ++chan)  {
          if (chan)  printf("\r\n                           ");
          printf("  %s", fmtRegVal(reg, NULL, chan, temp));
          if (!(reg->flags & OnePerChan))  break;
        }
        printf("\r\n");  lastReg = reg;  foundMatches = true;
      }
    }

    if (! foundMatches)  { printf("No matching registers\r\n");  Exit(-1); }

    lastCoil = NULL;  lastReg = NULL;

    while (CheckETX() != -1);  sleepMS(500);
  }

}


//-----------------------------------------------------------------------------
